@@ -38,6 +38,8 @@
 (define (nohelp) 
   (_ "No help available.") )
 
+(define (nathelp)
+  (_ "You can use 'make check' in src/transports/upnp/ to find out if your NAT supports UPnP.  You should disable this option if you are sure that you are not behind a NAT.  If your NAT box does not support UPnP, having this on will not do much harm (only cost a small amount of resources).") )
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -163,7 +165,7 @@ However, active testing and qualified feedback of these features is always welco
   "HELLOEXPIRES"
   (_ "How many minutes should peer advertisements last?")
   (_ 
-"How many minutes is the current IP valid?  (GNUnet will sign HELLO messages with this expiration timeline. If you are on dialup, 60 (for 1 hour) is suggested. If you are having a static IP address, you may want to set this to a large value (say 14400).  The default is 1440 (1 day). If your IP changes periodically, you will want to choose the expiration to be smaller than the frequency with which your IP changes." )
+"How many minutes is the current IP valid?  (GNUnet will sign HELLO messages with this expiration timeline. If you are on dialup, 60 (for 1 hour) is suggested. If you have a static IP address, you may want to set this to a large value (say 14400).  The default is 1440 (1 day). If your IP changes periodically, you will want to choose an expiry period smaller than the frequency with which your IP changes." )
   '()
   #t
   1440
@@ -185,7 +187,7 @@ If you want to setup an alternate hostlist server, you must run a permanent node
 If you do not specify a HOSTLISTURL, you must copy valid hostkeys to data/hosts manually.")
   '()
   #t
-  "http://gnunet.org/hostlist.php http://gnunet.mine.nu:8081/hostlist http://de.gnunet.org/cgi-bin/hostlist.cgi"
+  "http://gnunet.org/hostlist.php http://gnunet.mine.nu:8081/hostlist http://vserver1236.vserver-on.de/hostlist-074"
   '()
   'always) )
 
@@ -212,7 +214,7 @@ If you do not specify a HOSTLISTURL, you must copy valid hostkeys to data/hosts
 "Unless you want to share the directory directly using a webserver, the default is most likely just fine." )
   '()
   #t
-  "/var/lib/gnunet/data/hosts/"
+  "$GNUNETD_HOME/data/hosts/"
   '()
   'rare) )
 
@@ -347,13 +349,13 @@ If you do not specify a HOSTLISTURL, you must copy valid hostkeys to data/hosts
   "TRANSPORTS"
   (_ "Which transport mechanisms should GNUnet use?")
   (_ 
-"Use space-separated list of the modules, e.g.  \"udp smtp tcp\".  The available transports are udp, tcp, http, smtp, tcp6, udp6 and nat.
+"Use a space-separated list of modules, e.g.  \"udp smtp tcp\".  The available transports are udp, tcp, http, smtp and nat.
 		
-Loading the 'nat' and 'tcp' modules is required for peers behind NAT boxes that cannot directly be reached from the outside.  Peers that are NOT behind a NAT box and that want to *allow* peers that ARE behind a NAT box to connect must ALSO load the 'nat' module.  Note that the actual transfer will always be via tcp initiated by the peer behind the NAT box.  The nat transport requires the use of tcp, http, smtp and/or tcp6 in addition to nat itself.")
+Loading the 'nat' and 'tcp' modules is required for peers behind NAT boxes that cannot directly be reached from the outside.  Peers that are NOT behind a NAT box and that want to *allow* peers that ARE behind a NAT box to connect must ALSO load the 'nat' module.  Note that the actual transfer will always be via tcp initiated by the peer behind the NAT box.  The nat transport requires the use of tcp, http and/or smtp in addition to nat itself.")
   '()
   #t
   "udp tcp http nat"
-  (list "MC" "udp" "udp6" "tcp" "tcp6" "nat" "http" "smtp")
+  (list "MC" "udp" "tcp" "nat" "http" "smtp")
   'always) )
  
 
@@ -375,6 +377,8 @@ traffic: keeps track of how many messages were recently received and transmitted
 
 fs: needed for anonymous file sharing. You should always load this module.
 
+hostlist: integrated hostlist HTTP server.  Useful if you want to offer a hostlist and running Apache would be overkill.
+
 chat: broadcast chat (demo-application, ALPHA quality).	Required for gnunet-chat.  Note that the current implementation of chat is not considered to be secure.
 
 tbench: benchmark transport performance.  Required for gnunet-tbench.  Note that tbench allows other users to abuse your resources.
@@ -383,7 +387,7 @@ tracekit: topology visualization toolkit.  Required for gnunet-tracekit. Note th
   '()
   #t
   "advertising getoption fs stats traffic"
-  (list "MC" "advertising" "getoption" "fs" "stats" "traffic" "dht" "tracekit" "tbench" "vpn" "chat")
+  (list "MC" "advertising" "getoption" "fs" "hostlist" "stats" "traffic" "dht" "tracekit" "tbench" "vpn" "chat")
   'always) )
  
 
@@ -393,7 +397,7 @@ tracekit: topology visualization toolkit.  Required for gnunet-tracekit. Note th
  "TCPSERVER"
  "DISABLE"
  (_ "Disable client-server connections")
- (_ "This option can be used to tell gnunetd not to open the client port.  When run like this, gnunetd will participate as a peer in the network but not support any user interfaces.  This maybe useful for headless systems that are never expected to have end-user interactions.  Note that this will also prevent you from running diagnostic tools like gnunet-stats!")
+ (_ "This option can be used to tell gnunetd not to open the client port.  When run like this, gnunetd will participate as a peer in the network but not support any user interfaces.  This may be useful for headless systems that are never expected to have end-user interactions.  Note that this will also prevent you from running diagnostic tools like gnunet-stats!")
  '()
  #t
  #f
@@ -401,6 +405,19 @@ tracekit: topology visualization toolkit.  Required for gnunet-tracekit. Note th
  'rare) )
 
 
+(define (gnunetd-disable-ipv6 builder)
+ (builder
+ "GNUNETD"
+ "DISABLE-IPV6"
+ (_ "YES disables IPv6 support, NO enables IPv6 support")
+ (_ "This option may be useful on peers where the kernel does not support IPv6.  You might also want to set this option if you do not have an IPv6 network connection.")
+ '()
+ #t
+ #t
+ #t
+ 'advanced) )
+
+
 (define (gnunetd-private-network builder)
  (builder
  "GNUNETD"
@@ -442,7 +459,7 @@ tracekit: topology visualization toolkit.  Required for gnunet-tracekit. Note th
  "NETWORK"
  "HELLOEXCHANGE"
  (_ "Enable advertising of other peers by this peer")
- (_ "This option maybe useful during testing, but turning it off is dangerous! If in any doubt, set it to YES (which is the default).")
+ (_ "This option may be useful during testing, but turning it off is dangerous! If in any doubt, set it to YES (which is the default).")
  '()
  #t
  #t
@@ -461,11 +478,23 @@ tracekit: topology visualization toolkit.  Required for gnunet-tracekit. Note th
  (cons 1 65535)
  'advanced) )
 
+(define (hostlist-port builder)
+ (builder
+ "HOSTLIST"
+ "PORT"
+ (_ "Port for the integrated hostlist HTTP server")
+ (nohelp)
+ '()
+ #t
+ 8080
+ (cons 1 65535)
+ 'hostlist-loaded) )
+
 (define (network-trusted builder)
  (builder
  "NETWORK"
  "TRUSTED"
- (_ "IPs allowed to use gnunetd server")
+ (_ "IPv4 networks allowed to use gnunetd server")
  (_ "This option specifies which hosts are trusted enough to connect as clients (to the TCP port).  This is useful if you run gnunetd on one host of your network and want to allow all other hosts to use this node as their server.  By default, this is set to 'loopback only'.  The format is IP/NETMASK where the IP is specified in dotted-decimal and the netmask either in CIDR notation (/16) or in dotted decimal (255.255.0.0). Several entries must be separated by a semicolon, spaces are not allowed.")
  '()
  #t
@@ -473,6 +502,18 @@ tracekit: topology visualization toolkit.  Required for gnunet-tracekit. Note th
  '()
  'advanced) )
 
+(define (network-trusted6 builder)
+ (builder
+ "NETWORK"
+ "TRUSTED6"
+ (_ "IPv6 networks allowed to use gnunetd server")
+ (_ "This option specifies which hosts are trusted enough to connect as clients (to the TCP port).  This is useful if you run gnunetd on one host of your network and want to allow all other hosts to use this node as their server.  By default, this is set to 'loopback only'.  The format is IP/NETMASK where the IP is specified in dotted-decimal and the netmask either in CIDR notation (/16) or in dotted decimal (255.255.0.0). Several entries must be separated by a semicolon, spaces are not allowed.")
+ '()
+ #t
+ "::1;"
+ '()
+ 'ipv6) )
+
 
 (define (limit-allow builder)
  (builder
@@ -539,6 +580,7 @@ tracekit: topology visualization toolkit.  Required for gnunet-tracekit. Note th
   (_ "Settings that change the behavior of GNUnet in general")
   (list 
     (network-port builder) 
+    (hostlist-port builder)
     (network-trusted builder) 
     (general-hostlisturl builder)
     (general-hosts builder)
@@ -547,6 +589,7 @@ tracekit: topology visualization toolkit.  Required for gnunet-tracekit. Note th
     (fs-path builder) 
     (index-path builder) 
     (daemon-fdlimit builder) 
+    (gnunetd-disable-ipv6 builder) 
     (general-username builder) 
     (general-groupname builder) 
     (general-pidfile builder) 
@@ -581,7 +624,7 @@ tracekit: topology visualization toolkit.  Required for gnunet-tracekit. Note th
   "PATHS"
   ""
   (_ "Fundamentals")
-  (_ "")
+  ""
   (list 
     (paths-home builder) 
     (general-applications builder) 
@@ -644,7 +687,7 @@ In order to use sqstore_mysql, you must configure the mysql database, which is r
  (builder
  "F2F"
  "MINIMUM"
- (_ "Minimum number of connected friends before this peer is allowed to connect to peers that are not listed as friends")
+ (_ "The minimum number of connected friends before this peer is allowed to connect to peers that are not listed as friends")
  (_ "Note that this option does not guarantee that the peer will be able to connect to the specified number of friends.  Also, if the peer had connected to a sufficient number of friends and then established non-friend connections, some of the friends may drop out of the network, temporarily resulting in having fewer than the specified number of friends connected while being connected to non-friends.  However, it is guaranteed that the peer itself will never choose to drop a friend's connection if this would result in dropping below the specified number of friends (unless that number is higher than the overall connection target).")
  '()
  #t
@@ -753,6 +796,19 @@ Note that if you change the quota, you need to run gnunet-update afterwards.")
   'always))
 
 
+(define (fs-migration-buffer builder)
+ (builder
+  "FS"
+  "MIGRATIONBUFFERSIZE"
+  (_ "Number of entries in the migration buffer")
+  (_ "Each entry uses about 32k of memory.  More entries can reduce disk IO and CPU usage at the expense of having gnunetd use more memory. Very large values may again increase CPU usage.  A value of 0 will prevent your peer from sending unsolicited responses.")
+  '()
+  #t
+  64
+  (cons 0 1048576)
+  'always))
+
+
 (define (fs-gap-tablesize builder)
  (builder
   "GAP"
@@ -849,7 +905,7 @@ The size of the DSTORE QUOTA is specified in MB.")
  "NAT"
  "LIMITED"
  (_ "Is this machine unreachable behind a NAT?")
- (_ "Set to YES if this machine is behind a NAT that limits connections from the outside to the GNUnet port and that cannot be traversed using UPnP.  Note that if you have configured your NAT box to allow direct connections from other machines to the GNUnet ports or if GNUnet can open ports using UPnP, you should set the option to NO.  Set this only to YES if other peers cannot contact you directly.  You can use 'make check' in src/transports/upnp/ to find out if your NAT supports UPnP.  You can also use gnunet-transport-check with the '-p' option in order to determine which setting results in more connections.  Use YES only if you get no connections otherwise.  Set to AUTO to use YES if the local IP is belongs to a private IP network and NO otherwise.")
+ (_ "Set to YES if this machine is behind a NAT that limits connections from the outside to the GNUnet port and that cannot be traversed using UPnP. Note that if you have configured your NAT box to allow direct connections from other machines to the GNUnet ports or if GNUnet can open ports using UPnP, you should set the option to NO. Set this only to YES if other peers cannot contact you directly. You can use 'make check' in src/transports/upnp/ to find out if your NAT supports UPnP. You can also use gnunet-transport-check with the '-p' option in order to determine which setting results in more connections.  Use YES only if you get no connections otherwise. Set to AUTO to use YES if the local IP is belongs to a private IP network and NO otherwise.")
  '()
  #t
  "AUTO"
@@ -873,7 +929,7 @@ The size of the DSTORE QUOTA is specified in MB.")
  "TCP"
  "UPNP"
  (_ "Should we try to determine our external IP using UPnP?")
- (_ "You can use 'make check' in src/transports/upnp/ to find out if your NAT supports UPnP.  You should disable this option if you are sure that you are not behind a NAT.  If your NAT box does not support UPnP, having this on will not do much harm (only cost a small amount of resources).")
+ (nathelp)
  '()
  #t
  #t
@@ -884,7 +940,7 @@ The size of the DSTORE QUOTA is specified in MB.")
  (builder
  "TCP"
  "BLACKLIST"
- (_ "Which IPs are not allowed to connect?")
+ (_ "Which IP(v4)s are not allowed to connect?")
  (nohelp)
  '()
  #t
@@ -896,7 +952,7 @@ The size of the DSTORE QUOTA is specified in MB.")
  (builder
  "TCP"
  "WHITELIST"
- (_ "Which IPs are allowed to connect? Leave empty to use the IP of your primary network interface.")
+ (_ "Which IP(v4)s are allowed to connect? Leave empty to use the IP of your primary network interface.")
  (nohelp)
  '()
  #t
@@ -904,6 +960,31 @@ The size of the DSTORE QUOTA is specified in MB.")
  '()
  'advanced))
 
+(define (tcp6-blacklist builder)
+ (builder
+ "TCP"
+ "BLACKLISTV6"
+ (_ "Which IPv6s are not allowed to connect?")
+ (nohelp)
+ '()
+ #t
+ ""
+ '()
+ 'ipv6))
+
+(define (tcp6-whitelist builder)
+ (builder
+ "TCP"
+ "WHITELISTV6"
+ (_ "Which IPv6s are allowed to connect? Leave empty to allow any IP to connect.")
+ (nohelp)
+ '()
+ #t
+ ""
+ '()
+ 'ipv6))
+
+
 (define (tcp builder)
  (builder
  "TCP"
@@ -939,7 +1020,7 @@ The size of the DSTORE QUOTA is specified in MB.")
  "HTTP"
  "UPNP"
  (_ "Should we try to determine our external IP using UPnP?")
- (_ "You can use 'make check' in src/transports/upnp/ to find out if your NAT supports UPnP. You should disable this option if you are sure that you are not behind a NAT.  If your NAT box does not support UPnP, having this on will not do much harm (only cost a small amount of resources).")
+ (nathelp)
  '()
  #t
  #t
@@ -951,7 +1032,7 @@ The size of the DSTORE QUOTA is specified in MB.")
  "HTTP"
  "ADVERTISED-PORT"
  (_ "Which is the external port of the HTTP transport?")
- (_ "Use this option if your firewall maps say port 80 to your real HTTP port.  This can be useful in making the HTTP messages appear even more legit (without the need for running gnunetd as root due to the use of a priviledged port).")
+ (_ "Use this option if your firewall maps, say, port 80 to your real HTTP port.  This can be useful in making the HTTP messages appear even more legit (without needing to run gnunetd as root due to the use of a privileged port).")
  '()
  #t
  80
@@ -1033,7 +1114,7 @@ The size of the DSTORE QUOTA is specified in MB.")
  (_ "Have a look at contrib/dot-procmailrc for an example .procmailrc file.")
  '()
  #t
- "/var/lib/gnunet/smtp-pipe"
+ "$GNUNETD_HOME/smtp-pipe"
  '()
  'smtp-loaded))
 
@@ -1089,7 +1170,7 @@ The size of the DSTORE QUOTA is specified in MB.")
  "UDP"
  "UPNP"
  (_ "Should we try to determine our external IP using UPnP?")
- (_ "You can use 'make check' in src/transports/upnp/ to find out if your NAT supports UPnP. You should disable this option if you are sure that you are not behind a NAT.  If your NAT box does not support UPnP, having this on will not do much harm (only cost a small amount of resources).")
+ (nathelp)
  '()
  #t
  #t
@@ -1132,6 +1213,30 @@ The size of the DSTORE QUOTA is specified in MB.")
  '()
  'advanced))
 
+(define (udp6-blacklist builder)
+ (builder
+ "UDP"
+ "BLACKLISTV6"
+ (_ "Which IPv6s are not allowed to connect?")
+ (nohelp)
+ '()
+ #t
+ ""
+ '()
+ 'ipv6))
+
+(define (udp6-whitelist builder)
+ (builder
+ "UDP6"
+ "WHITELISTV6"
+ (_ "Which IPv6s are allowed to connect? Leave empty to allow any IP to connect.")
+ (nohelp)
+ '()
+ #t
+ ""
+ '()
+ 'ipv6))
+
 (define (udp builder)
  (builder
  "UDP"
@@ -1144,130 +1249,14 @@ The size of the DSTORE QUOTA is specified in MB.")
    (udp-mtu builder)
    (udp-blacklist builder)
    (udp-whitelist builder)
- )
- #t
- #f
- #f
- 'udp-loaded) )
-
-
-(define (tcp6-port builder)
- (builder
- "TCP6"
- "PORT"
- (_ "Which port should be used by the TCP IPv6 transport?")
- (nohelp)
- '()
- #t
- 2088
- (cons 0 65535)
- 'advanced))
-
-(define (tcp6-blacklist builder)
- (builder
- "TCP6"
- "BLACKLIST"
- (_ "Which IPs are not allowed to connect?")
- (nohelp)
- '()
- #t
- ""
- '()
- 'advanced))
-
-(define (tcp6-whitelist builder)
- (builder
- "TCP6"
- "WHITELIST"
- (_ "Which IPs are allowed to connect? Leave empty to allow any IP to connect.")
- (nohelp)
- '()
- #t
- ""
- '()
- 'advanced))
-
-(define (tcp6 builder)
- (builder
- "TCP6"
- ""
- (_ "TCP6 transport")
- (nohelp)
- (list 
-   (tcp6-port builder)
-   (tcp6-blacklist builder)
-   (tcp6-whitelist builder)
- )
- #t
- #f
- #f
- 'tcp6-loaded) )
-
-
-(define (udp6-port builder)
- (builder
- "UDP6"
- "PORT"
- (_ "Which port should be used by the UDP IPv6 transport?")
- (nohelp)
- '()
- #t
- 2088
- (cons 0 65535)
- 'advanced))
-
-(define (udp6-mtu builder)
- (builder
- "UDP6"
- "MTU"
- (_ "What is the maximum transfer unit for UDP 6?")
- (nohelp)
- '()
- #t
- 1452
- (cons 1200 65500)
- 'rare))
-
-(define (udp6-blacklist builder)
- (builder
- "UDP6"
- "BLACKLIST"
- (_ "Which IPs are not allowed to connect?")
- (nohelp)
- '()
- #t
- ""
- '()
- 'advanced))
-
-(define (udp6-whitelist builder)
- (builder
- "UDP6"
- "WHITELIST"
- (_ "Which IPs are allowed to connect? Leave empty to allow any IP to connect.")
- (nohelp)
- '()
- #t
- ""
- '()
- 'advanced))
-
-(define (udp6 builder)
- (builder
- "UDP6"
- ""
- (_ "UDP6 transport")
- (nohelp)
- (list 
-   (udp6-port builder)
-   (udp6-mtu builder)
    (udp6-blacklist builder)
    (udp6-whitelist builder)
  )
  #t
  #f
  #f
- 'udp6-loaded) )
+ 'udp-loaded) )
+
 
 
 (define (network-interface builder)
@@ -1304,7 +1293,7 @@ The size of the DSTORE QUOTA is specified in MB.")
  #t
  ""
  '()
- 'ip6-loaded) )
+ 'ipv6) )
 
 (define (transports builder)
  (builder
@@ -1317,9 +1306,7 @@ The size of the DSTORE QUOTA is specified in MB.")
     (network-interface builder)
     (network-ip builder)
     (tcp builder)
-    (tcp6 builder)
     (udp builder)
-    (udp6 builder)
     (http builder)
     (smtp builder)
   )
@@ -1359,8 +1346,7 @@ The size of the DSTORE QUOTA is specified in MB.")
  "LOAD"
  "MAXCPULOAD"
  (_ "What is the maximum CPU load (percentage)?")
- (_ 
-"Which CPU load can be tolerated.  Load here always refers to the total system load, that is it includes CPU utilization by other processes.  A value of 50 means that once your 1 minute-load average goes over 50% non-idle, GNUnet will try to reduce CPU consumption until the load goes under the threshold.  Reasonable values are typically between 50 and 100.  Multiprocessors may use values above 100." )
+ (_ "The highest tolerable CPU load. Load here always refers to the total system load, that is it includes CPU utilization by other processes.  A value of 50 means that once your 1 minute-load average goes over 50% non-idle, GNUnet will try to reduce CPU consumption until the load goes under the threshold.  Reasonable values are typically between 50 and 100.  Multiprocessors may use values above 100." )
  '()
  #t
  100
@@ -1373,7 +1359,7 @@ The size of the DSTORE QUOTA is specified in MB.")
  "MAXIOLOAD"
  (_ "What is the maximum IO load (permille)?")
  (_ 
-"Which IO load can be tolerated.  Load here refers to the percentage of CPU cycles wasted waiting for IO for the entire system, that is it includes disk utilization by other processes.  A value of 10 means that once the average number of cycles wasted waiting for IO is more than 10% non-idle, GNUnet will try to reduce IO until the load goes under the threshold.  Reasonable values are typically between 10 and 75." )
+"The highest tolerable IO load.  Load here refers to the percentage of CPU cycles wasted waiting for IO for the entire system, that is it includes disk utilization by other processes.  A value of 10 means that once the average number of cycles wasted waiting for IO is more than 10% non-idle, GNUnet will try to reduce IO until the load goes under the threshold.  Reasonable values are typically between 10 and 75." )
  '()
  #t
  50
@@ -1385,7 +1371,7 @@ The size of the DSTORE QUOTA is specified in MB.")
  "LOAD"
  "HARDCPULIMIT"
  (_ "What is the maximum CPU load (hard limit)?")
- (_ "Which CPU load can be tolerated.  This is the hard limit, so once it is reached, gnunetd will start to massively drop data to reduce the load.  Use with caution.")
+ (_ "The highest tolerable CPU load.  This is the hard limit, so once it is reached, gnunetd will start to massively drop data to reduce the load.  Use with caution.")
  '()
  #t
  0
@@ -1426,7 +1412,7 @@ The size of the DSTORE QUOTA is specified in MB.")
 
 While basic bandwidth limitation is simple and always works, there are some situations where it is not perfect.  Suppose you are running another application which performs a larger download. During that particular time, it would be nice if GNUnet would throttle its bandwidth consumption (automatically) and resume using more bandwidth after the download is complete.  This is obviously advanced magic since GNUnet will have to monitor the behavior of other applications. Another scenario is a monthly cap on bandwidth imposed by your ISP, which you would want to ensure is obeyed.  Here, you may want GNUnet to monitor the traffic from other applications to ensure that the combined long-term traffic is within the pre-set bounds.  Note that you should probably not set the bounds tightly since GNUnet may observe that the bounds are about to be broken but would be unable to stop other applications from continuing to use bandwidth.
 
-If either of these two scenarios applies, set BASICLIMITING to NO. Then set the bandwidth limits to the COMBINED amount of traffic that is accepable for both GNUnet and other applications.  GNUnet will then immediately throttle bandwidth consumption if the short-term average is above the limit, and it will also try to ensure that the long-term average is below the limit.  Note however that using NO can have the effect of GNUnet (almost) ceasing operations after other applications perform high-volume downloads that are beyond the defined limits.  GNUnet would reduce consumption until the long-term limits are again within bounds.
+If either of these two scenarios applies, set BASICLIMITING to NO. Then set the bandwidth limits to the COMBINED amount of traffic that is acceptable for both GNUnet and other applications.  GNUnet will then immediately throttle bandwidth consumption if the short-term average is above the limit, and it will also try to ensure that the long-term average is below the limit.  Note however that using NO can have the effect of GNUnet (almost) ceasing operations after other applications perform high-volume downloads that are beyond the defined limits.  GNUnet would reduce consumption until the long-term limits are again within bounds.
 
 NO only works on platforms where GNUnet can monitor the amount of traffic that the local host puts out on the network.  This is only implemented for Linux and Win32.  In order for the code to work, GNUnet needs to know the specific network interface that is used for the external connection (after all, the amount of traffic on loopback or on the LAN should never be counted since it is irrelevant).")
  '()
@@ -1516,6 +1502,7 @@ NO only works on platforms where GNUnet can monitor the amount of traffic that t
      (rare (get-option ctx "Meta" "RARE"))
      (nobasiclimit (not (get-option ctx "LOAD" "BASICLIMITING")))
      (experimental (get-option ctx "Meta" "EXPERIMENTAL"))
+     (ipv6 (not (get-option ctx "GNUNETD" "DISABLE-IPV6")))
      (f2fr (not (get-option ctx "F2F" "RESTRICT") ) )
      (f2f (or (get-option ctx "F2F" "FRIENDS-ONLY")
               (not (eq? (get-option ctx "F2F" "MINIMUM") 0) ) ) )
@@ -1524,11 +1511,10 @@ NO only works on platforms where GNUnet can monitor the amount of traffic that t
      (http-port-nz (eq? (get-option ctx "HTTP" "PORT") 0) )
      (mysql (string= (get-option ctx "MODULES" "sqstore") "sqstore_mysql") )
      (fs-loaded (list? (member "fs" (string-split (get-option ctx "GNUNETD" "APPLICATIONS") #\  ) ) ) )
+     (hostlist-loaded (list? (member "hostlist" (string-split (get-option ctx "GNUNETD" "APPLICATIONS") #\  ) ) ) )
      (nat-loaded (list? (member "nat" (string-split (get-option ctx "GNUNETD" "TRANSPORTS") #\  ) ) ) )
      (tcp-loaded (list? (member "tcp" (string-split (get-option ctx "GNUNETD" "TRANSPORTS") #\  ) ) ) )
      (udp-loaded (list? (member "udp" (string-split (get-option ctx "GNUNETD" "TRANSPORTS") #\  ) ) ) )
-     (tcp6-loaded (list? (member "tcp6" (string-split (get-option ctx "GNUNETD" "TRANSPORTS") #\  ) ) ) )
-     (udp6-loaded (list? (member "udp6" (string-split (get-option ctx "GNUNETD" "TRANSPORTS") #\  ) ) ) )
      (http-loaded (list? (member "http" (string-split (get-option ctx "GNUNETD" "TRANSPORTS") #\  ) ) ) )
      (smtp-loaded (list? (member "smtp" (string-split (get-option ctx "GNUNETD" "TRANSPORTS") #\  ) ) ) )
    )
@@ -1541,18 +1527,17 @@ NO only works on platforms where GNUnet can monitor the amount of traffic that t
             ((eq? i 'rare)         (change-visible ctx a b (and advanced rare)))
             ((eq? i 'experimental) (change-visible ctx a b (and advanced experimental)))
             ((eq? i 'f2f)          (change-visible ctx a b f2f))
+            ((eq? i 'ipv6)         (change-visible ctx a b ipv6))
             ((eq? i 'f2fr)         (change-visible ctx a b f2fr))
             ((eq? i 'mysql)        (change-visible ctx a b mysql))
             ((eq? i 'fs-loaded)    (change-visible ctx a b fs-loaded))
+            ((eq? i 'hostlist-loaded)    (change-visible ctx a b hostlist-loaded))
             ((eq? i 'nat-unlimited)(change-visible ctx a b nat-unlimited))
             ((eq? i 'tcp-port-nz)  (change-visible ctx a b tcp-port-nz))
             ((eq? i 'udp-port-nz)  (change-visible ctx a b udp-port-nz))
             ((eq? i 'nat-loaded)   (change-visible ctx a b nat-loaded))
             ((eq? i 'udp-loaded)   (change-visible ctx a b udp-loaded))
             ((eq? i 'tcp-loaded)   (change-visible ctx a b tcp-loaded))
-            ((eq? i 'udp6-loaded)  (change-visible ctx a b udp6-loaded))
-            ((eq? i 'tcp6-loaded)  (change-visible ctx a b tcp6-loaded))
-            ((eq? i 'ip6-loaded)   (change-visible ctx a b (or (tcp6-loaded udp6-loaded))))
             ((eq? i 'http-loaded)  (change-visible ctx a b http-loaded))
             ((eq? i 'smtp-loaded)  (change-visible ctx a b smtp-loaded))
             ((eq? i 'nobasiclimit) (change-visible ctx a b nobasiclimit))
