@@ -30,6 +30,7 @@
 #include "gnunet_protocols.h"
 #include "gnunet_bootstrap_service.h"
 #include "gnunet_stats_service.h"
+#include "gnunet_transport_service.h"
 
 #include <curl/curl.h>
 
@@ -38,6 +39,8 @@
  */
 static GNUNET_Stats_ServiceAPI *stats;
 
+static GNUNET_Transport_ServiceAPI *transport;
+
 static GNUNET_CoreAPIForPlugins *coreAPI;
 
 static int stat_hellodownloaded;
@@ -129,7 +132,9 @@ downloadHostlist (GNUNET_BootstrapHelloCallback callback,
                   GNUNET_BootstrapTerminateCallback termTest, void *targ)
 {
   BootstrapContext bctx;
+  unsigned long long protocols;
   char *url;
+  char *purl;
   char *proxy;
   CURL *curl;
   CURLcode ret;
@@ -140,12 +145,14 @@ downloadHostlist (GNUNET_BootstrapHelloCallback callback,
   fd_set ws;
   fd_set es;
   int max;
+  int sret;
   struct timeval tv;
   int running;
   struct CURLMsg *msg;
 #endif
   unsigned int urls;
   size_t pos;
+  int i;
 
   if (0 != curl_global_init (CURL_GLOBAL_WIN32))
     {
@@ -217,6 +224,19 @@ downloadHostlist (GNUNET_BootstrapHelloCallback callback,
   GNUNET_GE_LOG (ectx,
                  GNUNET_GE_INFO | GNUNET_GE_BULK | GNUNET_GE_USER,
                  _("Bootstrapping using `%s'.\n"), url);
+  purl = GNUNET_malloc (strlen (url) + 40);
+  protocols = 0;
+  for (i = GNUNET_TRANSPORT_PROTOCOL_NUMBER_MAX;
+       i > GNUNET_TRANSPORT_PROTOCOL_NUMBER_NAT; i--)
+    {
+      if (transport == NULL)
+        protocols |= (1LL << i);
+      else if (transport->test_available ((unsigned short) i))
+        protocols |= (1LL << i);
+    }
+  sprintf (purl, "%s?p=%llu", url, protocols);
+  GNUNET_free (url);
+  url = purl;
   bctx.url = url;
   bctx.total = 0;
   proxy = NULL;
@@ -228,14 +248,14 @@ downloadHostlist (GNUNET_BootstrapHelloCallback callback,
   if (ret != CURLE_OK)
     goto cleanup;
   CURL_EASY_SETOPT (curl, CURLOPT_FAILONERROR, 1);
-  CURL_EASY_SETOPT (curl, CURLOPT_URL, &url[pos]);
+  CURL_EASY_SETOPT (curl, CURLOPT_URL, url);
   GNUNET_GE_LOG (ectx,
                  GNUNET_GE_INFO | GNUNET_GE_USER | GNUNET_GE_BULK,
-                 _("Trying to download hostlist from `%s'\n"), &url[pos]);
+                 _("Trying to download hostlist from `%s'\n"), url);
   if (strlen (proxy) > 0)
     CURL_EASY_SETOPT (curl, CURLOPT_PROXY, proxy);
   CURL_EASY_SETOPT (curl, CURLOPT_BUFFERSIZE, 1024);    /* a bit more than one HELLO */
-  if (0 == strncmp (&url[pos], "http", 4))
+  if (0 == strncmp (url, "http", 4))
     CURL_EASY_SETOPT (curl, CURLOPT_USERAGENT, "GNUnet");
   CURL_EASY_SETOPT (curl, CURLOPT_CONNECTTIMEOUT, 150L);
   /* NOTE: use of CONNECTTIMEOUT without also
@@ -281,7 +301,14 @@ downloadHostlist (GNUNET_BootstrapHelloCallback callback,
          delay in the reaction than hanging... */
       tv.tv_sec = 0;
       tv.tv_usec = 1000;
-      SELECT (max + 1, &rs, &ws, &es, &tv);
+      sret = SELECT (max + 1, &rs, &ws, &es, &tv);
+      if (sret == -1)
+        {
+          GNUNET_GE_LOG_STRERROR (ectx,
+                                  GNUNET_GE_ERROR | GNUNET_GE_ADMIN |
+                                  GNUNET_GE_USER | GNUNET_GE_BULK, "select");
+          goto cleanup;
+        }
       if (GNUNET_YES != termTest (targ))
         break;
       do
@@ -390,7 +417,8 @@ provide_module_bootstrap (GNUNET_CoreAPIForPlugins * capi)
 
   coreAPI = capi;
   ectx = capi->ectx;
-  stats = coreAPI->request_service ("stats");
+  transport = coreAPI->service_request ("transport");
+  stats = coreAPI->service_request ("stats");
   if (stats != NULL)
     {
       stat_hellodownloaded
@@ -404,7 +432,10 @@ void
 release_module_bootstrap ()
 {
   if (stats != NULL)
-    coreAPI->release_service (stats);
+    coreAPI->service_release (stats);
+  if (transport != NULL)
+    coreAPI->service_release (transport);
+  transport = NULL;
   coreAPI = NULL;
 }
 
