@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2004, 2005, 2006 Christian Grothoff (and other contributing authors)
+     (C) 2004, 2005, 2006, 2008 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -36,7 +36,6 @@
  */
 
 #include "platform.h"
-#include "gnunet_blockstore.h"
 #include "gnunet_directories.h"
 #include "gnunet_collection_lib.h"
 #include "gnunet_util.h"
@@ -46,6 +45,8 @@
  */
 #define COLLECTION "collection"
 
+#define COLLECTION_ROOT "root"
+
 /**
  * How long does a collection advertisement live?
  */
@@ -58,24 +59,9 @@ typedef struct CollectionData
 {
 
   /**
-   * What is the last ID for the publication?
+   * What is the pseudonym ID for the publication?
    */
-  GNUNET_HashCode lastId;
-
-  /**
-   * What is the next ID for the publication?
-   */
-  GNUNET_HashCode nextId;
-
-  /**
-   * What is the update interval? (NBO!)
-   */
-  GNUNET_Int32Time updateInterval;
-
-  /**
-   * What is the update interval? (NBO!)
-   */
-  GNUNET_Int32Time lastPublication;
+  GNUNET_HashCode pid;
 
   /**
    * Anonymity level for the collection. (NBO)
@@ -87,6 +73,16 @@ typedef struct CollectionData
    */
   unsigned int priority;
 
+  /**
+   * Has this collection changed since the last publication?
+   */
+  int changed;
+
+  /**
+   * Revision counter
+   */
+  unsigned int revision;
+
 } CollectionData;
 
 
@@ -95,15 +91,10 @@ typedef struct
 
   CollectionData data;
 
-  /**
-   * Name of the collection
-   */
-  char *name;
-
   /**
    * Metadata describing the collection
    */
-  struct GNUNET_ECRS_MetaData *meta;
+  struct GNUNET_MetaData *meta;
 
   /**
    * Files in the collection.
@@ -115,10 +106,6 @@ typedef struct
    */
   unsigned int file_count;
 
-  /**
-   * Has this collection changed since the last publication?
-   */
-  int changed;
 
 } CollectionInfo;
 
@@ -133,21 +120,7 @@ static struct GNUNET_GC_Configuration *cfg;
 static char *
 getCollectionFileName ()
 {
-  char *fn;
-  char *fnBase;
-
-  GNUNET_GC_get_configuration_value_filename (cfg,
-                                              "GNUNET",
-                                              "GNUNET_HOME",
-                                              GNUNET_DEFAULT_HOME_DIRECTORY,
-                                              &fnBase);
-  fn = GNUNET_malloc (strlen (fnBase) + strlen (COLLECTION) + 4);
-  strcpy (fn, fnBase);
-  GNUNET_disk_directory_create (ectx, fn);
-  strcat (fn, DIR_SEPARATOR_STR);
-  strcat (fn, COLLECTION);
-  GNUNET_free (fnBase);
-  return fn;
+  return GNUNET_get_home_filename (ectx, cfg, GNUNET_NO, COLLECTION, NULL);
 }
 
 /**
@@ -184,7 +157,7 @@ GNUNET_CO_init (struct GNUNET_GE_Context *e,
       return;
     }
   if ((size > 0x7FFFFFFF) ||
-      (size < sizeof (CollectionData) + 4 * sizeof (int)))
+      (size < sizeof (CollectionData) + 2 * sizeof (int)))
     {
       GNUNET_GE_BREAK (ectx, 0);
       UNLINK (fn);
@@ -212,44 +185,21 @@ GNUNET_CO_init (struct GNUNET_GE_Context *e,
       return;
     }
   collectionData = GNUNET_malloc (sizeof (CollectionInfo));
+  memset (collectionData, 0, sizeof (CollectionInfo));
   memcpy (&collectionData->data, buf, sizeof (CollectionData));
   pos = &buf[sizeof (CollectionData)];
   rsize -= sizeof (CollectionData);
   len = ntohl (*(int *) pos);
   if (len > 1024 * 1024 * 4)
-    {
-      GNUNET_GE_BREAK (ectx, 0);
-      len = 1024 * 1024 * 4;
-    }
-  GNUNET_array_grow (collectionData->files, collectionData->file_count, len);
+    goto ERR;
   pos += sizeof (int);
-  collectionData->changed = ntohl (*(int *) pos);
-  pos += sizeof (int);
-  mlen = ntohl (*(int *) pos);
-  pos += sizeof (int);
-  len = ntohl (*(int *) pos);
-  if (len > 1024)
-    {
-      GNUNET_GE_BREAK (ectx, 0);
-      len = 1024;
-    }
-  collectionData->name = GNUNET_malloc (len + 1);
-  pos += sizeof (int);
-  rsize -= 4 * sizeof (int);
-  if (len > rsize)
-    {
-      GNUNET_GE_BREAK (ectx, 0);
-      len = rsize;
-    }
-  memcpy (collectionData->name, pos, len);
-  rsize -= len;
-  pos += len;
+  mlen = ntohl (*(unsigned int *) pos);
+  pos += sizeof (unsigned int);
+  rsize -= 2 * sizeof (int);
   if (mlen > rsize)
-    {
-      GNUNET_GE_BREAK (ectx, 0);
-      mlen = rsize;
-    }
-  collectionData->meta = GNUNET_ECRS_meta_data_deserialize (ectx, pos, mlen);
+    goto ERR;
+  GNUNET_array_grow (collectionData->files, collectionData->file_count, len);
+  collectionData->meta = GNUNET_meta_data_deserialize (ectx, pos, mlen);
   rsize -= mlen;
   pos += mlen;
   GNUNET_GE_BREAK (ectx, collectionData->meta != NULL);
@@ -284,7 +234,7 @@ GNUNET_CO_init (struct GNUNET_GE_Context *e,
       GNUNET_GE_ASSERT (ectx, collectionData->files[i].uri != NULL);
       GNUNET_free (tmp);
       collectionData->files[i].meta
-        = GNUNET_ECRS_meta_data_deserialize (ectx, pos, mlen);
+        = GNUNET_meta_data_deserialize (ectx, pos, mlen);
       GNUNET_GE_ASSERT (ectx, collectionData->files[i].meta != NULL);
       pos += mlen;
       rsize -= mlen;
@@ -302,13 +252,20 @@ GNUNET_CO_init (struct GNUNET_GE_Context *e,
       if (collectionData->files[i].uri != NULL)
         GNUNET_ECRS_uri_destroy (collectionData->files[i].uri);
       if (collectionData->files[i].meta != NULL)
-        GNUNET_ECRS_meta_data_destroy (collectionData->files[i].meta);
+        GNUNET_meta_data_destroy (collectionData->files[i].meta);
       collectionData->files[i]
         = collectionData->files[collectionData->file_count - 1];
       GNUNET_array_grow (collectionData->files,
                          collectionData->file_count,
                          collectionData->file_count - 1);
     }
+  return;
+ERR:
+  MUNMAP (buf, (size_t) size);
+  CLOSE (fd);
+  GNUNET_free (fn);
+  GNUNET_free (collectionData);
+  collectionData = NULL;
 }
 
 static void
@@ -335,12 +292,11 @@ writeCO ()
 
   /* write collection data */
   mlen =
-    GNUNET_ECRS_meta_data_get_serialized_size (collectionData->meta,
-                                               GNUNET_NO);
+    GNUNET_meta_data_get_serialized_size (collectionData->meta, GNUNET_NO);
   buf = GNUNET_malloc (mlen);
-  if (mlen != GNUNET_ECRS_meta_data_serialize (ectx,
-                                               collectionData->meta, buf,
-                                               mlen, GNUNET_NO))
+  if (mlen != GNUNET_meta_data_serialize (ectx,
+                                          collectionData->meta, buf,
+                                          mlen, GNUNET_NO))
     {
       GNUNET_GE_BREAK (ectx, 0);
       GNUNET_free (buf);
@@ -363,23 +319,18 @@ writeCO ()
   GNUNET_GE_BREAK (ectx, collectionData->file_count <= 1024 * 1024 * 4);
   WRITE (fd, collectionData, sizeof (CollectionData));
   WRITEINT (fd, collectionData->file_count);
-  WRITEINT (fd, collectionData->changed);
   WRITEINT (fd, mlen);
-  GNUNET_GE_BREAK (ectx, strlen (collectionData->name) < 1024);
-  WRITEINT (fd, strlen (collectionData->name));
-  WRITE (fd, collectionData->name, strlen (collectionData->name));
   WRITE (fd, buf, mlen);
   GNUNET_free (buf);
   for (i = 0; i < collectionData->file_count; i++)
     {
       mlen =
-        GNUNET_ECRS_meta_data_get_serialized_size (collectionData->files[i].
-                                                   meta, GNUNET_NO);
+        GNUNET_meta_data_get_serialized_size (collectionData->files[i].meta,
+                                              GNUNET_NO);
       buf = GNUNET_malloc (mlen);
-      if (mlen != GNUNET_ECRS_meta_data_serialize (ectx,
-                                                   collectionData->files[i].
-                                                   meta, buf, mlen,
-                                                   GNUNET_NO))
+      if (mlen != GNUNET_meta_data_serialize (ectx,
+                                              collectionData->files[i].meta,
+                                              buf, mlen, GNUNET_NO))
         {
           GNUNET_GE_BREAK (ectx, 0);
           GNUNET_free (buf);
@@ -398,6 +349,24 @@ writeCO ()
   GNUNET_free (fn);
 }
 
+static void
+free_collection_data ()
+{
+  unsigned int i;
+
+  if (collectionData == NULL)
+    return;
+  GNUNET_meta_data_destroy (collectionData->meta);
+  for (i = 0; i < collectionData->file_count; i++)
+    {
+      GNUNET_meta_data_destroy (collectionData->files[i].meta);
+      GNUNET_ECRS_uri_destroy (collectionData->files[i].uri);
+    }
+  GNUNET_array_grow (collectionData->files, collectionData->file_count, 0);
+  GNUNET_free (collectionData);
+  collectionData = NULL;
+}
+
 /**
  * Shutdown collection module.
  */
@@ -405,7 +374,7 @@ void
 GNUNET_CO_done ()
 {
   writeCO ();
-  GNUNET_CO_collection_stop ();
+  free_collection_data ();
   GNUNET_mutex_destroy (lock);
   lock = NULL;
   ectx = NULL;
@@ -425,48 +394,37 @@ GNUNET_CO_done ()
 int
 GNUNET_CO_collection_start (unsigned int anonymityLevel,
                             unsigned int prio,
-                            GNUNET_Int32Time updateInterval,
-                            const char *name,
-                            const struct GNUNET_ECRS_MetaData *meta)
+                            const struct GNUNET_MetaData *meta)
 {
   struct GNUNET_ECRS_URI *advertisement;
   struct GNUNET_ECRS_URI *rootURI;
-  GNUNET_HashCode nextId;
-  GNUNET_Int32Time now;
+  const char *root = COLLECTION_ROOT;
 
   GNUNET_mutex_lock (lock);
   GNUNET_CO_collection_stop (); /* cancel old collection */
-  GNUNET_GE_ASSERT (ectx, name != NULL);
   advertisement = GNUNET_ECRS_keyword_string_to_uri (ectx, COLLECTION);
   GNUNET_GE_ASSERT (ectx, advertisement != NULL);
-  GNUNET_get_time_int32 (&now);
-  GNUNET_create_random_hash (&nextId);
   rootURI = GNUNET_ECRS_namespace_create (ectx,
                                           cfg,
-                                          name,
                                           meta,
                                           anonymityLevel,
                                           prio,
                                           GNUNET_get_time () +
                                           COLLECTION_ADV_LIFETIME,
-                                          advertisement, &nextId);
+                                          advertisement, root);
+  GNUNET_ECRS_uri_destroy (advertisement);
   if (rootURI == NULL)
     {
-      GNUNET_ECRS_uri_destroy (advertisement);
       GNUNET_mutex_unlock (lock);
       return GNUNET_SYSERR;
     }
-  GNUNET_ECRS_uri_destroy (advertisement);
-  GNUNET_ECRS_uri_destroy (rootURI);
   collectionData = GNUNET_malloc (sizeof (CollectionInfo));
   memset (collectionData, 0, sizeof (CollectionInfo));
-  GNUNET_create_random_hash (&collectionData->data.lastId);
-  collectionData->data.nextId = nextId;
-  collectionData->data.updateInterval = htonl (updateInterval);
-  collectionData->data.anonymityLevel = htonl (anonymityLevel);
+  GNUNET_ECRS_uri_get_namespace_from_sks (rootURI, &collectionData->data.pid);
+  GNUNET_ECRS_uri_destroy (rootURI);
   collectionData->data.priority = htonl (prio);
-  collectionData->meta = GNUNET_ECRS_meta_data_duplicate (meta);
-  collectionData->name = GNUNET_strdup (name);
+  collectionData->data.anonymityLevel = htonl (anonymityLevel);
+  collectionData->meta = GNUNET_meta_data_duplicate (meta);
   GNUNET_mutex_unlock (lock);
   return GNUNET_OK;
 }
@@ -479,25 +437,14 @@ GNUNET_CO_collection_start (unsigned int anonymityLevel,
 int
 GNUNET_CO_collection_stop ()
 {
-  unsigned int i;
-
   GNUNET_mutex_lock (lock);
   if (collectionData == NULL)
     {
       GNUNET_mutex_unlock (lock);
       return GNUNET_SYSERR;
     }
-  GNUNET_ECRS_namespace_delete (ectx, cfg, collectionData->name);
-  GNUNET_ECRS_meta_data_destroy (collectionData->meta);
-  for (i = 0; i < collectionData->file_count; i++)
-    {
-      GNUNET_ECRS_meta_data_destroy (collectionData->files[i].meta);
-      GNUNET_ECRS_uri_destroy (collectionData->files[i].uri);
-    }
-  GNUNET_array_grow (collectionData->files, collectionData->file_count, 0);
-  GNUNET_free (collectionData->name);
-  GNUNET_free (collectionData);
-  collectionData = NULL;
+  GNUNET_ECRS_namespace_delete (ectx, cfg, &collectionData->data.pid);
+  free_collection_data ();
   GNUNET_mutex_unlock (lock);
   return GNUNET_OK;
 }
@@ -505,12 +452,12 @@ GNUNET_CO_collection_stop ()
 /**
  * Are we using a collection?
  *
- * @return NULL if there is no collection, otherwise its name
+ * @return NULL if there is no collection, otherwise its metadata
  */
-char *
+struct GNUNET_MetaData *
 GNUNET_CO_collection_get_name ()
 {
-  char *name;
+  struct GNUNET_MetaData *meta;
 
   GNUNET_mutex_lock (lock);
   if (collectionData == NULL)
@@ -518,73 +465,36 @@ GNUNET_CO_collection_get_name ()
       GNUNET_mutex_unlock (lock);
       return NULL;
     }
-  name = GNUNET_strdup (collectionData->name);
+  meta = GNUNET_meta_data_duplicate (collectionData->meta);
   GNUNET_mutex_unlock (lock);
-  return name;
+  return meta;
 }
 
 /**
- * GNUNET_ND_UPLOAD an update of the current collection information to the
+ * Publish an update of the current collection information to the
  * network now.  The function has no effect if the collection has not
  * changed since the last publication.  If we are currently not
  * collecting, this function does nothing.
- *
- * Note that clients typically don't have to call this
- * function explicitly.  CO will call the function on
- * exit (for sporadically updated collections), on any
- * change to the collection (for immediately updated
- * content) or when the publication time has arrived
- * (for periodically updated collections).
- *
- * However, clients may want to call this function if
- * explicit publication of an update at another
- * time is desired.
  */
 void
 GNUNET_CO_collection_publish_now ()
 {
-  GNUNET_HashCode delta;
-  GNUNET_Int32Time now;
   struct GNUNET_ECRS_URI *uri;
   struct GNUNET_ECRS_URI *directoryURI;
   unsigned long long dirLen;
   char *tmpName;
   int fd;
   char *dirData;
+  char this_revision_string[128];
+  char next_revision_string[128];
 
   GNUNET_mutex_lock (lock);
-  if ((collectionData == NULL) || (collectionData->changed == GNUNET_NO))
+  if ((collectionData == NULL) ||
+      (ntohl (collectionData->data.changed) == GNUNET_NO))
     {
       GNUNET_mutex_unlock (lock);
       return;
     }
-  GNUNET_get_time_int32 (&now);
-  if ((ntohl (collectionData->data.updateInterval) !=
-       GNUNET_ECRS_SBLOCK_UPDATE_NONE)
-      && (ntohl (collectionData->data.updateInterval) !=
-          GNUNET_ECRS_SBLOCK_UPDATE_SPORADIC)
-      && (ntohl (collectionData->data.lastPublication) +
-          ntohl (collectionData->data.updateInterval) < now))
-    {
-      GNUNET_mutex_unlock (lock);
-      return;
-    }
-  if ((ntohl (collectionData->data.updateInterval) !=
-       GNUNET_ECRS_SBLOCK_UPDATE_NONE)
-      && (ntohl (collectionData->data.updateInterval) !=
-          GNUNET_ECRS_SBLOCK_UPDATE_SPORADIC))
-    {
-      GNUNET_hash_difference (&collectionData->data.nextId,
-                              &collectionData->data.lastId, &delta);
-      collectionData->data.lastId = collectionData->data.nextId;
-      GNUNET_hash_sum (&collectionData->data.nextId,
-                       &delta, &collectionData->data.nextId);
-    }
-  else
-    {
-      collectionData->data.lastId = collectionData->data.nextId;
-      GNUNET_create_random_hash (&collectionData->data.nextId);
-    }
   tmpName = GNUNET_strdup ("/tmp/gnunet-collectionXXXXXX");
   fd = mkstemp (tmpName);
   if (fd == -1)
@@ -635,25 +545,33 @@ GNUNET_CO_collection_publish_now ()
     }
   UNLINK (tmpName);
   GNUNET_free (tmpName);
+  if (ntohl (collectionData->data.revision) == 0)
+    strcpy (this_revision_string, COLLECTION_ROOT);
+  else
+    GNUNET_snprintf (this_revision_string,
+                     sizeof (this_revision_string),
+                     _("Revision %u"), ntohl (collectionData->data.revision));
+  GNUNET_snprintf (next_revision_string,
+                   sizeof (next_revision_string),
+                   _("Revision %u"),
+                   ntohl (collectionData->data.revision) + 1);
   uri = GNUNET_ECRS_namespace_add_content (ectx,
                                            cfg,
-                                           collectionData->name,
+                                           &collectionData->data.pid,
                                            ntohl (collectionData->data.
                                                   anonymityLevel),
                                            ntohl (collectionData->data.
                                                   priority),
                                            GNUNET_get_time () +
-                                           COLLECTION_ADV_LIFETIME, now,
-                                           ntohl (collectionData->data.
-                                                  updateInterval),
-                                           &collectionData->data.lastId,
-                                           &collectionData->data.nextId,
-                                           directoryURI,
+                                           COLLECTION_ADV_LIFETIME,
+                                           this_revision_string,
+                                           next_revision_string, directoryURI,
                                            collectionData->meta);
   if (uri != NULL)
     {
-      collectionData->data.lastPublication = htonl (now);
-      collectionData->changed = GNUNET_NO;
+      collectionData->data.revision =
+        htonl (ntohl (collectionData->data.revision) + 1);
+      collectionData->data.changed = htonl (GNUNET_NO);
       GNUNET_ECRS_uri_destroy (uri);
     }
   GNUNET_mutex_unlock (lock);
@@ -664,13 +582,6 @@ GNUNET_CO_collection_publish_now ()
  * the given file information in that collection.
  * If we are currently not collecting, this function
  * does nothing.
- *
- * Note that clients typically don't have to call this
- * function explicitly -- by using the CO library it
- * should be called automatically by CO code whenever
- * needed.  However, the function maybe useful if you're
- * inserting files using libECRS directly or need other
- * ways to explicitly extend a collection.
  */
 void
 GNUNET_CO_collection_add_item (const GNUNET_ECRS_FileInfo * fi)
@@ -703,12 +614,9 @@ GNUNET_CO_collection_add_item (const GNUNET_ECRS_FileInfo * fi)
         }
     }
   fc.uri = GNUNET_ECRS_uri_duplicate (fi->uri);
-  fc.meta = GNUNET_ECRS_meta_data_duplicate (fi->meta);
+  fc.meta = GNUNET_meta_data_duplicate (fi->meta);
   GNUNET_array_append (collectionData->files, collectionData->file_count, fc);
-  collectionData->changed = GNUNET_YES;
-  if (ntohl (collectionData->data.updateInterval) ==
-      GNUNET_ECRS_SBLOCK_UPDATE_NONE)
-    GNUNET_CO_collection_publish_now ();
+  collectionData->data.changed = htonl (GNUNET_YES);
   GNUNET_mutex_unlock (lock);
 }
 
