@@ -59,23 +59,24 @@ verifyKBlock (struct GNUNET_GE_Context *ectx,
   GNUNET_ECRS_FileInfo fi;
   unsigned int size;
   GNUNET_HashCode query;
-  KBlock *kb;
+  GNUNET_EC_KBlock *kb;
   const char *dstURI;
   int j;
 
   type = ntohl (value->type);
   size = ntohl (value->size) - sizeof (GNUNET_DatastoreValue);
   if (GNUNET_OK !=
-      GNUNET_EC_file_block_check_and_get_query (size, (DBlock *) & value[1],
-                                                GNUNET_YES, &query))
+      GNUNET_EC_file_block_check_and_get_query (size,
+                                                (GNUNET_EC_DBlock *) &
+                                                value[1], GNUNET_YES, &query))
     return GNUNET_SYSERR;
   GNUNET_GE_ASSERT (ectx, type == GNUNET_ECRS_BLOCKTYPE_KEYWORD);
 
-  if (size < sizeof (KBlock))
+  if (size < sizeof (GNUNET_EC_KBlock))
     return GNUNET_SYSERR;
-  kb = (KBlock *) & value[1];
-  GNUNET_ECRS_decryptInPlace (key, &kb[1], size - sizeof (KBlock));
-  j = sizeof (KBlock);
+  kb = (GNUNET_EC_KBlock *) & value[1];
+  GNUNET_ECRS_decryptInPlace (key, &kb[1], size - sizeof (GNUNET_EC_KBlock));
+  j = sizeof (GNUNET_EC_KBlock);
   while ((j < size) && (((const char *) kb)[j] != '\0'))
     j++;
   if (j == size)
@@ -85,9 +86,8 @@ verifyKBlock (struct GNUNET_GE_Context *ectx,
     }
   dstURI = (const char *) &kb[1];
   j++;
-  fi.meta = GNUNET_ECRS_meta_data_deserialize (ectx,
-                                               &((const char *) kb)[j],
-                                               size - j);
+  fi.meta = GNUNET_meta_data_deserialize (ectx,
+                                          &((const char *) kb)[j], size - j);
   if (fi.meta == NULL)
     {
       GNUNET_GE_BREAK (ectx, 0);        /* kblock malformed */
@@ -97,11 +97,11 @@ verifyKBlock (struct GNUNET_GE_Context *ectx,
   if (fi.uri == NULL)
     {
       GNUNET_GE_BREAK (ectx, 0);        /* kblock malformed */
-      GNUNET_ECRS_meta_data_destroy (fi.meta);
+      GNUNET_meta_data_destroy (fi.meta);
       return GNUNET_SYSERR;
     }
   GNUNET_ECRS_uri_destroy (fi.uri);
-  GNUNET_ECRS_meta_data_destroy (fi.meta);
+  GNUNET_meta_data_destroy (fi.meta);
   return GNUNET_OK;
 }
 
@@ -125,7 +125,7 @@ GNUNET_ECRS_publish_under_keyword (struct GNUNET_GE_Context *ectx,
                                    unsigned int priority,
                                    GNUNET_CronTime expirationTime,
                                    const struct GNUNET_ECRS_URI *dst,
-                                   const struct GNUNET_ECRS_MetaData *md)
+                                   const struct GNUNET_MetaData *md)
 {
   struct GNUNET_ClientServerConnection *sock;
   GNUNET_DatastoreValue *value;
@@ -134,8 +134,9 @@ GNUNET_ECRS_publish_under_keyword (struct GNUNET_GE_Context *ectx,
   unsigned int mdsize;
   struct GNUNET_RSA_PrivateKey *pk;
   char *dstURI;
-  KBlock *kb;
+  GNUNET_EC_KBlock *kb;
   char **keywords;
+  const char *keyword;
   unsigned int keywordCount;
   int i;
 #if DEBUG_KEYSPACE
@@ -153,54 +154,53 @@ GNUNET_ECRS_publish_under_keyword (struct GNUNET_GE_Context *ectx,
       GNUNET_GE_BREAK (ectx, 0);
       return GNUNET_SYSERR;
     }
-  mdsize =
-    GNUNET_ECRS_meta_data_get_serialized_size (md,
-                                               GNUNET_ECRS_SERIALIZE_PART);
+  mdsize = GNUNET_meta_data_get_serialized_size (md, GNUNET_SERIALIZE_PART);
   dstURI = GNUNET_ECRS_uri_to_string (dst);
-  size = mdsize + sizeof (KBlock) + strlen (dstURI) + 1;
+  size = mdsize + sizeof (GNUNET_EC_KBlock) + strlen (dstURI) + 1;
   if (size > MAX_KBLOCK_SIZE)
     {
       size = MAX_KBLOCK_SIZE;
       value = GNUNET_malloc (sizeof (GNUNET_DatastoreValue) + size);
-      kb = (KBlock *) & value[1];
+      kb = (GNUNET_EC_KBlock *) & value[1];
       kb->type = htonl (GNUNET_ECRS_BLOCKTYPE_KEYWORD);
       memcpy (&kb[1], dstURI, strlen (dstURI) + 1);
-      mdsize = size - sizeof (KBlock) - strlen (dstURI) - 1;
-      mdsize = GNUNET_ECRS_meta_data_serialize (ectx,
-                                                md,
-                                                &((char *)
-                                                  &kb[1])[strlen (dstURI) +
-                                                          1], mdsize,
-                                                GNUNET_ECRS_SERIALIZE_PART);
+      mdsize = size - sizeof (GNUNET_EC_KBlock) - strlen (dstURI) - 1;
+      mdsize = GNUNET_meta_data_serialize (ectx,
+                                           md,
+                                           &((char *)
+                                             &kb[1])[strlen (dstURI) +
+                                                     1], mdsize,
+                                           GNUNET_SERIALIZE_PART);
       if (mdsize == -1)
         {
           GNUNET_GE_BREAK (ectx, 0);
           GNUNET_free (dstURI);
+          GNUNET_free (value);
           return GNUNET_SYSERR;
         }
-      size = sizeof (KBlock) + strlen (dstURI) + 1 + mdsize;
+      size = sizeof (GNUNET_EC_KBlock) + strlen (dstURI) + 1 + mdsize;
     }
   else
     {
       value = GNUNET_malloc (sizeof (GNUNET_DatastoreValue) + size);
-      kb = (KBlock *) & value[1];
+      kb = (GNUNET_EC_KBlock *) & value[1];
       kb->type = htonl (GNUNET_ECRS_BLOCKTYPE_KEYWORD);
       memcpy (&kb[1], dstURI, strlen (dstURI) + 1);
       GNUNET_GE_ASSERT (ectx,
                         mdsize ==
-                        GNUNET_ECRS_meta_data_serialize (ectx,
-                                                         md,
-                                                         &((char *)
-                                                           &kb[1])[strlen
-                                                                   (dstURI) +
-                                                                   1], mdsize,
-                                                         GNUNET_ECRS_SERIALIZE_FULL));
+                        GNUNET_meta_data_serialize (ectx,
+                                                    md,
+                                                    &((char *)
+                                                      &kb[1])[strlen
+                                                              (dstURI) +
+                                                              1], mdsize,
+                                                    GNUNET_SERIALIZE_FULL));
     }
   value->size = htonl (sizeof (GNUNET_DatastoreValue) + size);
   value->type = htonl (GNUNET_ECRS_BLOCKTYPE_KEYWORD);
-  value->prio = htonl (priority);
-  value->anonymityLevel = htonl (anonymityLevel);
-  value->expirationTime = GNUNET_htonll (expirationTime);
+  value->priority = htonl (priority);
+  value->anonymity_level = htonl (anonymityLevel);
+  value->expiration_time = GNUNET_htonll (expirationTime);
   sock = GNUNET_client_connection_create (ectx, cfg);
   ret = GNUNET_OK;
 
@@ -218,7 +218,10 @@ GNUNET_ECRS_publish_under_keyword (struct GNUNET_GE_Context *ectx,
   for (i = 0; i < keywordCount; i++)
     {
       memcpy (&kb[1], cpy, mdsize + strlen (dstURI) + 1);
-      GNUNET_hash (keywords[i], strlen (keywords[i]), &key);
+      keyword = keywords[i];
+      /* first character of keyword indicates if it is
+         mandatory or not -- ignore for hashing */
+      GNUNET_hash (&keyword[1], strlen (&keyword[1]), &key);
 #if DEBUG_KEYSPACE
       IF_GELOG (ectx, GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
                 GNUNET_hash_to_enc (&key, &enc));
@@ -240,8 +243,8 @@ GNUNET_ECRS_publish_under_keyword (struct GNUNET_GE_Context *ectx,
       GNUNET_GE_ASSERT (ectx,
                         GNUNET_OK ==
                         GNUNET_EC_file_block_check_and_get_query (size,
-                                                                  (DBlock *)
-                                                                  kb,
+                                                                  (GNUNET_EC_DBlock
+                                                                   *) kb,
                                                                   GNUNET_YES,
                                                                   &hc));
 #endif
