@@ -39,6 +39,8 @@ GNUNET_ECRS_getopt_configure_set_keywords (GNUNET_CommandLineProcessorContext
 {
   struct GNUNET_ECRS_URI **uri = scls;
   struct GNUNET_ECRS_URI *u = *uri;
+  char *val;
+  size_t slen;
 
   if (u == NULL)
     {
@@ -52,15 +54,55 @@ GNUNET_ECRS_getopt_configure_set_keywords (GNUNET_CommandLineProcessorContext
     {
       GNUNET_GE_ASSERT (NULL, u->type == ksk);
     }
+  slen = strlen (value);
+  if (slen == 0)
+    return GNUNET_SYSERR;       /* cannot be empty */
+  if (value[0] == '+')
+    {
+      /* simply preserve the "mandatory" flag */
+      if (slen < 2)
+        return GNUNET_SYSERR;   /* empty keywords not allowed */
+      if ((value[1] == '"') && (slen > 3) && (value[slen - 1] == '"'))
+        {
+          /* remove the quotes, keep the '+' */
+          val = GNUNET_malloc (slen - 1);
+          val[0] = '+';
+          memcpy (&val[1], &value[2], slen - 3);
+          val[slen - 2] = '\0';
+        }
+      else
+        {
+          /* no quotes, just keep the '+' */
+          val = GNUNET_strdup (value);
+        }
+    }
+  else
+    {
+      if ((value[0] == '"') && (slen > 2) && (value[slen - 1] == '"'))
+        {
+          /* remove the quotes, add a space */
+          val = GNUNET_malloc (slen);
+          val[0] = ' ';
+          memcpy (&val[1], &value[1], slen - 2);
+          val[slen - 1] = '\0';
+        }
+      else
+        {
+          /* add a space to indicate "not mandatory" */
+          val = GNUNET_malloc (slen + 2);
+          strcpy (val, " ");
+          strcat (val, value);
+        }
+    }
   GNUNET_array_grow (u->data.ksk.keywords,
                      u->data.ksk.keywordCount, u->data.ksk.keywordCount + 1);
-  u->data.ksk.keywords[u->data.ksk.keywordCount - 1] = GNUNET_strdup (value);
+  u->data.ksk.keywords[u->data.ksk.keywordCount - 1] = val;
   return GNUNET_OK;
 }
 
 
 /**
- * @param scls must be of type "struct GNUNET_ECRS_MetaData **"
+ * @param scls must be of type "struct GNUNET_MetaData **"
  */
 int
 GNUNET_ECRS_getopt_configure_set_metadata (GNUNET_CommandLineProcessorContext
@@ -68,17 +110,17 @@ GNUNET_ECRS_getopt_configure_set_metadata (GNUNET_CommandLineProcessorContext
                                            const char *option,
                                            const char *value)
 {
-  struct GNUNET_ECRS_MetaData **mm = scls;
+  struct GNUNET_MetaData **mm = scls;
   EXTRACTOR_KeywordType type;
   const char *typename;
   const char *typename_i18n;
-  struct GNUNET_ECRS_MetaData *meta;
+  struct GNUNET_MetaData *meta;
   char *tmp;
 
   meta = *mm;
   if (meta == NULL)
     {
-      meta = GNUNET_ECRS_meta_data_create ();
+      meta = GNUNET_meta_data_create ();
       *mm = meta;
     }
 
@@ -99,8 +141,7 @@ GNUNET_ECRS_getopt_configure_set_metadata (GNUNET_CommandLineProcessorContext
           (tmp[strlen (typename)] == ':') &&
           (0 == strncmp (typename, tmp, strlen (typename))))
         {
-          GNUNET_ECRS_meta_data_insert (meta, type,
-                                        &tmp[strlen (typename) + 1]);
+          GNUNET_meta_data_insert (meta, type, &tmp[strlen (typename) + 1]);
           GNUNET_free (tmp);
           tmp = NULL;
           break;
@@ -109,8 +150,8 @@ GNUNET_ECRS_getopt_configure_set_metadata (GNUNET_CommandLineProcessorContext
           (tmp[strlen (typename_i18n)] == ':') &&
           (0 == strncmp (typename_i18n, tmp, strlen (typename_i18n))))
         {
-          GNUNET_ECRS_meta_data_insert (meta, type,
-                                        &tmp[strlen (typename_i18n) + 1]);
+          GNUNET_meta_data_insert (meta, type,
+                                   &tmp[strlen (typename_i18n) + 1]);
           GNUNET_free (tmp);
           tmp = NULL;
           break;
@@ -118,7 +159,7 @@ GNUNET_ECRS_getopt_configure_set_metadata (GNUNET_CommandLineProcessorContext
     }
   if (tmp != NULL)
     {
-      GNUNET_ECRS_meta_data_insert (meta, EXTRACTOR_UNKNOWN, tmp);
+      GNUNET_meta_data_insert (meta, EXTRACTOR_UNKNOWN, tmp);
       GNUNET_free (tmp);
       printf (_
               ("Unknown metadata type in metadata option `%s'.  Using metadata type `unknown' instead.\n"),
@@ -126,3 +167,5 @@ GNUNET_ECRS_getopt_configure_set_metadata (GNUNET_CommandLineProcessorContext
     }
   return GNUNET_OK;
 }
+
+/* end of parser.c */
