@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2004, 2005, 2006 Christian Grothoff (and other contributing authors)
+     (C) 2004, 2005, 2006, 2008 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -31,9 +31,13 @@
 
 #define CHECK(a) if (!(a)) { ok = GNUNET_NO; GNUNET_GE_BREAK(NULL, 0); goto FAILURE; }
 
+static unsigned int app_killer;
+
 static int
 testTerminate (void *unused)
 {
+  if (app_killer++ > 10000)
+    return GNUNET_SYSERR;
   return GNUNET_OK;
 }
 
@@ -44,11 +48,11 @@ searchCB (const GNUNET_ECRS_FileInfo * fi,
           const GNUNET_HashCode * key, int isRoot, void *closure)
 {
   int *cnt = closure;
-#if 1
+#if 0
   char *st;
 
   st = GNUNET_ECRS_uri_to_string (fi->uri);
-  printf ("Got result `%s'\n", st);
+  printf ("Got result `%.*s...'\n", 40, st);
   GNUNET_free (st);
 #endif
   (*cnt)--;
@@ -66,10 +70,7 @@ searchFile (const struct GNUNET_ECRS_URI *uri, int resultCount)
 {
   GNUNET_ECRS_search (NULL,
                       cfg,
-                      uri,
-                      0,
-                      60 * 15 * GNUNET_CRON_SECONDS,
-                      &searchCB, &resultCount, &testTerminate, NULL);
+                      uri, 0, &searchCB, &resultCount, &testTerminate, NULL);
   if (resultCount > 0)
     return GNUNET_SYSERR;
   return GNUNET_OK;
@@ -82,10 +83,8 @@ main (int argc, char *argv[])
   int ok;
   struct GNUNET_ClientServerConnection *sock;
   struct GNUNET_ECRS_URI *uri;
-  struct GNUNET_ECRS_MetaData *meta;
+  struct GNUNET_MetaData *meta;
   struct GNUNET_ECRS_URI *key;
-  const char *keywords[6];
-
 
   cfg = GNUNET_GC_create ();
   if (-1 == GNUNET_GC_parse_configuration (cfg, "check.conf"))
@@ -105,45 +104,38 @@ main (int argc, char *argv[])
   CHECK (sock != NULL);
   /* ACTUAL TEST CODE */
   /* first, simple insertion => one result */
-#if 1
+#if 0
   printf ("Testing search for 'XXtest' with one result.\n");
 #endif
   uri = GNUNET_ECRS_string_to_uri (NULL,
                                    "gnunet://ecrs/sks/C282GG70GKK41O4551011DO413KFBVTVMQG1OG30I0K4045N0G41HAPB82G680A02JRVVFO8URVRU2F159011DO41000000022RG820/test");
-  meta = GNUNET_ECRS_meta_data_create ();
-  keywords[0] = "XXtest";
-  keywords[1] = NULL;
+  CHECK (uri != NULL);
+  meta = GNUNET_meta_data_create ();
 
-  key = GNUNET_ECRS_keyword_strings_to_uri (keywords);
+  key = GNUNET_ECRS_keyword_string_to_uri (NULL, "XXtest");
+  CHECK (GNUNET_OK == GNUNET_ECRS_publish_under_keyword (NULL, cfg, key, 0, 0, GNUNET_get_time () + 10 * GNUNET_CRON_MINUTES,   /* expire */
+                                                         uri, meta));
+  CHECK (GNUNET_OK == searchFile (key, 1));
+  GNUNET_ECRS_uri_destroy (uri);
+
+  uri = GNUNET_ECRS_string_to_uri (NULL,
+                                   "gnunet://ecrs/sks/C282GG70GKK41O4551011DO413KFBVTVMQG1OG30I0K4045N0G41HAPB82G680A02JRVVFO8URVRU2F159011DO41000000022RG820/test-different");
+  CHECK (GNUNET_OK == GNUNET_ECRS_publish_under_keyword (NULL, cfg, key, 0, 0, GNUNET_get_time () + 10 * GNUNET_CRON_MINUTES,   /* expire */
+                                                         uri, meta));
+  GNUNET_ECRS_uri_destroy (key);
+  key = GNUNET_ECRS_keyword_string_to_uri (NULL, "binary");
   CHECK (GNUNET_OK == GNUNET_ECRS_publish_under_keyword (NULL, cfg, key, 0, 0, GNUNET_get_time () + 10 * GNUNET_CRON_MINUTES,   /* expire */
                                                          uri, meta));
   CHECK (GNUNET_OK == searchFile (key, 1));
   GNUNET_ECRS_uri_destroy (key);
   GNUNET_ECRS_uri_destroy (uri);
-
-  /* inserting another URI under the 'XXtest' keyword and under 'binary'
-     should give both URIs since ECRS knows nothing about 'AND'ing: */
-#if 1
-  printf ("Testing search for 'XXtest AND binary' with two results.\n");
-#endif
-  uri = GNUNET_ECRS_string_to_uri (NULL,
-                                   "gnunet://ecrs/sks/C282GG70GKK41O4551011DO413KFBVTVMQG1OG30I0K4045N0G41HAPB82G680A02JRVVFO8URVRU2F159011DO41000000022RG820/test-different");
-  keywords[1] = "binary";
-  keywords[2] = NULL;
-  key = GNUNET_ECRS_keyword_strings_to_uri (keywords);
-  CHECK (GNUNET_OK == GNUNET_ECRS_publish_under_keyword (NULL, cfg, key, 0, 0, GNUNET_get_time () + 10 * GNUNET_CRON_MINUTES,   /* expire */
-                                                         uri, meta));
-  CHECK (GNUNET_OK == searchFile (key, 2));
-  GNUNET_ECRS_uri_destroy (key);
-  GNUNET_ECRS_uri_destroy (uri);
-  GNUNET_ECRS_meta_data_destroy (meta);
+  GNUNET_meta_data_destroy (meta);
 
   /* now searching just for 'XXtest' should again give 2 results! */
 #if 0
   printf ("Testing search for 'XXtest' with two results.\n");
 #endif
-  keywords[1] = NULL;
-  key = GNUNET_ECRS_keyword_strings_to_uri (keywords);
+  key = GNUNET_ECRS_keyword_string_to_uri (NULL, "XXtest");
   CHECK (GNUNET_OK == searchFile (key, 2));
   GNUNET_ECRS_uri_destroy (key);
 
@@ -152,6 +144,7 @@ FAILURE:
   if (sock != NULL)
     GNUNET_client_connection_destroy (sock);
   GNUNET_GE_ASSERT (NULL, GNUNET_OK == GNUNET_daemon_stop (NULL, daemon));
+  GNUNET_GC_free (cfg);
   return (ok == GNUNET_YES) ? 0 : 1;
 }
 
