@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2003, 2004, 2005, 2006, 2007 Christian Grothoff (and other contributing authors)
+     (C) 2003, 2004, 2005, 2006, 2007, 2008 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -67,16 +67,15 @@
  * </li><li>
  *
  * The last category identifies a datum on a specific machine.  The
- * format is "gnunet://ecrs/loc/HEX1.HEX2.SIZE.PEER.SIG1.EXPTIME".  PEER is
- * the BinName of the public key of the peer storing the datum, SIG1 certifies
- * that this peer has this content; SIG2 is a signature for a HELLO
- * about peer, which is encoded in PROTO, SAS, MTU, EXPTIME and ADDR.
+ * format is "gnunet://ecrs/loc/HEX1.HEX2.SIZE.PEER.SIG.EXPTIME".  PEER is
+ * the BinName of the public key of the peer storing the datum.  The
+ * signature (SIG) certifies that this peer has this content.
  * HEX1, HEX2 and SIZE correspond to a 'chk' URI.
  *
  * </li></ul>
  *
  * The encoding for hexadecimal values is defined in the hashing.c
- * module (GNUNET_EncName) in the gnunet-util library and discussed there.
+ * module (GNUNET_EncName) in the gnunetutil library and discussed there.
  * <p>
  */
 
@@ -85,6 +84,17 @@
 #include "gnunet_protocols.h"
 #include "gnunet_ecrs_lib.h"
 
+/**
+ * In URI-encoding, does the given character
+ * need to be encoded using %-encoding?
+ */
+static int
+needs_percent (char c)
+{
+  return (!((isalnum (c)) ||
+            (c == '-') || (c == '_') || (c == '.') || (c == '~')));
+}
+
 /**
  * Generate a keyword URI.
  * @return NULL on error (i.e. keywordCount == 0)
@@ -95,20 +105,54 @@ createKeywordURI (char **keywords, unsigned int keywordCount)
   size_t n;
   char *ret;
   unsigned int i;
+  unsigned int j;
+  unsigned int wpos;
+  size_t slen;
+  const char *keyword;
 
   n =
     keywordCount + strlen (GNUNET_ECRS_URI_PREFIX) +
     strlen (GNUNET_ECRS_SEARCH_INFIX) + 1;
   for (i = 0; i < keywordCount; i++)
-    n += strlen (keywords[i]);
+    {
+      keyword = keywords[i];
+      slen = strlen (keyword);
+      n += slen;
+      for (j = 0; j < slen; j++)
+        {
+          if ((j == 0) && (keyword[j] == ' '))
+            {
+              n--;
+              continue;         /* skip leading space */
+            }
+          if (needs_percent (keyword[j]))
+            n += 2;             /* will use %-encoding */
+        }
+    }
   ret = GNUNET_malloc (n);
   strcpy (ret, GNUNET_ECRS_URI_PREFIX);
   strcat (ret, GNUNET_ECRS_SEARCH_INFIX);
+  wpos = strlen (ret);
   for (i = 0; i < keywordCount; i++)
     {
-      strcat (ret, keywords[i]);
+      keyword = keywords[i];
+      slen = strlen (keyword);
+      for (j = 0; j < slen; j++)
+        {
+          if ((j == 0) && (keyword[j] == ' '))
+            continue;           /* skip leading space */
+          if (needs_percent (keyword[j]))
+            {
+              sprintf (&ret[wpos], "%%%02X", keyword[j]);
+              wpos += 3;
+            }
+          else
+            {
+              ret[wpos++] = keyword[j];
+            }
+        }
       if (i != keywordCount - 1)
-        strcat (ret, "+");
+        ret[wpos++] = '+';
     }
   return ret;
 }
@@ -117,24 +161,21 @@ createKeywordURI (char **keywords, unsigned int keywordCount)
  * Generate a subspace URI.
  */
 static char *
-createSubspaceURI (const GNUNET_HashCode * namespace,
-                   const GNUNET_HashCode * identifier)
+createSubspaceURI (const GNUNET_HashCode * namespace, const char *identifier)
 {
   size_t n;
   char *ret;
   GNUNET_EncName ns;
-  GNUNET_EncName id;
 
   n =
-    sizeof (GNUNET_EncName) * 2 + strlen (GNUNET_ECRS_URI_PREFIX) +
-    strlen (GNUNET_ECRS_SUBSPACE_INFIX) + 1;
+    sizeof (GNUNET_EncName) + strlen (GNUNET_ECRS_URI_PREFIX) +
+    strlen (GNUNET_ECRS_SUBSPACE_INFIX) + 1 + strlen (identifier);
   ret = GNUNET_malloc (n);
   GNUNET_hash_to_enc (namespace, &ns);
-  GNUNET_hash_to_enc (identifier, &id);
   GNUNET_snprintf (ret, n,
                    "%s%s%s/%s",
                    GNUNET_ECRS_URI_PREFIX, GNUNET_ECRS_SUBSPACE_INFIX,
-                   (char *) &ns, (char *) &id);
+                   (const char *) &ns, identifier);
   return ret;
 }
 
@@ -142,7 +183,7 @@ createSubspaceURI (const GNUNET_HashCode * namespace,
  * Generate a file URI.
  */
 static char *
-createFileURI (const FileIdentifier * fi)
+createFileURI (const GNUNET_EC_FileIdentifier * fi)
 {
   char *ret;
   GNUNET_EncName keyhash;
@@ -219,7 +260,7 @@ GNUNET_ECRS_uri_to_string (const struct GNUNET_ECRS_URI *uri)
                                uri->data.ksk.keywordCount);
     case sks:
       return createSubspaceURI (&uri->data.sks.namespace,
-                                &uri->data.sks.identifier);
+                                uri->data.sks.identifier);
     case chk:
       return createFileURI (&uri->data.fi);
     case loc:
@@ -231,7 +272,118 @@ GNUNET_ECRS_uri_to_string (const struct GNUNET_ECRS_URI *uri)
 }
 
 /**
- * Parses an AFS search URI.
+ * Convert keyword URI to a human readable format
+ * (i.e. the search query that was used in the first place)
+ */
+char *
+GNUNET_ECRS_ksk_uri_to_human_readable_string (const struct GNUNET_ECRS_URI
+                                              *uri)
+{
+  size_t n;
+  char *ret;
+  unsigned int i;
+  const char *keyword;
+  char **keywords;
+  unsigned int keywordCount;
+
+  if ((uri == NULL) || (uri->type != ksk))
+    {
+      GNUNET_GE_BREAK (NULL, 0);
+      return NULL;
+    }
+  keywords = uri->data.ksk.keywords;
+  keywordCount = uri->data.ksk.keywordCount;
+  n = keywordCount + 1;
+  for (i = 0; i < keywordCount; i++)
+    {
+      keyword = keywords[i];
+      n += strlen (keyword) - 1;
+      if (NULL != strstr (&keyword[1], " "))
+        n += 2;
+      if (keyword[0] == '+')
+        n++;
+    }
+  ret = GNUNET_malloc (n);
+  strcpy (ret, "");
+  for (i = 0; i < keywordCount; i++)
+    {
+      keyword = keywords[i];
+      if (NULL != strstr (&keyword[1], " "))
+        {
+          strcat (ret, "\"");
+          if (keyword[0] == '+')
+            strcat (ret, keyword);
+          else
+            strcat (ret, &keyword[1]);
+          strcat (ret, "\"");
+        }
+      else
+        {
+          if (keyword[0] == '+')
+            strcat (ret, keyword);
+          else
+            strcat (ret, &keyword[1]);
+        }
+      strcat (ret, " ");
+    }
+  return ret;
+}
+
+/**
+ * Given a keyword with %-encoding (and possibly quotes to protect
+ * spaces), return a copy of the keyword without %-encoding and
+ * without double-quotes (%22).  Also, add a space at the beginning
+ * if there is not a '+'.
+ */
+static char *
+percent_decode_keyword (const char *in)
+{
+  char *out;
+  char *ret;
+  unsigned int rpos;
+  unsigned int wpos;
+  unsigned int hx;
+
+  out = GNUNET_strdup (in);
+  rpos = 0;
+  wpos = 0;
+  while (out[rpos] != '\0')
+    {
+      if (out[rpos] == '%')
+        {
+          if (1 != sscanf (&out[rpos + 1], "%2X", &hx))
+            {
+              GNUNET_free (out);
+              return NULL;
+            }
+          rpos += 3;
+          if (hx == '"')
+            continue;           /* skip double quote */
+          out[wpos++] = (char) hx;
+        }
+      else
+        {
+          out[wpos++] = out[rpos++];
+        }
+    }
+  out[wpos] = '\0';
+  if (out[0] == '+')
+    {
+      ret = GNUNET_strdup (out);
+    }
+  else
+    {
+      /* need to prefix with space */
+      ret = GNUNET_malloc (strlen (out) + 2);
+      strcpy (ret, " ");
+      strcat (ret, out);
+    }
+  GNUNET_free (out);
+  return ret;
+}
+
+/**
+ * Parses an ECRS search URI.
  *
  * @param uri an uri string
  * @param keyword will be set to an array with the keywords
@@ -248,6 +400,7 @@ parseKeywordURI (struct GNUNET_GE_Context *ectx, const char *uri,
   int i;
   size_t slen;
   char *dup;
+  int saw_quote;
 
   GNUNET_GE_ASSERT (ectx, uri != NULL);
 
@@ -271,30 +424,58 @@ parseKeywordURI (struct GNUNET_GE_Context *ectx, const char *uri,
     return GNUNET_SYSERR;       /* no keywords / malformed */
 
   ret = 1;
+  saw_quote = 0;
   for (i = pos; i < slen; i++)
     {
-      if (uri[i] == '+')
+      if ((uri[i] == '%') && (&uri[i] == strstr (&uri[i], "%22")))
+        {
+          saw_quote = (saw_quote + 1) % 2;
+          i += 3;
+          continue;
+        }
+      if ((uri[i] == '+') && (saw_quote == 0))
         {
           ret++;
           if (uri[i - 1] == '+')
             return GNUNET_SYSERR;       /* "++" not allowed */
         }
     }
+  if (saw_quote == 1)
+    return GNUNET_SYSERR;       /* quotes not balanced */
   iret = ret;
   dup = GNUNET_strdup (uri);
   (*keywords) = GNUNET_malloc (ret * sizeof (char *));
+  for (i = 0; i < ret; i++)
+    (*keywords)[i] = NULL;
   for (i = slen - 1; i >= pos; i--)
     {
-      if (dup[i] == '+')
+      if ((uri[i] == '%') && (&uri[i] == strstr (&uri[i], "%22")))
         {
-          (*keywords)[--ret] = GNUNET_strdup (&dup[i + 1]);
+          saw_quote = (saw_quote + 1) % 2;
+          i += 3;
+          continue;
+        }
+      if ((dup[i] == '+') && (saw_quote == 0))
+        {
+          (*keywords)[--ret] = percent_decode_keyword (&dup[i + 1]);
+          if (NULL == (*keywords)[ret])
+            goto CLEANUP;
           dup[i] = '\0';
         }
     }
-  (*keywords)[--ret] = GNUNET_strdup (&dup[pos]);
+  (*keywords)[--ret] = percent_decode_keyword (&dup[pos]);
+  if (NULL == (*keywords)[ret])
+    goto CLEANUP;
   GNUNET_GE_ASSERT (ectx, ret == 0);
   GNUNET_free (dup);
   return iret;
+CLEANUP:
+  for (i = 0; i < ret; i++)
+    GNUNET_free_non_null ((*keywords)[i]);
+  GNUNET_free (*keywords);
+  *keywords = NULL;
+  GNUNET_free (dup);
+  return GNUNET_SYSERR;
 }
 
 /**
@@ -308,7 +489,7 @@ parseKeywordURI (struct GNUNET_GE_Context *ectx, const char *uri,
 static int
 parseSubspaceURI (struct GNUNET_GE_Context *ectx,
                   const char *uri,
-                  GNUNET_HashCode * namespace, GNUNET_HashCode * identifier)
+                  GNUNET_HashCode * namespace, char **identifier)
 {
   unsigned int pos;
   size_t slen;
@@ -338,15 +519,7 @@ parseSubspaceURI (struct GNUNET_GE_Context *ectx,
       GNUNET_free (up);
       return GNUNET_SYSERR;
     }
-  if ((slen != pos + 2 * sizeof (GNUNET_EncName) - 1) ||
-      (GNUNET_OK !=
-       GNUNET_enc_to_hash (&up[pos + sizeof (GNUNET_EncName)], identifier)))
-    {
-      if (up[slen - 1] == '\\')
-        up[--slen] = '\0';
-      GNUNET_hash (&up[pos + sizeof (GNUNET_EncName)],
-                   slen - (pos + sizeof (GNUNET_EncName)), identifier);
-    }
+  *identifier = GNUNET_strdup (&up[pos + sizeof (GNUNET_EncName)]);
   GNUNET_free (up);
   return GNUNET_OK;
 }
@@ -360,7 +533,7 @@ parseSubspaceURI (struct GNUNET_GE_Context *ectx,
  */
 static int
 parseFileURI (struct GNUNET_GE_Context *ectx, const char *uri,
-              FileIdentifier * fi)
+              GNUNET_EC_FileIdentifier * fi)
 {
   unsigned int pos;
   size_t slen;
@@ -473,7 +646,7 @@ parseLocationURI (struct GNUNET_GE_Context *ectx, const char *uri,
     goto ERR;
   /* Finally: verify sigs! */
   if (GNUNET_OK != GNUNET_RSA_verify (&loc->fi,
-                                      sizeof (FileIdentifier) +
+                                      sizeof (GNUNET_EC_FileIdentifier) +
                                       sizeof (GNUNET_PeerIdentity) +
                                       sizeof (GNUNET_Int32Time),
                                       &loc->contentSignature, &loc->peer))
@@ -542,6 +715,9 @@ GNUNET_ECRS_uri_destroy (struct GNUNET_ECRS_URI *uri)
       GNUNET_array_grow (uri->data.ksk.keywords, uri->data.ksk.keywordCount,
                          0);
       break;
+    case sks:
+      GNUNET_free (uri->data.sks.identifier);
+      break;
     case loc:
       break;
     default:
@@ -560,22 +736,6 @@ GNUNET_ECRS_uri_test_sks (const struct GNUNET_ECRS_URI *uri)
   return uri->type == sks;
 }
 
-/**
- * Get the (globally unique) name for the given namespace.
- *
- * @return the name (GNUNET_hash) of the namespace, caller
- *  must free it.
- */
-char *
-GNUNET_ECRS_get_namespace_name (const GNUNET_HashCode * id)
-{
-  char *ret;
-
-  ret = GNUNET_malloc (sizeof (GNUNET_EncName));
-  GNUNET_hash_to_enc (id, (GNUNET_EncName *) ret);
-  return ret;
-}
-
 /**
  * Get the (globally unique) ID of the namespace
  * from the given namespace URI.
@@ -596,21 +756,19 @@ GNUNET_ECRS_uri_get_namespace_from_sks (const struct GNUNET_ECRS_URI *uri,
 }
 
 /**
- * Get the content ID of an SKS URI.
+ * Get the content identifier of an SKS URI.
  *
- * @return GNUNET_OK on success
+ * @return NULL on error
  */
-int
-GNUNET_ECRS_uri_get_content_hash_from_sks (const struct GNUNET_ECRS_URI *uri,
-                                           GNUNET_HashCode * id)
+char *
+GNUNET_ECRS_uri_get_content_id_from_sks (const struct GNUNET_ECRS_URI *uri)
 {
   if (!GNUNET_ECRS_uri_test_sks (uri))
     {
       GNUNET_GE_BREAK (NULL, 0);
-      return GNUNET_SYSERR;
+      return NULL;
     }
-  *id = uri->data.sks.identifier;
-  return GNUNET_OK;
+  return GNUNET_strdup (uri->data.sks.identifier);
 }
 
 /**
@@ -654,19 +812,22 @@ GNUNET_ECRS_uri_get_keywords_from_ksk (const struct GNUNET_ECRS_URI *uri,
                                        GNUNET_ECRS_KeywordIterator iterator,
                                        void *cls)
 {
-  int i;
+  unsigned int i;
+  char *keyword;
+
   if (uri->type != ksk)
+    return -1;
+  if (iterator == NULL)
+    return uri->data.ksk.keywordCount;
+  for (i = 0; i < uri->data.ksk.keywordCount; i++)
     {
-      return -1;
-    }
-  else
-    {
-      for (i = 0; i < uri->data.ksk.keywordCount; i++)
-        if (iterator != NULL)
-          if (GNUNET_OK != iterator (uri->data.ksk.keywords[i], cls))
-            return i;
-      return i;
+      keyword = uri->data.ksk.keywords[i];
+      /* first character of keyword indicates
+         if it is mandatory or not */
+      if (GNUNET_OK != iterator (&keyword[1], keyword[0] == '+', cls))
+        return i;
     }
+  return i;
 }
 
 
@@ -731,6 +892,11 @@ GNUNET_ECRS_uri_duplicate (const URI * uri)
             ret->data.ksk.keywords[i] =
               GNUNET_strdup (uri->data.ksk.keywords[i]);
         }
+      else
+        ret->data.ksk.keywords = NULL;  /* just to be sure */
+      break;
+    case sks:
+      ret->data.sks.identifier = GNUNET_strdup (uri->data.sks.identifier);
       break;
     case loc:
       break;
@@ -790,6 +956,22 @@ GNUNET_ECRS_uri_expand_keywords_with_date (const URI * uri)
   return ret;
 }
 
+static int
+gather_uri_data (EXTRACTOR_KeywordType type, const char *data, void *cls)
+{
+  URI *uri = cls;
+  char *nkword;
+  int j;
+
+  for (j = uri->data.ksk.keywordCount - 1; j >= 0; j--)
+    if (0 == strcmp (&uri->data.ksk.keywords[j][1], data))
+      return GNUNET_OK;
+  nkword = GNUNET_malloc (strlen (data) + 2);
+  strcpy (nkword, " ");         /* not mandatory */
+  strcat (nkword, data);
+  uri->data.ksk.keywords[uri->data.ksk.keywordCount++] = nkword;
+  return GNUNET_OK;
+}
 
 /**
  * Construct a keyword-URI from meta-data (take all entries
@@ -797,13 +979,9 @@ GNUNET_ECRS_uri_expand_keywords_with_date (const URI * uri)
  * that lists all keywords that can be found in the meta-data).
  */
 URI *
-GNUNET_ECRS_meta_data_to_uri (const MetaData * md)
+GNUNET_meta_data_to_uri (const struct GNUNET_MetaData * md)
 {
   URI *ret;
-  int i;
-  int j;
-  int havePreview;
-  int add;
 
   if (md == NULL)
     return NULL;
@@ -811,84 +989,13 @@ GNUNET_ECRS_meta_data_to_uri (const MetaData * md)
   ret->type = ksk;
   ret->data.ksk.keywordCount = 0;
   ret->data.ksk.keywords = NULL;
-  havePreview = 0;
-  for (i = md->itemCount - 1; i >= 0; i--)
-    {
-      if (md->items[i].type == EXTRACTOR_THUMBNAIL_DATA)
-        {
-          havePreview++;
-        }
-      else
-        {
-          for (j = md->itemCount - 1; j > i; j--)
-            {
-              if (0 == strcmp (md->items[i].data, md->items[j].data))
-                {
-                  havePreview++;        /* duplicate! */
-                  break;
-                }
-            }
-        }
-    }
-  GNUNET_array_grow (ret->data.ksk.keywords,
-                     ret->data.ksk.keywordCount, md->itemCount - havePreview);
-  for (i = md->itemCount - 1; i >= 0; i--)
-    {
-      if (md->items[i].type == EXTRACTOR_THUMBNAIL_DATA)
-        {
-          havePreview--;
-        }
-      else
-        {
-          add = 1;
-          for (j = md->itemCount - 1; j > i; j--)
-            {
-              if (0 == strcmp (md->items[i].data, md->items[j].data))
-                {
-                  havePreview--;
-                  add = 0;
-                  break;
-                }
-            }
-          if (add == 1)
-            {
-              GNUNET_GE_ASSERT (NULL, md->items[i].data != NULL);
-              ret->data.ksk.keywords[i - havePreview]
-                = GNUNET_strdup (md->items[i].data);
-            }
-        }
-    }
+  ret->data.ksk.keywords
+    = GNUNET_malloc (sizeof (char *) *
+                     GNUNET_meta_data_get_contents (md, NULL, NULL));
+  GNUNET_meta_data_get_contents (md, &gather_uri_data, ret);
   return ret;
 }
 
-/**
- * Convert a NULL-terminated array of keywords
- * to an ECRS URI.
- */
-struct GNUNET_ECRS_URI *
-GNUNET_ECRS_keyword_strings_to_uri (const char *keyword[])
-{
-  unsigned int count;
-  URI *ret;
-  unsigned int i;
-
-  count = 0;
-  while (keyword[count] != NULL)
-    count++;
-
-  ret = GNUNET_malloc (sizeof (URI));
-  ret->type = ksk;
-  ret->data.ksk.keywordCount = 0;
-  ret->data.ksk.keywords = NULL;
-  GNUNET_array_grow (ret->data.ksk.keywords, ret->data.ksk.keywordCount,
-                     count);
-  for (i = 0; i < count; i++)
-    ret->data.ksk.keywords[i] = GNUNET_strdup (keyword[i]);
-  return ret;
-}
-
-
-
 /**
  * Are these two URIs equal?
  */
@@ -908,16 +1015,15 @@ GNUNET_ECRS_uri_test_equal (const struct GNUNET_ECRS_URI *uri1,
     {
     case chk:
       if (0 == memcmp (&uri1->data.fi,
-                       &uri2->data.fi, sizeof (FileIdentifier)))
+                       &uri2->data.fi, sizeof (GNUNET_EC_FileIdentifier)))
         return GNUNET_YES;
       return GNUNET_NO;
     case sks:
       if ((0 == memcmp (&uri1->data.sks.namespace,
                         &uri2->data.sks.namespace,
                         sizeof (GNUNET_HashCode))) &&
-          (0 == memcmp (&uri1->data.sks.identifier,
-                        &uri2->data.sks.identifier,
-                        sizeof (GNUNET_HashCode))))
+          (0 == strcmp (uri1->data.sks.identifier,
+                        uri2->data.sks.identifier)))
 
         return GNUNET_YES;
       return GNUNET_NO;
@@ -943,7 +1049,7 @@ GNUNET_ECRS_uri_test_equal (const struct GNUNET_ECRS_URI *uri1,
     case loc:
       if (memcmp (&uri1->data.loc,
                   &uri2->data.loc,
-                  sizeof (FileIdentifier) +
+                  sizeof (GNUNET_EC_FileIdentifier) +
                   sizeof (GNUNET_RSA_PublicKey) +
                   sizeof (GNUNET_Int32Time) +
                   sizeof (unsigned short) + sizeof (unsigned short)) != 0)
@@ -992,7 +1098,7 @@ GNUNET_ECRS_uri_get_content_uri_from_loc (const struct GNUNET_ECRS_URI *uri)
  *
  * @param baseURI content offered by the sender
  * @param sender identity of the peer with the content
- * @param expirationTime how long will the content be offered?
+ * @param expiration_time how long will the content be offered?
  * @param proto transport protocol to reach the peer
  * @param sas sender address size (for HELLO)
  * @param address sas bytes of address information
@@ -1018,7 +1124,7 @@ GNUNET_ECRS_location_to_uri (const struct GNUNET_ECRS_URI *baseUri,
   uri->data.loc.peer = *sender;
   uri->data.loc.expirationTime = expirationTime;
   signer (signer_cls,
-          sizeof (FileIdentifier) +
+          sizeof (GNUNET_EC_FileIdentifier) +
           sizeof (GNUNET_PeerIdentity) +
           sizeof (GNUNET_Int32Time),
           &uri->data.loc.fi, &uri->data.loc.contentSignature);
