@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2001, 2002, 2003, 2004, 2005, 2006 Christian Grothoff (and other contributing authors)
+     (C) 2001, 2002, 2003, 2004, 2005, 2006, 2008 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -65,7 +65,7 @@ static GNUNET_FSUI_DownloadList *startDownload (struct GNUNET_FSUI_Context
                                                 const struct GNUNET_ECRS_URI
                                                 *uri,
                                                 const struct
-                                                GNUNET_ECRS_MetaData *meta,
+                                                GNUNET_MetaData *meta,
                                                 const char *filename,
                                                 struct GNUNET_FSUI_SearchList
                                                 *psearch,
@@ -103,7 +103,7 @@ triggerRecursiveDownload (const GNUNET_ECRS_FileInfo * fi,
         return GNUNET_OK;       /* already downloading */
       pos = pos->next;
     }
-  filename = GNUNET_ECRS_meta_data_get_by_type (fi->meta, EXTRACTOR_FILENAME);
+  filename = GNUNET_meta_data_get_by_type (fi->meta, EXTRACTOR_FILENAME);
   if (filename == NULL)
     {
       char *tmp = GNUNET_ECRS_uri_to_string (fi->uri);
@@ -137,6 +137,107 @@ triggerRecursiveDownload (const GNUNET_ECRS_FileInfo * fi,
   return GNUNET_OK;
 }
 
+/**
+ * Trigger recursive download.
+ */
+static void
+download_recursive (GNUNET_FSUI_DownloadList * dl)
+{
+  char *dirBlock;
+  int fd;
+  char *fn;
+  size_t totalBytes;
+  struct GNUNET_MetaData *md;
+
+  totalBytes = GNUNET_ECRS_uri_get_file_size (dl->fi.uri);
+  fn =
+    GNUNET_malloc (strlen (dl->filename) + strlen (GNUNET_DIRECTORY_EXT) + 1);
+  strcpy (fn, dl->filename);
+  fd = strlen (fn) - 1;
+  if (fn[fd] == '/' || fn[fd] == '\\')
+    {
+      fn[fd] = '\0';
+      strcat (fn, GNUNET_DIRECTORY_EXT);
+    }
+  fd = GNUNET_disk_file_open (dl->ctx->ectx, fn, O_LARGEFILE | O_RDONLY);
+  if (fd != -1)
+    {
+      dirBlock = MMAP (NULL, totalBytes, PROT_READ, MAP_SHARED, fd, 0);
+      if (MAP_FAILED == dirBlock)
+        {
+          GNUNET_GE_LOG_STRERROR_FILE (dl->ctx->ectx,
+                                       GNUNET_GE_ERROR | GNUNET_GE_BULK |
+                                       GNUNET_GE_ADMIN | GNUNET_GE_USER,
+                                       "mmap", fn);
+        }
+      else
+        {
+          md = NULL;
+          GNUNET_ECRS_directory_list_contents (dl->ctx->ectx,
+                                               dirBlock,
+                                               totalBytes,
+                                               &md,
+                                               &listURIfoundDirectory, dl);
+          if (md != NULL)
+            GNUNET_meta_data_destroy (md);
+          if (dl->is_recursive)
+            {
+              int n;
+              /* load directory, start downloads */
+              md = NULL;
+              GNUNET_mutex_lock (dl->ctx->lock);
+              n = GNUNET_ECRS_directory_list_contents (dl->ctx->ectx,
+                                                       dirBlock,
+                                                       totalBytes,
+                                                       &md,
+                                                       &triggerRecursiveDownload,
+                                                       dl);
+              GNUNET_mutex_unlock (dl->ctx->lock);
+              if (n == 0)
+                GNUNET_disk_directory_create (dl->ctx->ectx, dl->filename);
+              GNUNET_meta_data_destroy (md);
+              MUNMAP (dirBlock, totalBytes);
+            }
+        }
+      CLOSE (fd);
+    }
+  GNUNET_free (fn);
+}
+
+
+/**
+ * Update the progress bits (shifting).
+ */
+static void
+update_progress_bits(GNUNET_CronTime now,
+		     GNUNET_FSUI_DownloadList * dl)
+{
+  GNUNET_CronTime delta;
+  unsigned int minutes;
+
+  if (now < dl->lastProgressTime)
+    {
+      GNUNET_GE_BREAK(NULL, 0);
+      return;
+    }
+  delta = now - dl->lastProgressTime;
+  minutes = delta / GNUNET_CRON_MINUTES;
+  if (minutes == 0)
+    return;
+  if (minutes > 64)
+    {
+      dl->progressBits = 0;
+      dl->lastProgressTime = now;
+      return;
+    }
+  dl->progressBits <<= minutes;
+  if (dl->progressBits == 0)
+    dl->lastProgressTime = now;
+  else
+    dl->lastProgressTime += minutes * GNUNET_CRON_MINUTES;
+}
+
+
 /**
  * Progress notification from ECRS.  Tell FSUI client.
  */
@@ -150,10 +251,32 @@ downloadProgressCallback (unsigned long long totalBytes,
 {
   GNUNET_FSUI_DownloadList *dl = cls;
   GNUNET_FSUI_Event event;
-  struct GNUNET_ECRS_MetaData *md;
   GNUNET_CronTime now;
   GNUNET_CronTime run_time;
 
+  now = GNUNET_get_time ();
+  update_progress_bits(now, dl);
+  dl->progressBits |= 1;
+  if (dl->total + 1 == totalBytes)
+    {
+      /* error! */
+      dl->state = GNUNET_FSUI_ERROR;
+      event.type = GNUNET_FSUI_download_error;
+      event.data.DownloadError.dc.pos = dl;
+      event.data.DownloadError.dc.cctx = dl->cctx;
+      event.data.DownloadError.dc.ppos =
+        dl->parent == &dl->ctx->activeDownloads ? NULL : dl->parent;
+      event.data.DownloadError.dc.pcctx = dl->parent->cctx;
+      event.data.DownloadError.dc.spos = dl->search;
+      event.data.DownloadError.dc.sctx =
+        dl->search == NULL ? NULL : dl->search->cctx;
+      event.data.DownloadError.message = lastBlock;
+      GNUNET_URITRACK_add_state (dl->ctx->ectx,
+                                 dl->ctx->cfg, dl->fi.uri,
+                                 GNUNET_URITRACK_DOWNLOAD_ABORTED);
+      dl->ctx->ecb (dl->ctx->ecbClosure, &event);
+      return;
+    }
   GNUNET_GE_ASSERT (dl->ctx->ectx, dl->total == totalBytes);
   dl->completed = completedBytes;
   event.type = GNUNET_FSUI_download_progress;
@@ -168,7 +291,6 @@ downloadProgressCallback (unsigned long long totalBytes,
   event.data.DownloadProgress.completed = dl->completed;
   event.data.DownloadProgress.total = dl->total;
   event.data.DownloadProgress.last_offset = lastBlockOffset;
-  now = GNUNET_get_time ();
   run_time = now - dl->startTime;
   if ((dl->total == 0) || (dl->completed == 0))
     {
@@ -192,225 +314,22 @@ downloadProgressCallback (unsigned long long totalBytes,
   if ((lastBlockOffset == 0) && (dl->is_directory == GNUNET_SYSERR))
     {
       /* check if this is a directory */
-      if ((lastBlockSize > strlen (GNUNET_DIRECTORY_MAGIC)) &&
+      if ((dl->filename[strlen (dl->filename) - 1] == '/') &&
+          (lastBlockSize > strlen (GNUNET_DIRECTORY_MAGIC)) &&
           (0 == strncmp (GNUNET_DIRECTORY_MAGIC,
                          lastBlock, strlen (GNUNET_DIRECTORY_MAGIC))))
         dl->is_directory = GNUNET_YES;
       else
         dl->is_directory = GNUNET_NO;
     }
-  if (dl->is_directory == GNUNET_YES)
-    {
-      md = NULL;
-      GNUNET_ECRS_directory_list_contents (dl->ctx->ectx,
-                                           lastBlock,
-                                           lastBlockSize, &md,
-                                           &listURIfoundDirectory, dl);
-      if (md != NULL)
-        GNUNET_ECRS_meta_data_destroy (md);
-    }
-  if ((dl->is_recursive == GNUNET_YES) && (dl->is_directory == GNUNET_YES))
-    {
-      md = NULL;
-      GNUNET_mutex_lock (dl->ctx->lock);
-      GNUNET_ECRS_directory_list_contents (dl->ctx->ectx,
-                                           lastBlock,
-                                           lastBlockSize, &md,
-                                           &triggerRecursiveDownload, dl);
-      GNUNET_mutex_unlock (dl->ctx->lock);
-      if (md != NULL)
-        GNUNET_ECRS_meta_data_destroy (md);
-    }
-}
-
-/**
- * Check if termination of this download is desired.
- */
-static int
-testTerminate (void *cls)
-{
-  GNUNET_FSUI_DownloadList *dl = cls;
-
-  if ((dl->state == GNUNET_FSUI_ERROR) || (dl->state == GNUNET_FSUI_ABORTED))
-    return GNUNET_SYSERR;       /* aborted - delete! */
-  if (dl->state != GNUNET_FSUI_ACTIVE)
-    return GNUNET_NO;           /* suspended */
-  return GNUNET_OK;
-}
-
-/**
- * Thread that downloads a file.
- */
-static void *
-downloadThread (void *cls)
-{
-  GNUNET_FSUI_DownloadList *dl = cls;
-  int ret;
-  GNUNET_FSUI_Event event;
-  struct GNUNET_GE_Context *ectx;
-  struct GNUNET_GE_Memory *mem;
-  struct GNUNET_GE_Context *ee;
-
-  dl->startTime = GNUNET_get_time () - dl->runTime;
-  ectx = dl->ctx->ectx;
-#if DEBUG_DTM
-  GNUNET_GE_LOG (ectx,
-                 GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                 "Download thread for `%s' started...\n", dl->filename);
-#endif
-  GNUNET_GE_ASSERT (ectx, dl->ctx != NULL);
-  GNUNET_GE_ASSERT (ectx, dl->filename != NULL);
-  mem = GNUNET_GE_memory_create (2);
-  ee =
-    GNUNET_GE_create_context_memory (GNUNET_GE_USER | GNUNET_GE_ADMIN |
-                                     GNUNET_GE_ERROR | GNUNET_GE_WARNING |
-                                     GNUNET_GE_FATAL | GNUNET_GE_BULK |
-                                     GNUNET_GE_IMMEDIATE, mem);
-  ret =
-    GNUNET_ECRS_file_download (ee, dl->ctx->cfg, dl->fi.uri, dl->filename,
-                               dl->anonymityLevel, &downloadProgressCallback,
-                               dl, &testTerminate, dl);
-  if (ret == GNUNET_OK)
+  if (totalBytes == completedBytes)
     {
       dl->state = GNUNET_FSUI_COMPLETED;
-      event.type = GNUNET_FSUI_download_completed;
-      event.data.DownloadCompleted.dc.pos = dl;
-      event.data.DownloadCompleted.dc.cctx = dl->cctx;
-      event.data.DownloadCompleted.dc.ppos =
-        dl->parent == &dl->ctx->activeDownloads ? NULL : dl->parent;
-      event.data.DownloadCompleted.dc.pcctx = dl->parent->cctx;
-      event.data.DownloadCompleted.dc.spos = dl->search;
-      event.data.DownloadCompleted.dc.sctx =
-        dl->search == NULL ? NULL : dl->search->cctx;
-      event.data.DownloadCompleted.total = dl->total;
-      event.data.DownloadCompleted.filename = dl->filename;
-      event.data.DownloadCompleted.uri = dl->fi.uri;
       GNUNET_URITRACK_add_state (dl->ctx->ectx,
                                  dl->ctx->cfg,
                                  dl->fi.uri,
                                  GNUNET_URITRACK_DOWNLOAD_COMPLETED);
-      dl->ctx->ecb (dl->ctx->ecbClosure, &event);
     }
-  else if (dl->state == GNUNET_FSUI_ACTIVE)
-    {
-      const char *error;
-
-      /* ECRS error */
-      dl->state = GNUNET_FSUI_ERROR;
-      event.type = GNUNET_FSUI_download_error;
-      event.data.DownloadError.dc.pos = dl;
-      event.data.DownloadError.dc.cctx = dl->cctx;
-      event.data.DownloadError.dc.ppos =
-        dl->parent == &dl->ctx->activeDownloads ? NULL : dl->parent;
-      event.data.DownloadError.dc.pcctx = dl->parent->cctx;
-      event.data.DownloadError.dc.spos = dl->search;
-      event.data.DownloadError.dc.sctx =
-        dl->search == NULL ? NULL : dl->search->cctx;
-      error = GNUNET_GE_memory_get (mem, 0);
-      if (error == NULL)
-        error = _("Download failed (no reason given)");
-      event.data.DownloadError.message = error;
-      GNUNET_URITRACK_add_state (dl->ctx->ectx,
-                                 dl->ctx->cfg, dl->fi.uri,
-                                 GNUNET_URITRACK_DOWNLOAD_ABORTED);
-      dl->ctx->ecb (dl->ctx->ecbClosure, &event);
-    }
-  else if (dl->state == GNUNET_FSUI_ABORTED)
-    {                           /* aborted */
-      event.type = GNUNET_FSUI_download_aborted;
-      event.data.DownloadAborted.dc.pos = dl;
-      event.data.DownloadAborted.dc.cctx = dl->cctx;
-      event.data.DownloadAborted.dc.ppos =
-        dl->parent == &dl->ctx->activeDownloads ? NULL : dl->parent;
-      event.data.DownloadAborted.dc.pcctx = dl->parent->cctx;
-      event.data.DownloadAborted.dc.spos = dl->search;
-      event.data.DownloadAborted.dc.sctx =
-        dl->search == NULL ? NULL : dl->search->cctx;
-      GNUNET_URITRACK_add_state (dl->ctx->ectx, dl->ctx->cfg, dl->fi.uri,
-                                 GNUNET_URITRACK_DOWNLOAD_ABORTED);
-      dl->ctx->ecb (dl->ctx->ecbClosure, &event);
-    }
-  else
-    {
-      /* else: suspended */
-      GNUNET_GE_BREAK (NULL, dl->state == GNUNET_FSUI_SUSPENDING);
-    }
-
-
-  if ((ret == GNUNET_OK) &&
-      (dl->is_directory == GNUNET_YES)
-      && (GNUNET_ECRS_uri_get_file_size (dl->fi.uri) > 0))
-    {
-      char *dirBlock;
-      int fd;
-      char *fn;
-      size_t totalBytes;
-      struct GNUNET_ECRS_MetaData *md;
-
-      totalBytes = GNUNET_ECRS_uri_get_file_size (dl->fi.uri);
-      fn =
-        GNUNET_malloc (strlen (dl->filename) + strlen (GNUNET_DIRECTORY_EXT) +
-                       1);
-      strcpy (fn, dl->filename);
-      fd = strlen (fn) - 1;
-      if (fn[fd] == '/' || fn[fd] == '\\')
-        {
-          fn[fd] = '\0';
-          strcat (fn, GNUNET_DIRECTORY_EXT);
-        }
-      fd = GNUNET_disk_file_open (ectx, fn, O_LARGEFILE | O_RDONLY);
-      if (fd != -1)
-        {
-          dirBlock = MMAP (NULL, totalBytes, PROT_READ, MAP_SHARED, fd, 0);
-          if (MAP_FAILED == dirBlock)
-            {
-              GNUNET_GE_LOG_STRERROR_FILE (ectx,
-                                           GNUNET_GE_ERROR | GNUNET_GE_BULK |
-                                           GNUNET_GE_ADMIN | GNUNET_GE_USER,
-                                           "mmap", fn);
-            }
-          else
-            {
-              md = NULL;
-              GNUNET_ECRS_directory_list_contents (dl->ctx->ectx,
-                                                   dirBlock,
-                                                   totalBytes,
-                                                   &md,
-                                                   &listURIfoundDirectory,
-                                                   dl);
-              if (md != NULL)
-                GNUNET_ECRS_meta_data_destroy (md);
-
-              if (dl->is_recursive)
-                {
-                  /* load directory, start downloads */
-                  md = NULL;
-                  GNUNET_mutex_lock (dl->ctx->lock);
-                  GNUNET_ECRS_directory_list_contents (dl->ctx->ectx,
-                                                       dirBlock,
-                                                       totalBytes,
-                                                       &md,
-                                                       &triggerRecursiveDownload,
-                                                       dl);
-                  GNUNET_mutex_unlock (dl->ctx->lock);
-                  GNUNET_ECRS_meta_data_destroy (md);
-                  MUNMAP (dirBlock, totalBytes);
-                }
-            }
-          CLOSE (fd);
-        }
-      GNUNET_free (fn);
-    }
-#if DEBUG_DTM
-  GNUNET_GE_LOG (ectx,
-                 GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                 "Download thread for `%s' terminated (%s)...\n",
-                 dl->filename, ret == GNUNET_OK ? "COMPLETED" : "ABORTED");
-#endif
-  dl->runTime = GNUNET_get_time () - dl->startTime;
-  GNUNET_GE_free_context (ee);
-  GNUNET_GE_memory_free (mem);
-  return NULL;
 }
 
 /**
@@ -421,7 +340,7 @@ startDownload (struct GNUNET_FSUI_Context *ctx,
                unsigned int anonymityLevel,
                int is_recursive,
                const struct GNUNET_ECRS_URI *uri,
-               const struct GNUNET_ECRS_MetaData *meta,
+               const struct GNUNET_MetaData *meta,
                const char *filename,
                struct GNUNET_FSUI_SearchList *psearch,
                GNUNET_FSUI_DownloadList * parent)
@@ -449,10 +368,12 @@ startDownload (struct GNUNET_FSUI_Context *ctx,
   dl->ctx = ctx;
   dl->filename = GNUNET_strdup (filename);
   dl->fi.uri = GNUNET_ECRS_uri_duplicate (uri);
-  dl->fi.meta = GNUNET_ECRS_meta_data_duplicate (meta);
+  dl->fi.meta = GNUNET_meta_data_duplicate (meta);
   dl->total = GNUNET_ECRS_uri_get_file_size (uri);
   dl->child = NULL;
   dl->cctx = NULL;
+  dl->lastProgressTime = GNUNET_get_time();
+  dl->progressBits = 1;
   /* signal start! */
   event.type = GNUNET_FSUI_download_started;
   event.data.DownloadStarted.dc.pos = dl;
@@ -496,7 +417,7 @@ GNUNET_FSUI_download_start (struct GNUNET_FSUI_Context *ctx,
                             unsigned int anonymityLevel,
                             int doRecursive,
                             const struct GNUNET_ECRS_URI *uri,
-                            const struct GNUNET_ECRS_MetaData *meta,
+                            const struct GNUNET_MetaData *meta,
                             const char *filename,
                             struct GNUNET_FSUI_SearchList *psearch,
                             struct GNUNET_FSUI_DownloadList *pdownload)
@@ -525,13 +446,13 @@ GNUNET_FSUI_updateDownloadThread (GNUNET_FSUI_DownloadList * list)
 {
   struct GNUNET_GE_Context *ectx;
   GNUNET_FSUI_DownloadList *dpos;
-  void *unused;
+  GNUNET_FSUI_Event event;
+  GNUNET_CronTime now;
   int ret;
 
   if (list == NULL)
     return GNUNET_NO;
   ectx = list->ctx->ectx;
-
 #if DEBUG_DTM
   GNUNET_GE_LOG (ectx,
                  GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
@@ -540,12 +461,19 @@ GNUNET_FSUI_updateDownloadThread (GNUNET_FSUI_DownloadList * list)
                  list->ctx->activeDownloadThreads, list->ctx->threadPoolSize);
 #endif
   ret = GNUNET_NO;
+  now = GNUNET_get_time();
   /* should this one be started? */
   if ((list->ctx->threadPoolSize
        > list->ctx->activeDownloadThreads) &&
       (list->state == GNUNET_FSUI_PENDING) &&
+      ( (list->block_resume == 0) ||
+	(list->block_resume == list->ctx->min_block_resume) ||
+	(list->ctx->threadPoolSize > list->ctx->activeDownloadThreads + 1) ) &&
       ((list->total > list->completed) || (list->total == 0)))
     {
+      if (list->block_resume == list->ctx->min_block_resume)
+	list->ctx->min_block_resume = -1;	
+      list->block_resume = 0;
 #if DEBUG_DTM
       GNUNET_GE_LOG (ectx,
                      GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
@@ -553,26 +481,40 @@ GNUNET_FSUI_updateDownloadThread (GNUNET_FSUI_DownloadList * list)
                      list->filename);
 #endif
       list->state = GNUNET_FSUI_ACTIVE;
-      list->handle = GNUNET_thread_create (&downloadThread, list, 128 * 1024);
+      list->startTime = now - list->runTime;
+      list->handle =
+        GNUNET_ECRS_file_download_partial_start (list->ctx->ectx,
+                                                 list->ctx->cfg,
+                                                 NULL,
+                                                 list->fi.uri,
+                                                 list->filename, 0,
+                                                 GNUNET_ECRS_uri_get_file_size
+                                                 (list->fi.uri),
+                                                 list->anonymityLevel,
+                                                 GNUNET_NO,
+                                                 &downloadProgressCallback,
+                                                 list);
+      list->progressBits = 1;
+      list->lastProgressTime = now;
       if (list->handle != NULL)
-        {
-          list->ctx->activeDownloadThreads++;
-        }
+        list->ctx->activeDownloadThreads++;
       else
-        {
-          GNUNET_GE_LOG_STRERROR (ectx,
-                                  GNUNET_GE_ADMIN | GNUNET_GE_USER |
-                                  GNUNET_GE_BULK | GNUNET_GE_ERROR,
-                                  "pthread_create");
-          list->state = GNUNET_FSUI_ERROR_JOINED;
-        }
+        list->state = GNUNET_FSUI_ERROR_JOINED;
     }
-
+  if (list->state == GNUNET_FSUI_ACTIVE)
+    update_progress_bits(now, list);
   /* should this one be stopped? */
-  if ((list->ctx->threadPoolSize
-       < list->ctx->activeDownloadThreads)
-      && (list->state == GNUNET_FSUI_ACTIVE))
+  if ( (list->state == GNUNET_FSUI_ACTIVE) &&
+       ( (list->ctx->threadPoolSize
+	  < list->ctx->activeDownloadThreads) ||
+	 ( (list->ctx->threadPoolSize == list->ctx->activeDownloadThreads) &&
+	   (0 == (list->progressBits & GNUNET_FSUI_DL_KILL_TIME_MASK)) ) ) )
     {
+      if ( (list->ctx->threadPoolSize == list->ctx->activeDownloadThreads) &&
+	   (0 == (list->progressBits & GNUNET_FSUI_DL_KILL_TIME_MASK)) )
+	list->block_resume = now;
+      else
+	list->block_resume = 0;
 #if DEBUG_DTM
       GNUNET_GE_LOG (ectx,
                      GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
@@ -583,18 +525,34 @@ GNUNET_FSUI_updateDownloadThread (GNUNET_FSUI_DownloadList * list)
 #endif
       list->state = GNUNET_FSUI_SUSPENDING;
       GNUNET_GE_ASSERT (ectx, list->handle != NULL);
-      GNUNET_thread_stop_sleep (list->handle);
-      GNUNET_thread_join (list->handle, &unused);
+      GNUNET_ECRS_file_download_partial_stop (list->handle);
       list->handle = NULL;
       list->ctx->activeDownloadThreads--;
       list->state = GNUNET_FSUI_PENDING;
       ret = GNUNET_YES;
     }
 
+
+  /* Trigger any recursive sub-downloads */
+  if (((list->state == GNUNET_FSUI_COMPLETED) ||
+       (list->state == GNUNET_FSUI_COMPLETED_JOINED)) &&
+      (list->is_directory == GNUNET_YES))
+    {
+      /* in case there is no sub-download, still
+         create the (possibly empty) directory! */
+      GNUNET_disk_directory_create (list->ctx->ectx, list->filename);
+      if ((list->is_recursive == GNUNET_YES) &&
+          (GNUNET_ECRS_uri_get_file_size (list->fi.uri) > 0))
+        {
+          download_recursive (list);
+          list->is_recursive = GNUNET_NO;
+        }
+    }
+
   /* has this one "died naturally"? */
   if ((list->state == GNUNET_FSUI_COMPLETED) ||
-      (list->state == GNUNET_FSUI_ABORTED)
-      || (list->state == GNUNET_FSUI_ERROR))
+      (list->state == GNUNET_FSUI_ABORTED) ||
+      (list->state == GNUNET_FSUI_ERROR))
     {
 #if DEBUG_DTM
       GNUNET_GE_LOG (ectx,
@@ -602,10 +560,26 @@ GNUNET_FSUI_updateDownloadThread (GNUNET_FSUI_DownloadList * list)
                      "Download thread manager collects inactive download of file `%s'\n",
                      list->filename);
 #endif
-      GNUNET_thread_stop_sleep (list->handle);
-      GNUNET_thread_join (list->handle, &unused);
+      GNUNET_ECRS_file_download_partial_stop (list->handle);
       list->handle = NULL;
       list->ctx->activeDownloadThreads--;
+      if (list->state == GNUNET_FSUI_COMPLETED)
+        {
+          /* generate event */
+          event.type = GNUNET_FSUI_download_completed;
+          event.data.DownloadCompleted.dc.pos = list;
+          event.data.DownloadCompleted.dc.cctx = list->cctx;
+          event.data.DownloadCompleted.dc.ppos =
+            list->parent == &list->ctx->activeDownloads ? NULL : list->parent;
+          event.data.DownloadCompleted.dc.pcctx = list->parent->cctx;
+          event.data.DownloadCompleted.dc.spos = list->search;
+          event.data.DownloadCompleted.dc.sctx =
+            list->search == NULL ? NULL : list->search->cctx;
+          event.data.DownloadCompleted.total = list->total;
+          event.data.DownloadCompleted.filename = list->filename;
+          event.data.DownloadCompleted.uri = list->fi.uri;
+          list->ctx->ecb (list->ctx->ecbClosure, &event);
+        }
       list->state++;            /* adds _JOINED */
       ret = GNUNET_YES;
     }
@@ -617,40 +591,88 @@ GNUNET_FSUI_updateDownloadThread (GNUNET_FSUI_DownloadList * list)
         ret = GNUNET_YES;
       dpos = dpos->next;
     }
+  if ( (list->block_resume != 0) &&
+       (list->state == GNUNET_FSUI_PENDING) &&
+       (list->block_resume < list->ctx->next_min_block_resume) )
+    list->ctx->next_min_block_resume = list->block_resume;
   return ret;
 }
 
 
 /**
- * Abort a download (and all child-downloads).
+ * Abort a download (and all child-downloads).  This will also
+ * delete all of the files associated with the download
+ * (except if the download has already completed, in which
+ * case GNUNET_NO will be returned).  If this is a recursive
+ * download and some files have been completed, these files
+ * will not be removed (only incomplete downloads will be
+ * removed).
  *
  * @return GNUNET_SYSERR if no such download is pending,
  *         GNUNET_NO if the download has already finished
  */
 int
-GNUNET_FSUI_download_abort (struct GNUNET_FSUI_Context *ctx,
-                            struct GNUNET_FSUI_DownloadList *dl)
+GNUNET_FSUI_download_abort (struct GNUNET_FSUI_DownloadList *dl)
 {
+  struct GNUNET_FSUI_Context *ctx;
   struct GNUNET_FSUI_DownloadList *c;
+  GNUNET_FSUI_Event event;
 
-  GNUNET_GE_ASSERT (ctx->ectx, dl != NULL);
+  if (dl == NULL)
+    return GNUNET_SYSERR;
+  ctx = dl->ctx;
   c = dl->child;
   while (c != NULL)
     {
-      GNUNET_FSUI_download_abort (ctx, c);
+      GNUNET_FSUI_download_abort (c);
       c = c->next;
     }
+  GNUNET_mutex_lock (ctx->lock);
   if ((dl->state != GNUNET_FSUI_ACTIVE) && (dl->state != GNUNET_FSUI_PENDING))
-    return GNUNET_NO;
+    {
+      GNUNET_mutex_unlock (ctx->lock);
+      return GNUNET_NO;
+    }
   if (dl->state == GNUNET_FSUI_ACTIVE)
     {
-      dl->state = GNUNET_FSUI_ABORTED;
-      GNUNET_thread_stop_sleep (dl->handle);
+      dl->state = GNUNET_FSUI_ABORTED_JOINED;
+      GNUNET_ECRS_file_download_partial_stop (dl->handle);
+      dl->handle = NULL;
+      dl->ctx->activeDownloadThreads--;
+      dl->runTime = GNUNET_get_time() - dl->startTime;
+      event.type = GNUNET_FSUI_download_aborted;
+      event.data.DownloadAborted.dc.pos = dl;
+      event.data.DownloadAborted.dc.cctx = dl->cctx;
+      event.data.DownloadAborted.dc.ppos =
+        dl->parent == &dl->ctx->activeDownloads ? NULL : dl->parent;
+      event.data.DownloadAborted.dc.pcctx = dl->parent->cctx;
+      event.data.DownloadAborted.dc.spos = dl->search;
+      event.data.DownloadAborted.dc.sctx =
+        dl->search == NULL ? NULL : dl->search->cctx;
+      GNUNET_URITRACK_add_state (dl->ctx->ectx, dl->ctx->cfg, dl->fi.uri,
+                                 GNUNET_URITRACK_DOWNLOAD_ABORTED);
+      dl->ctx->ecb (dl->ctx->ecbClosure, &event);
     }
   else
     {
       dl->state = GNUNET_FSUI_ABORTED_JOINED;
     }
+  if (0 != UNLINK (dl->filename))
+    {
+      if (errno == EISDIR)
+	{
+	  if ( (0 != RMDIR(dl->filename)) &&
+	       (errno != ENOTEMPTY) ) 
+	    GNUNET_GE_LOG_STRERROR_FILE (dl->ctx->ectx,
+					 GNUNET_GE_WARNING | GNUNET_GE_USER |
+					 GNUNET_GE_BULK, "rmdir", dl->filename);
+	}
+      else if (errno != ENOENT)
+	GNUNET_GE_LOG_STRERROR_FILE (dl->ctx->ectx,
+				     GNUNET_GE_WARNING | GNUNET_GE_USER |
+				     GNUNET_GE_BULK, "unlink", dl->filename);
+    }
+  GNUNET_mutex_unlock (ctx->lock);
   return GNUNET_OK;
 }
 
@@ -661,17 +683,18 @@ GNUNET_FSUI_download_abort (struct GNUNET_FSUI_Context *ctx,
  * @return GNUNET_SYSERR if no such download is pending
  */
 int
-GNUNET_FSUI_download_stop (struct GNUNET_FSUI_Context *ctx,
-                           struct GNUNET_FSUI_DownloadList *dl)
+GNUNET_FSUI_download_stop (struct GNUNET_FSUI_DownloadList *dl)
 {
-  void *unused;
   struct GNUNET_FSUI_DownloadList *prev;
+  struct GNUNET_FSUI_Context *ctx;
   GNUNET_FSUI_Event event;
   int i;
 
-  GNUNET_GE_ASSERT (ctx->ectx, dl != NULL);
+  if (dl == NULL)
+    return GNUNET_SYSERR;
+  ctx = dl->ctx;
   while (dl->child != NULL)
-    GNUNET_FSUI_download_stop (ctx, dl->child);
+    GNUNET_FSUI_download_stop (dl->child);
   GNUNET_mutex_lock (ctx->lock);
   prev =
     (dl->parent != NULL) ? dl->parent->child : ctx->activeDownloads.child;
@@ -682,7 +705,7 @@ GNUNET_FSUI_download_stop (struct GNUNET_FSUI_Context *ctx,
       GNUNET_mutex_unlock (ctx->lock);
       GNUNET_GE_LOG (ctx->ectx,
                      GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                     "GNUNET_FSUI_stopDownload failed to locate download.\n");
+                     "GNUNET_FSUI_download_stop failed to locate download.\n");
       return GNUNET_SYSERR;
     }
   if (prev == dl)
@@ -695,8 +718,9 @@ GNUNET_FSUI_download_stop (struct GNUNET_FSUI_Context *ctx,
       (dl->state == GNUNET_FSUI_ABORTED) || (dl->state == GNUNET_FSUI_ERROR))
     {
       GNUNET_GE_ASSERT (ctx->ectx, dl->handle != NULL);
-      GNUNET_thread_stop_sleep (dl->handle);
-      GNUNET_thread_join (dl->handle, &unused);
+      GNUNET_ECRS_file_download_partial_stop (dl->handle);
+      dl->handle = NULL;
+      dl->runTime = GNUNET_get_time() - dl->startTime;
       GNUNET_mutex_lock (ctx->lock);
       dl->ctx->activeDownloadThreads--;
       GNUNET_mutex_unlock (ctx->lock);
@@ -738,7 +762,8 @@ GNUNET_FSUI_download_stop (struct GNUNET_FSUI_Context *ctx,
     GNUNET_ECRS_uri_destroy (dl->completedDownloads[i]);
   GNUNET_array_grow (dl->completedDownloads, dl->completedDownloadsCount, 0);
   GNUNET_ECRS_uri_destroy (dl->fi.uri);
-  GNUNET_ECRS_meta_data_destroy (dl->fi.meta);
+  if (dl->fi.meta != NULL)
+    GNUNET_meta_data_destroy (dl->fi.meta);
   GNUNET_free (dl->filename);
   GNUNET_free (dl);
   return GNUNET_OK;
