@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2001, 2002, 2003, 2004, 2005, 2006 Christian Grothoff (and other contributing authors)
+     (C) 2001, 2002, 2003, 2004, 2005, 2006, 2008 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -28,13 +28,40 @@
 #include "gnunet_fsui_lib.h"
 #include "gnunet_directories.h"
 #include "fsui.h"
+#include "fs.h"
 
 #define DEBUG_PERSISTENCE GNUNET_NO
 
 /* ***************** CRON code ***************** */
 
+/**
+ * How often should cron run?
+ */
 #define GNUNET_FSUI_UDT_FREQUENCY (2 * GNUNET_CRON_SECONDS)
 
+#define SQUARE(x) ((x)*(x))
+
+/**
+ * We made progress on a test download.  Since
+ * a test download only contains a single block,
+ * any progress means that the test succeeded.
+ * We just set the flag to notify the cron in
+ * the next iteration.
+ */
+static void
+test_download_progress (unsigned long long totalBytes,
+                        unsigned long long completedBytes,
+                        GNUNET_CronTime eta,
+                        unsigned long long lastBlockOffset,
+                        const char *lastBlock, unsigned int lastBlockSize,
+                        void *closure)
+{
+  struct SearchResultList *srl = closure;
+  if (lastBlockSize > 0)        /* check against IBlock events */
+    srl->test_download_start_time = 0;
+}
+
+
 /**
  * Cron job for download load management.
  */
@@ -43,6 +70,12 @@ updateDownloadThreads (void *c)
 {
   GNUNET_FSUI_Context *ctx = c;
   GNUNET_FSUI_DownloadList *dpos;
+  struct SearchResultList *srl;
+  struct GNUNET_FSUI_SearchList *sl;
+  unsigned long long off;
+  unsigned long long len;
+  GNUNET_CronTime now;
+  GNUNET_FSUI_Event event;
 
   GNUNET_mutex_lock (ctx->lock);
   dpos = ctx->activeDownloads.child;
@@ -52,11 +85,118 @@ updateDownloadThreads (void *c)
                    GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
                    "Download thread manager schedules pending downloads...\n");
 #endif
+  ctx->next_min_block_resume = -1;
   while (dpos != NULL)
     {
       GNUNET_FSUI_updateDownloadThread (dpos);
       dpos = dpos->next;
     }
+  ctx->min_block_resume = ctx->next_min_block_resume;
+  now = GNUNET_get_time ();
+  sl = ctx->activeSearches;
+  while (sl != NULL)
+    {
+      srl = sl->resultsReceived;
+      while (srl != NULL)
+        {
+          if (srl->test_download != NULL)
+            {
+              if (srl->test_download_start_time == 0)
+                {
+                  /* probe was successful, kill */
+                  GNUNET_ECRS_file_download_partial_stop (srl->test_download);
+                  srl->test_download = NULL;
+                  srl->probeSuccess++;
+                  event.type = GNUNET_FSUI_search_update;
+                  event.data.SearchUpdate.sc.pos = sl;
+                  event.data.SearchUpdate.sc.cctx = sl->cctx;
+                  event.data.SearchUpdate.fi = srl->fi;
+                  event.data.SearchUpdate.searchURI = sl->uri;
+                  event.data.SearchUpdate.availability_rank =
+                    srl->probeSuccess - srl->probeFailure;
+                  event.data.SearchUpdate.availability_certainty =
+                    srl->probeSuccess + srl->probeFailure;
+                  event.data.SearchUpdate.applicability_rank =
+                    srl->matchingSearchCount;
+                  ctx->ecb (ctx->ecbClosure, &event);
+                  ctx->active_probes--;
+                  srl->last_probe_time = now;
+                }
+              else
+                {
+                  /* consider stopping */
+                  if ((now - srl->test_download_start_time)
+                      >
+                      SQUARE (srl->probeSuccess + srl->probeFailure +
+                              1) * GNUNET_FSUI_PROBE_TIME_FACTOR)
+                    {
+                      /* timeout hit! */
+                      GNUNET_ECRS_file_download_partial_stop
+                        (srl->test_download);
+                      srl->test_download = NULL;
+                      srl->probeFailure++;
+                      event.type = GNUNET_FSUI_search_update;
+                      event.data.SearchUpdate.sc.pos = sl;
+                      event.data.SearchUpdate.sc.cctx = sl->cctx;
+                      event.data.SearchUpdate.fi = srl->fi;
+                      event.data.SearchUpdate.searchURI = sl->uri;
+                      event.data.SearchUpdate.availability_rank =
+                        srl->probeSuccess - srl->probeFailure;
+                      event.data.SearchUpdate.availability_certainty =
+                        srl->probeSuccess + srl->probeFailure;
+                      event.data.SearchUpdate.applicability_rank =
+                        srl->matchingSearchCount;
+                      ctx->ecb (ctx->ecbClosure, &event);
+                      ctx->active_probes--;
+                      srl->last_probe_time = now;
+                    }
+                }
+            }
+          else
+            {
+              len = GNUNET_ECRS_uri_get_file_size (srl->fi.uri);
+              if (len == 0)
+                srl->probeSuccess = -1; /* MAX */
+              /* consider starting */
+              if (((srl->probeSuccess + srl->probeFailure) <
+                   GNUNET_FSUI_MAX_PROBES)
+                  &&
+                  ((srl->last_probe_time <
+                    now +
+                    GNUNET_FSUI_PROBE_DELAY * SQUARE (ctx->active_probes) +
+                    GNUNET_random_u64 (GNUNET_RANDOM_QUALITY_WEAK,
+                                       GNUNET_FSUI_PROBE_DELAY)))
+                  && (ctx->active_probes < GNUNET_FSUI_HARD_PROBE_LIMIT))
+                {
+                  off = len / GNUNET_ECRS_DBLOCK_SIZE;
+                  if (off > 0)
+                    off = GNUNET_random_u32 (GNUNET_RANDOM_QUALITY_WEAK, off);
+                  off *= GNUNET_ECRS_DBLOCK_SIZE;
+                  if (len - off < GNUNET_ECRS_DBLOCK_SIZE)
+                    len = len - off;
+                  else
+                    len = GNUNET_ECRS_DBLOCK_SIZE;
+                  srl->test_download
+                    = GNUNET_ECRS_file_download_partial_start (ctx->ectx,
+                                                               ctx->cfg,
+                                                               sl->probe_context,
+                                                               srl->fi.uri,
+                                                               NULL, off, len,
+                                                               1, GNUNET_YES,
+                                                               &test_download_progress,
+                                                               srl);
+                  if (srl->test_download != NULL)
+                    {
+                      srl->test_download_start_time = now;
+                      ctx->active_probes++;
+                    }
+                }
+            }
+
+          srl = srl->next;
+        }
+      sl = sl->next;
+    }
   GNUNET_mutex_unlock (ctx->lock);
 }
 
@@ -196,8 +336,15 @@ GNUNET_FSUI_start (struct GNUNET_GE_Context *ectx,
   GNUNET_FSUI_Context *ret;
   GNUNET_FSUI_SearchList *list;
   GNUNET_FSUI_UnindexList *xlist;
+  struct SearchResultList *pos;
+  struct SearchRecordList *rec;
+  unsigned int valid;
+  unsigned int i;
+  GNUNET_ECRS_FileInfo *fis;
+  int *av_ranks;
+  unsigned int *av_certs;
+  unsigned int *ap_ranks;
   char *fn;
-  char *gh;
   unsigned long long size;
 
   GNUNET_GE_ASSERT (ectx, cfg != NULL);
@@ -212,23 +359,14 @@ GNUNET_FSUI_start (struct GNUNET_GE_Context *ectx,
   if (ret->threadPoolSize == 0)
     ret->threadPoolSize = 32;
   ret->activeDownloadThreads = 0;
-
-  GNUNET_GC_get_configuration_value_filename (cfg,
-                                              "GNUNET",
-                                              "GNUNET_HOME",
-                                              GNUNET_DEFAULT_HOME_DIRECTORY,
-                                              &gh);
-  GNUNET_disk_directory_create (ectx, gh);
-  fn = GNUNET_malloc (strlen (gh) + strlen (name) + 2 + 5);
-  strcpy (fn, gh);
-  GNUNET_free (gh);
-  strcat (fn, DIR_SEPARATOR_STR);
-  strcat (fn, name);
-  ret->name = fn;
-
+  ret->name = GNUNET_get_home_filename (ectx,
+                                        cfg, GNUNET_NO, "fsui", name, NULL);
   /* 1) read state  in */
   if (doResume)
     {
+      fn = GNUNET_get_home_filename (ectx,
+                                     cfg,
+                                     GNUNET_NO, "fsui-locks", name, NULL);
       ret->ipc = GNUNET_IPC_semaphore_create (ectx, fn, 1);
 #if DEBUG_PERSISTENCE
       GNUNET_GE_LOG (ectx,
@@ -241,7 +379,7 @@ GNUNET_FSUI_start (struct GNUNET_GE_Context *ectx,
                      GNUNET_GE_INFO | GNUNET_GE_REQUEST | GNUNET_GE_USER,
                      "Aquired IPC lock.\n");
 #endif
-      strcat (fn, ".res");
+      GNUNET_free (fn);
       GNUNET_FSUI_deserialize (ret);
     }
   else
@@ -251,23 +389,63 @@ GNUNET_FSUI_start (struct GNUNET_GE_Context *ectx,
   ret->lock = GNUNET_mutex_create (GNUNET_YES);
 
   /* 2) do resume events */
-  /* 2a) signal download restarts */
-  signalDownloadResume (ret->activeDownloads.child, ret);
-  /* 2b) signal search restarts */
+  /* 2a) signal search restarts */
   list = ret->activeSearches;
   while (list != NULL)
     {
+      valid = 0;
+      pos = list->resultsReceived;
+      while (pos != NULL)
+        {
+          if (pos->mandatoryMatchesRemaining == 0)
+            valid++;
+          pos = pos->next;
+        }
+      fis = NULL;
+      av_ranks = NULL;
+      av_certs = NULL;
+      ap_ranks = NULL;
+      if (valid > 0)
+        {
+          fis = GNUNET_malloc (sizeof (GNUNET_ECRS_FileInfo) * valid);
+          av_ranks = GNUNET_malloc (sizeof (int) * valid);
+          av_certs = GNUNET_malloc (sizeof (unsigned int) * valid);
+          ap_ranks = GNUNET_malloc (sizeof (unsigned int) * valid);
+          pos = list->resultsReceived;
+          i = 0;
+          while (pos != NULL)
+            {
+              if (pos->mandatoryMatchesRemaining == 0)
+                {
+                  fis[i] = pos->fi;
+                  av_ranks[i] = pos->probeSuccess - pos->probeFailure;
+                  av_certs[i] = pos->probeSuccess + pos->probeFailure;
+                  ap_ranks[i] = pos->matchingSearchCount;
+                  i++;
+                }
+              pos = pos->next;
+            }
+        }
       event.type = GNUNET_FSUI_search_resumed;
       event.data.SearchResumed.sc.pos = list;
       event.data.SearchResumed.sc.cctx = NULL;
-      event.data.SearchResumed.fis = list->resultsReceived;
-      event.data.SearchResumed.fisSize = list->sizeResultsReceived;
+      event.data.SearchResumed.fis = fis;
+      event.data.SearchResumed.fisSize = valid;
       event.data.SearchResumed.anonymityLevel = list->anonymityLevel;
       event.data.SearchResumed.searchURI = list->uri;
       event.data.SearchResumed.state = list->state;
+      event.data.SearchResumed.availability_rank = av_ranks;
+      event.data.SearchResumed.availability_certainty = av_certs;
+      event.data.SearchResumed.applicability_rank = ap_ranks;
       list->cctx = cb (closure, &event);
+      GNUNET_free_non_null (fis);
+      GNUNET_free_non_null (av_ranks);
+      GNUNET_free_non_null (av_certs);
+      GNUNET_free_non_null (ap_ranks);
       list = list->next;
     }
+  /* 2b) signal download restarts */
+  signalDownloadResume (ret->activeDownloads.child, ret);
   /* 2c) signal upload restarts */
   signalUploadResume (ret->activeUploads.child, ret);
   /* 2d) signal unindex restarts */
@@ -292,14 +470,53 @@ GNUNET_FSUI_start (struct GNUNET_GE_Context *ectx,
 
   /* 3) restart processing */
   ret->cron = GNUNET_cron_create (ectx);
-  /* 3a) resume downloads */
-  GNUNET_cron_add_job (ret->cron,
-                       &updateDownloadThreads, 0, GNUNET_FSUI_UDT_FREQUENCY,
-                       ret);
-  GNUNET_cron_start (ret->cron);
-  /* 3b) resume uploads */
-  doResumeUploads (ret->activeUploads.child, ret);
-  /* 3c) resume unindexing */
+  /* 3a) resume searching */
+  list = ret->activeSearches;
+  while (list != NULL)
+    {
+      list->probe_context
+        = GNUNET_FS_create_search_context (ret->ectx, ret->cfg);
+      if (list->state == GNUNET_FSUI_PENDING)
+        {
+          list->state = GNUNET_FSUI_ACTIVE;
+          rec = list->searches;
+          while (rec != NULL)
+            {
+              rec->search = GNUNET_ECRS_search_start (list->ctx->ectx,
+                                                      list->ctx->cfg,
+                                                      list->probe_context,
+                                                      rec->uri,
+                                                      list->anonymityLevel,
+                                                      &GNUNET_FSUI_search_progress_callback,
+                                                      list);
+              if (rec->search == NULL)
+                {
+                  GNUNET_GE_LOG (ectx,
+                                 GNUNET_GE_FATAL | GNUNET_GE_ADMIN |
+                                 GNUNET_GE_IMMEDIATE,
+                                 "Failed to resume search\n");
+                  list->state = GNUNET_FSUI_PENDING;
+                }
+              rec = rec->next;
+            }
+          if (list->state != GNUNET_FSUI_ACTIVE)
+            {
+              /* stop searches, we failed... */
+              rec = list->searches;
+              while (rec != NULL)
+                {
+                  if (rec->search != NULL)
+                    {
+                      GNUNET_ECRS_search_stop (rec->search);
+                      rec->search = NULL;
+                    }
+                  rec = rec->next;
+                }
+            }
+        }
+      list = list->next;
+    }
+  /* 3b) resume unindexing */
   xlist = ret->unindexOperations;
   while (xlist != NULL)
     {
@@ -315,23 +532,13 @@ GNUNET_FSUI_start (struct GNUNET_GE_Context *ectx,
         }
       xlist = xlist->next;
     }
-  /* 3d) resume searching */
-  list = ret->activeSearches;
-  while (list != NULL)
-    {
-      if (list->state == GNUNET_FSUI_PENDING)
-        {
-          list->state = GNUNET_FSUI_ACTIVE;
-          list->handle =
-            GNUNET_thread_create (&GNUNET_FSUI_searchThread, list, 32 * 1024);
-          if (list->handle == NULL)
-            GNUNET_GE_DIE_STRERROR (ectx,
-                                    GNUNET_GE_FATAL | GNUNET_GE_ADMIN |
-                                    GNUNET_GE_IMMEDIATE, "pthread_create");
-        }
-      list = list->next;
-    }
-
+  /* 3c) resume downloads */
+  GNUNET_cron_add_job (ret->cron,
+                       &updateDownloadThreads, 0, GNUNET_FSUI_UDT_FREQUENCY,
+                       ret);
+  GNUNET_cron_start (ret->cron);
+  /* 3d) resume uploads */
+  doResumeUploads (ret->activeUploads.child, ret);
   return ret;
 }
 
@@ -397,7 +604,7 @@ freeDownloadList (GNUNET_FSUI_DownloadList * list)
     {
       freeDownloadList (list->child);
       GNUNET_ECRS_uri_destroy (list->fi.uri);
-      GNUNET_ECRS_meta_data_destroy (list->fi.meta);
+      GNUNET_meta_data_destroy (list->fi.meta);
       GNUNET_free (list->filename);
       for (i = 0; i < list->completedDownloadsCount; i++)
         GNUNET_ECRS_uri_destroy (list->completedDownloads[i]);
@@ -425,7 +632,7 @@ freeUploadList (struct GNUNET_FSUI_Context *ctx,
       next = list->next;
       GNUNET_free (list->filename);
       if (list->meta != NULL)
-        GNUNET_ECRS_meta_data_destroy (list->meta);
+        GNUNET_meta_data_destroy (list->meta);
       if (list->keywords != NULL)
         GNUNET_ECRS_uri_destroy (list->keywords);
       if (list->uri != NULL)
@@ -444,6 +651,20 @@ freeUploadList (struct GNUNET_FSUI_Context *ctx,
     }
 }
 
+static void
+suspend_active_upload (struct GNUNET_FSUI_UploadList *ul)
+{
+  while (ul != NULL)
+    {
+      if (ul->state == GNUNET_FSUI_ACTIVE)
+        {
+          ul->state = GNUNET_FSUI_PENDING;
+          suspend_active_upload (ul->child);
+        }
+      ul = ul->next;
+    }
+}
+
 /**
  * Stop all processes under FSUI control (serialize state, continue
  * later if possible).
@@ -456,9 +677,10 @@ GNUNET_FSUI_stop (struct GNUNET_FSUI_Context *ctx)
   GNUNET_FSUI_DownloadList *dpos;
   GNUNET_FSUI_UnindexList *xpos;
   GNUNET_FSUI_UploadList *upos;
+  struct SearchRecordList *rec;
+  struct SearchResultList *res;
   GNUNET_FSUI_Event event;
   void *unused;
-  int i;
 
   ectx = ctx->ectx;
   if (ctx->ipc != NULL)
@@ -491,11 +713,32 @@ GNUNET_FSUI_stop (struct GNUNET_FSUI_Context *ctx)
         {
           if (spos->state == GNUNET_FSUI_ACTIVE)
             spos->state = GNUNET_FSUI_PENDING;
-          GNUNET_thread_stop_sleep (spos->handle);
-          GNUNET_thread_join (spos->handle, &unused);
+          rec = spos->searches;
+          while (rec != NULL)
+            {
+              GNUNET_ECRS_search_stop (rec->search);
+              rec->search = NULL;
+              rec = rec->next;
+            }
+          res = spos->resultsReceived;
+          while (res != NULL)
+            {
+              if (res->test_download != NULL)
+                {
+                  GNUNET_ECRS_file_download_partial_stop (res->test_download);
+                  res->test_download = NULL;
+                  ctx->active_probes--;
+                }
+              res = res->next;
+            }
           if (spos->state != GNUNET_FSUI_PENDING)
             spos->state++;      /* _JOINED */
         }
+      if (spos->probe_context != NULL)
+        {
+          GNUNET_FS_destroy_search_context (spos->probe_context);
+          spos->probe_context = NULL;
+        }
       spos = spos->next;
     }
   /* 1c) stop unindexing */
@@ -525,10 +768,9 @@ GNUNET_FSUI_stop (struct GNUNET_FSUI_Context *ctx)
           (upos->state == GNUNET_FSUI_ERROR)
           || (upos->state == GNUNET_FSUI_COMPLETED))
         {
-          /* NOTE: will force transitive termination
-             of rest of tree! */
           if (upos->state == GNUNET_FSUI_ACTIVE)
             upos->state = GNUNET_FSUI_PENDING;
+          suspend_active_upload (upos->child);
           GNUNET_thread_stop_sleep (upos->shared->handle);
           GNUNET_thread_join (upos->shared->handle, &unused);
           if (upos->state != GNUNET_FSUI_PENDING)
@@ -538,21 +780,11 @@ GNUNET_FSUI_stop (struct GNUNET_FSUI_Context *ctx)
     }
 
   /* 2) signal suspension events */
-  /* 2a) signal search suspension */
-  spos = ctx->activeSearches;
-  while (spos != NULL)
-    {
-      event.type = GNUNET_FSUI_search_suspended;
-      event.data.SearchSuspended.sc.pos = spos;
-      event.data.SearchSuspended.sc.cctx = spos->cctx;
-      ctx->ecb (ctx->ecbClosure, &event);
-      spos = spos->next;
-    }
-  /* 2b) signal uploads suspension */
+  /* 2a) signal uploads suspension */
   signalUploadSuspend (ectx, ctx, ctx->activeUploads.child);
-  /* 2c) signal downloads suspension */
+  /* 2b) signal downloads suspension */
   signalDownloadSuspend (ectx, ctx, ctx->activeDownloads.child);
-  /* 2d) signal unindex suspension */
+  /* 2c) signal unindex suspension */
   xpos = ctx->unindexOperations;
   while (xpos != NULL)
     {
@@ -562,6 +794,16 @@ GNUNET_FSUI_stop (struct GNUNET_FSUI_Context *ctx)
       ctx->ecb (ctx->ecbClosure, &event);
       xpos = xpos->next;
     }
+  /* 2d) signal search suspension */
+  spos = ctx->activeSearches;
+  while (spos != NULL)
+    {
+      event.type = GNUNET_FSUI_search_suspended;
+      event.data.SearchSuspended.sc.pos = spos;
+      event.data.SearchSuspended.sc.cctx = spos->cctx;
+      ctx->ecb (ctx->ecbClosure, &event);
+      spos = spos->next;
+    }
 
   /* 3) serialize all of the FSUI state */
   if (ctx->ipc != NULL)
@@ -574,25 +816,22 @@ GNUNET_FSUI_stop (struct GNUNET_FSUI_Context *ctx)
       spos = ctx->activeSearches;
       ctx->activeSearches = spos->next;
       GNUNET_ECRS_uri_destroy (spos->uri);
-      for (i = spos->sizeResultsReceived - 1; i >= 0; i--)
+      while (spos->searches != NULL)
         {
-          GNUNET_ECRS_FileInfo *fi;
-          fi = &spos->resultsReceived[i];
-          GNUNET_ECRS_meta_data_destroy (fi->meta);
-          GNUNET_ECRS_uri_destroy (fi->uri);
+          rec = spos->searches;
+          spos->searches = rec->next;
+          GNUNET_ECRS_uri_destroy (rec->uri);
+          GNUNET_free (rec);
         }
-      GNUNET_array_grow (spos->resultsReceived, spos->sizeResultsReceived, 0);
-      for (i = spos->sizeUnmatchedResultsReceived - 1; i >= 0; i--)
+      while (spos->resultsReceived != NULL)
         {
-          ResultPending *rp;
-
-          rp = &spos->unmatchedResultsReceived[i];
-          GNUNET_array_grow (rp->matchingKeys, rp->matchingKeyCount, 0);
-          GNUNET_ECRS_meta_data_destroy (rp->fi.meta);
-          GNUNET_ECRS_uri_destroy (rp->fi.uri);
+          res = spos->resultsReceived;
+          spos->resultsReceived = res->next;
+          GNUNET_meta_data_destroy (res->fi.meta);
+          GNUNET_ECRS_uri_destroy (res->fi.uri);
+          GNUNET_free (res->matchingSearches);
+          GNUNET_free (res);
         }
-      GNUNET_array_grow (spos->unmatchedResultsReceived,
-                         spos->sizeUnmatchedResultsReceived, 0);
       GNUNET_free (spos);
     }
   /* 4b) free unindex memory */
