@@ -59,7 +59,7 @@ progressCallbackR (unsigned long long totalBytes,
   event.data.UploadProgress.uc.cctx = utc->cctx;
   event.data.UploadProgress.uc.ppos = utc->parent;
   event.data.UploadProgress.uc.pcctx = utc->parent->cctx;
-  if (GNUNET_YES == GNUNET_ECRS_meta_data_test_for_directory (utc->meta))
+  if (GNUNET_YES == GNUNET_meta_data_test_for_directory (utc->meta))
     {
       if (direct == GNUNET_YES)
         unaccounted = GNUNET_YES;
@@ -146,7 +146,7 @@ static char *
 createDirectoryHelper (struct GNUNET_GE_Context *ectx,
                        struct GNUNET_GC_Configuration *cfg,
                        struct GNUNET_FSUI_UploadList *children,
-                       struct GNUNET_ECRS_MetaData *meta, char **error)
+                       struct GNUNET_MetaData *meta, char **error)
 {
   GNUNET_ECRS_FileInfo *fis;
   unsigned int count;
@@ -293,17 +293,18 @@ GNUNET_FSUI_uploadThread (void *cls)
   GNUNET_FSUI_Event event;
   GNUNET_ECRS_FileInfo fi;
   int ret;
+  int is_directory;
   struct GNUNET_GE_Context *ectx;
   char *filename;
   char *pfn;
   struct GNUNET_ECRS_URI *uri;
   struct GNUNET_ECRS_URI *loc;
   size_t tpos;
+  size_t tend;
   char *error;
   struct GNUNET_GE_Memory *mem;
   struct GNUNET_GE_Context *ee;
 
-
   ectx = utc->shared->ctx->ectx;
   GNUNET_GE_ASSERT (ectx, utc->filename != NULL);
   cpos = utc->child;
@@ -315,9 +316,15 @@ GNUNET_FSUI_uploadThread (void *cls)
     }
   if (utc->state != GNUNET_FSUI_ACTIVE)
     return NULL;                /* aborted or suspended */
+  if (GNUNET_shutdown_test ())
+    {
+      signalError (utc, _("Application aborted."));
+      return NULL;
+    }
   if (GNUNET_YES == GNUNET_disk_directory_test (ectx, utc->filename))
     {
       error = NULL;
+      is_directory = 1;
       filename = createDirectoryHelper (ectx,
                                         utc->shared->ctx->cfg,
                                         utc->child, utc->meta, &error);
@@ -333,6 +340,7 @@ GNUNET_FSUI_uploadThread (void *cls)
     }
   else
     {
+      is_directory = 0;
       filename = GNUNET_strdup (utc->filename);
     }
   utc->start_time = GNUNET_get_time ();
@@ -347,7 +355,8 @@ GNUNET_FSUI_uploadThread (void *cls)
                              utc->shared->doIndex ==
                              GNUNET_YES ? (utc->child ==
                                            NULL ? GNUNET_YES : GNUNET_NO) :
-                             GNUNET_NO, utc->shared->anonymityLevel,
+                             utc->shared->doIndex,
+                             utc->shared->anonymityLevel,
                              utc->shared->priority, utc->shared->expiration,
                              &progressCallback, utc, &testTerminate, utc,
                              &utc->uri);
@@ -382,102 +391,145 @@ GNUNET_FSUI_uploadThread (void *cls)
       return NULL;
     }
   utc->state = GNUNET_FSUI_COMPLETED;
-  if (utc->child == NULL)
-    GNUNET_ECRS_meta_data_extract_from_file (utc->shared->ctx->ectx,
-                                             utc->meta, utc->filename,
-                                             utc->shared->extractors);
-  while (GNUNET_OK ==
-         GNUNET_ECRS_meta_data_delete (utc->meta, EXTRACTOR_FILENAME, NULL));
-  /* only publish the last part of the path
-     -- we do not want to publish $HOME or similar
-     trivially deanonymizing information */
-  tpos = strlen (utc->filename) - 1;
-  if ((utc->filename[tpos] == DIR_SEPARATOR) && (tpos > 0))
-    tpos--;
-  while ((tpos > 0) && (utc->filename[tpos] != DIR_SEPARATOR))
-    tpos--;
-  pfn = GNUNET_malloc (strlen (&utc->filename[tpos + 1]) + 2);
-  strcpy (pfn, &utc->filename[tpos + 1]);
-  if ((utc->child != NULL) &&
-      ((strlen (pfn) == 0) || (pfn[strlen (pfn) - 1] != DIR_SEPARATOR)))
-    strcat (pfn, DIR_SEPARATOR_STR);
-  GNUNET_ECRS_meta_data_insert (utc->meta, EXTRACTOR_FILENAME, pfn);
-  GNUNET_free (pfn);
-  if ((utc->shared->anonymityLevel == 0)
-      && (utc->shared->doIndex == GNUNET_YES))
+  if (utc->shared->doIndex != GNUNET_SYSERR)
     {
-      /* generate location URI for non-anonymous download */
-      struct GNUNET_ClientServerConnection *sock;
-      GNUNET_MessageHello *hello;
-
-      sock = GNUNET_client_connection_create (utc->shared->ctx->ectx,
-                                              utc->shared->ctx->cfg);
-
-      if (GNUNET_OK == GNUNET_IDENTITY_get_self (sock, &hello))
+      if (utc->child == NULL)
+        GNUNET_meta_data_extract_from_file (utc->shared->ctx->ectx,
+                                            utc->meta, utc->filename,
+                                            utc->shared->extractors);
+      while (GNUNET_OK ==
+             GNUNET_meta_data_delete (utc->meta, EXTRACTOR_FILENAME, NULL));
+      /* only publish the last part of the path
+         -- we do not want to publish $HOME or similar
+         trivially deanonymizing information */
+      tpos = strlen (utc->filename) - 1;
+      if ((utc->filename[tpos] == DIR_SEPARATOR) && (tpos > 0))
+        tpos--;
+      while ((tpos > 0) && (utc->filename[tpos] != DIR_SEPARATOR))
+        tpos--;
+      pfn = GNUNET_malloc (strlen (&utc->filename[tpos + 1]) + 2);
+      strcpy (pfn, &utc->filename[tpos + 1]);
+      if ((is_directory || (utc->child != NULL)) &&
+          ((strlen (pfn) == 0) || (pfn[strlen (pfn) - 1] != DIR_SEPARATOR)))
+        strcat (pfn, DIR_SEPARATOR_STR);
+      GNUNET_meta_data_insert (utc->meta, EXTRACTOR_FILENAME, pfn);
+      GNUNET_free (pfn);
+      if (0 != strcmp (utc->shared->top_filename, utc->filename))
         {
-          loc = GNUNET_ECRS_location_to_uri (utc->uri,
-                                             &hello->publicKey,
-                                             ntohl (hello->expirationTime),
-                                             (GNUNET_ECRS_SignFunction) &
-                                             GNUNET_IDENTITY_sign_function,
-                                             sock);
+          /* this is NOT the top-level upload, so we
+             should add the directory name of our
+             parent to the meta data */
+          tend = tpos;          /* index of '/' */
+          if ((utc->filename[tpos] == DIR_SEPARATOR) && (tpos > 0))
+            tpos--;
+          while ((tpos > 0) && (utc->filename[tpos] != DIR_SEPARATOR))
+            tpos--;
+          if (tpos + 1 < tend)
+            {
+              char *p;
+              pfn = p = GNUNET_malloc (tend - tpos + 1);
+              pfn[tend - tpos] = '\0';
+              memcpy (pfn, &utc->filename[tpos + 1], tend - tpos);
+              /* change OS native dir separators to unix '/' and others to '_' */
+              while (*p != '\0')
+                {
+                  if (*p == DIR_SEPARATOR)
+                    *p = '/';
+                  else if (*p == '\\')
+                    *p = '_';
+                  p++;
+                }
+              GNUNET_meta_data_insert (utc->meta, EXTRACTOR_RELATION, pfn);
+              GNUNET_free (pfn);
+            }
+        }
+      if ((utc->shared->anonymityLevel == 0)
+          && (utc->shared->doIndex == GNUNET_YES))
+        {
+          /* generate location URI for non-anonymous download */
+          struct GNUNET_ClientServerConnection *sock;
+          GNUNET_MessageHello *hello;
 
-          GNUNET_free (hello);
+          sock = GNUNET_client_connection_create (utc->shared->ctx->ectx,
+                                                  utc->shared->ctx->cfg);
+
+          if (GNUNET_OK == GNUNET_IDENTITY_get_self (sock, &hello))
+            {
+              loc = GNUNET_ECRS_location_to_uri (utc->uri,
+                                                 &hello->publicKey,
+                                                 ntohl
+                                                 (hello->expiration_time),
+                                                 (GNUNET_ECRS_SignFunction) &
+                                                 GNUNET_IDENTITY_sign_function,
+                                                 sock);
+
+              GNUNET_free (hello);
+            }
+          else
+            {
+              /* may happen if no transports are available... */
+              loc = GNUNET_ECRS_uri_duplicate (utc->uri);
+            }
+          GNUNET_client_connection_destroy (sock);
         }
       else
         {
-          /* may happen if no transports are available... */
+          /* no location URI, use standard URI
+             (copied here to allow free later) */
           loc = GNUNET_ECRS_uri_duplicate (utc->uri);
         }
-      GNUNET_client_connection_destroy (sock);
+      if (utc->shared->individualKeywords == GNUNET_YES)
+	{
+	  /* need to convert to URI *before*
+	     removing split/lower from meta */
+	  uri = GNUNET_meta_data_to_uri (utc->meta);
+	}
+      while (GNUNET_OK ==
+	     GNUNET_meta_data_delete (utc->meta, EXTRACTOR_SPLIT, NULL));
+      while (GNUNET_OK ==
+	     GNUNET_meta_data_delete (utc->meta, EXTRACTOR_LOWERCASE, NULL));
+      if (utc->shared->individualKeywords == GNUNET_YES)
+        {
+          GNUNET_ECRS_publish_under_keyword (ectx,
+                                             utc->shared->ctx->cfg,
+                                             uri,
+                                             utc->shared->anonymityLevel,
+                                             utc->shared->priority,
+                                             utc->shared->expiration, loc,
+                                             utc->meta);
+          GNUNET_ECRS_uri_destroy (uri);
+        }
+      if (utc->shared->global_keywords != NULL)
+        GNUNET_ECRS_publish_under_keyword (ectx,
+                                           utc->shared->ctx->cfg,
+                                           utc->shared->global_keywords,
+                                           utc->shared->anonymityLevel,
+                                           utc->shared->priority,
+                                           utc->shared->expiration, loc,
+                                           utc->meta);
+      if (utc->keywords != NULL)
+        GNUNET_ECRS_publish_under_keyword (ectx,
+                                           utc->shared->ctx->cfg,
+                                           utc->keywords,
+                                           utc->shared->anonymityLevel,
+                                           utc->shared->priority,
+                                           utc->shared->expiration, loc,
+                                           utc->meta);
+      GNUNET_ECRS_uri_destroy (loc);
+      loc = NULL;
+      fi.meta = utc->meta;
+      fi.uri = utc->uri;
+      if (utc->shared->doIndex != GNUNET_SYSERR)
+        {
+          GNUNET_URITRACK_track (ectx, utc->shared->ctx->cfg, &fi);
+          GNUNET_URITRACK_add_state (ectx,
+                                     utc->shared->ctx->cfg,
+                                     utc->uri,
+                                     utc->shared->doIndex ==
+                                     GNUNET_YES ? GNUNET_URITRACK_INDEXED :
+                                     GNUNET_URITRACK_INSERTED);
+        }
     }
-  else
-    {
-      /* no location URI, use standard URI
-         (copied here to allow free later) */
-      loc = GNUNET_ECRS_uri_duplicate (utc->uri);
-    }
-  if (utc->shared->global_keywords != NULL)
-    GNUNET_ECRS_publish_under_keyword (ectx,
-                                       utc->shared->ctx->cfg,
-                                       utc->shared->global_keywords,
-                                       utc->shared->anonymityLevel,
-                                       utc->shared->priority,
-                                       utc->shared->expiration, loc,
-                                       utc->meta);
-  if (utc->keywords != NULL)
-    GNUNET_ECRS_publish_under_keyword (ectx,
-                                       utc->shared->ctx->cfg,
-                                       utc->keywords,
-                                       utc->shared->anonymityLevel,
-                                       utc->shared->priority,
-                                       utc->shared->expiration, loc,
-                                       utc->meta);
-  if (utc->shared->individualKeywords == GNUNET_YES)
-    {
-      uri = GNUNET_ECRS_meta_data_to_uri (utc->meta);
-      GNUNET_ECRS_publish_under_keyword (ectx,
-                                         utc->shared->ctx->cfg,
-                                         uri,
-                                         utc->shared->anonymityLevel,
-                                         utc->shared->priority,
-                                         utc->shared->expiration, loc,
-                                         utc->meta);
-      GNUNET_ECRS_uri_destroy (uri);
-    }
-  GNUNET_ECRS_uri_destroy (loc);
-  loc = NULL;
-  while (GNUNET_OK ==
-         GNUNET_ECRS_meta_data_delete (utc->meta, EXTRACTOR_SPLIT, NULL));
-  fi.meta = utc->meta;
-  fi.uri = utc->uri;
-  GNUNET_URITRACK_track (ectx, utc->shared->ctx->cfg, &fi);
-  GNUNET_URITRACK_add_state (ectx,
-                             utc->shared->ctx->cfg,
-                             utc->uri,
-                             utc->shared->doIndex ==
-                             GNUNET_YES ? GNUNET_URITRACK_INDEXED :
-                             GNUNET_URITRACK_INSERTED);
   event.type = GNUNET_FSUI_upload_completed;
   event.data.UploadCompleted.uc.pos = utc;
   event.data.UploadCompleted.uc.cctx = utc->cctx;
@@ -526,9 +578,15 @@ freeUploadList (struct GNUNET_FSUI_UploadList *ul)
   if (ul->keywords != NULL)
     GNUNET_ECRS_uri_destroy (ul->keywords);
   if (ul->uri != NULL)
-    GNUNET_ECRS_uri_destroy (ul->uri);
+    {
+      GNUNET_ECRS_uri_destroy (ul->uri);
+      ul->uri = NULL;
+    }
   if (ul->meta != NULL)
-    GNUNET_ECRS_meta_data_destroy (ul->meta);
+    {
+      GNUNET_meta_data_destroy (ul->meta);
+      ul->meta = NULL;
+    }
   /* unlink from parent */
   next = ul->parent->child;
   if (next == NULL)
@@ -566,7 +624,7 @@ static struct GNUNET_FSUI_UploadList *addUploads (struct
                                                   const struct GNUNET_ECRS_URI
                                                   *keywords,
                                                   const struct
-                                                  GNUNET_ECRS_MetaData *md,
+                                                  GNUNET_MetaData *md,
                                                   struct
                                                   GNUNET_FSUI_UploadList
                                                   *parent);
@@ -577,16 +635,17 @@ addChildUpload (const char *name, const char *dirName, void *data)
   struct GNUNET_FSUI_UploadList *parent = data;
   char *filename;
   struct GNUNET_FSUI_UploadList *child;
-  struct GNUNET_ECRS_MetaData *md;
+  struct GNUNET_MetaData *md_tmp;
 
   filename = GNUNET_malloc (strlen (dirName) + strlen (name) + 2);
   strcpy (filename, dirName);
-  strcat (filename, DIR_SEPARATOR_STR);
+  if (dirName[strlen (dirName) - 1] != DIR_SEPARATOR)
+    strcat (filename, DIR_SEPARATOR_STR);
   strcat (filename, name);
-  md = GNUNET_ECRS_meta_data_create ();
-  child = addUploads (parent->shared, filename, NULL, md, parent);
+  md_tmp = GNUNET_meta_data_create ();
+  child = addUploads (parent->shared, filename, NULL, md_tmp, parent);
   GNUNET_free (filename);
-  GNUNET_ECRS_meta_data_destroy (md);
+  GNUNET_meta_data_destroy (md_tmp);
   if (child == NULL)
     return GNUNET_SYSERR;
   parent->total += child->total;
@@ -597,7 +656,7 @@ static struct GNUNET_FSUI_UploadList *
 addUploads (struct GNUNET_FSUI_UploadShared *shared,
             const char *filename,
             const struct GNUNET_ECRS_URI *keywords,
-            const struct GNUNET_ECRS_MetaData *md,
+            const struct GNUNET_MetaData *md,
             struct GNUNET_FSUI_UploadList *parent)
 {
   GNUNET_FSUI_UploadList *utc;
@@ -615,6 +674,7 @@ addUploads (struct GNUNET_FSUI_UploadShared *shared,
   utc->state = GNUNET_FSUI_ACTIVE;
   if (GNUNET_YES == GNUNET_disk_file_test (shared->ctx->ectx, filename))
     {
+      utc->is_directory = GNUNET_NO;
       /* add this file */
       if (GNUNET_OK != GNUNET_disk_file_size (shared->ctx->ectx,
                                               filename, &utc->total,
@@ -623,10 +683,13 @@ addUploads (struct GNUNET_FSUI_UploadShared *shared,
           GNUNET_free (utc);
           return NULL;
         }
-      utc->meta = GNUNET_ECRS_meta_data_duplicate (md);
+      utc->meta =
+        (md ==
+         NULL) ? GNUNET_meta_data_create () : GNUNET_meta_data_duplicate (md);
     }
   else
     {
+      utc->is_directory = GNUNET_YES;
       if (GNUNET_SYSERR == shared->dsc (shared->dscClosure,
                                         filename, &addChildUpload, utc))
         {
@@ -636,10 +699,9 @@ addUploads (struct GNUNET_FSUI_UploadShared *shared,
           GNUNET_free (utc);
           return NULL;
         }
-      utc->meta = GNUNET_ECRS_meta_data_duplicate (md);
-      GNUNET_ECRS_meta_data_insert (utc->meta,
-                                    EXTRACTOR_MIMETYPE,
-                                    GNUNET_DIRECTORY_MIME);
+      utc->meta = GNUNET_meta_data_duplicate (md);
+      GNUNET_meta_data_insert (utc->meta,
+                               EXTRACTOR_MIMETYPE, GNUNET_DIRECTORY_MIME);
     }
   if (keywords != NULL)
     utc->keywords = GNUNET_ECRS_uri_duplicate (keywords);
@@ -682,6 +744,7 @@ freeShared (struct GNUNET_FSUI_UploadShared *shared)
     GNUNET_ECRS_uri_destroy (shared->global_keywords);
   EXTRACTOR_removeAll (shared->extractors);
   GNUNET_free_non_null (shared->extractor_config);
+  GNUNET_free (shared->top_filename);
   GNUNET_free (shared);
 }
 
@@ -706,7 +769,7 @@ GNUNET_FSUI_upload_start (struct GNUNET_FSUI_Context *ctx,
                           int doExtract,
                           int individualKeywords,
                           GNUNET_CronTime expiration,
-                          const struct GNUNET_ECRS_MetaData *md,
+                          const struct GNUNET_MetaData *md,
                           const struct GNUNET_ECRS_URI *globalURI,
                           const struct GNUNET_ECRS_URI *keyUri)
 {
@@ -746,7 +809,7 @@ GNUNET_FSUI_upload_start (struct GNUNET_FSUI_Context *ctx,
   shared->anonymityLevel = anonymityLevel;
   shared->priority = priority;
   shared->individualKeywords = individualKeywords;
-  shared->handle = NULL;
+  shared->top_filename = GNUNET_strdup (filename);
   ul = addUploads (shared, filename, keyUri, md, &ctx->activeUploads);
   if (ul == NULL)
     {
@@ -778,12 +841,16 @@ GNUNET_FSUI_upload_start (struct GNUNET_FSUI_Context *ctx,
  * @return GNUNET_SYSERR on error
  */
 int
-GNUNET_FSUI_upload_abort (struct GNUNET_FSUI_Context *ctx,
-                          struct GNUNET_FSUI_UploadList *ul)
+GNUNET_FSUI_upload_abort (struct GNUNET_FSUI_UploadList *ul)
 {
   GNUNET_FSUI_UploadList *c;
+  GNUNET_FSUI_UploadList *p;
+  struct GNUNET_FSUI_Context *ctx;
+  GNUNET_FSUI_Event event;
 
-  GNUNET_GE_ASSERT (ctx->ectx, ul != NULL);
+  if (ul == NULL)
+    return GNUNET_SYSERR;
+  ctx = ul->shared->ctx;
   if ((ul->state != GNUNET_FSUI_ACTIVE) && (ul->state != GNUNET_FSUI_PENDING))
     return GNUNET_NO;
   if (ul->state == GNUNET_FSUI_ACTIVE)
@@ -792,10 +859,16 @@ GNUNET_FSUI_upload_abort (struct GNUNET_FSUI_Context *ctx,
       c = ul->child;
       while (c != NULL)
         {
-          GNUNET_FSUI_upload_abort (ctx, c);
+          GNUNET_FSUI_upload_abort (c);
           c = c->next;
         }
       GNUNET_thread_stop_sleep (ul->shared->handle);
+      event.type = GNUNET_FSUI_upload_aborted;
+      event.data.UploadAborted.uc.pos = ul;
+      event.data.UploadAborted.uc.cctx = ul->cctx;
+      event.data.UploadAborted.uc.ppos = ul->parent;
+      event.data.UploadAborted.uc.pcctx = ul->parent->cctx;
+      ctx->ecb (ctx->ecbClosure, &event);
     }
   else
     {
@@ -803,9 +876,37 @@ GNUNET_FSUI_upload_abort (struct GNUNET_FSUI_Context *ctx,
       c = ul->child;
       while (c != NULL)
         {
-          GNUNET_FSUI_upload_abort (ctx, c);
+          GNUNET_FSUI_upload_abort (c);
           c = c->next;
         }
+      event.type = GNUNET_FSUI_upload_aborted;
+      event.data.UploadAborted.uc.pos = ul;
+      event.data.UploadAborted.uc.cctx = ul->cctx;
+      event.data.UploadAborted.uc.ppos = ul->parent;
+      event.data.UploadAborted.uc.pcctx = ul->parent->cctx;
+      ctx->ecb (ctx->ecbClosure, &event);
+    }
+  if (!ul->is_directory)
+    {
+      /* reduce total size of all parents accordingly
+         and generate progress events */
+      p = ul->parent;
+      while (p != &ctx->activeUploads)
+        {
+          p->total -= ul->total;
+          event.type = GNUNET_FSUI_upload_progress;
+          event.data.UploadProgress.uc.pos = p;
+          event.data.UploadProgress.uc.cctx = p->cctx;
+          event.data.UploadProgress.uc.ppos = p->parent;
+          event.data.UploadProgress.uc.pcctx = p->parent->cctx;
+          event.data.UploadProgress.completed = p->completed;
+          event.data.UploadProgress.total = p->total;
+          /* use "now" for ETA, given that the user is aborting stuff */
+          event.data.UploadProgress.eta = GNUNET_get_time ();
+          event.data.UploadProgress.filename = p->filename;
+          ctx->ecb (ctx->ecbClosure, &event);
+          p = p->parent;
+        }
     }
   return GNUNET_OK;
 }
@@ -817,13 +918,15 @@ GNUNET_FSUI_upload_abort (struct GNUNET_FSUI_Context *ctx,
  * @return GNUNET_SYSERR on error
  */
 int
-GNUNET_FSUI_upload_stop (struct GNUNET_FSUI_Context *ctx,
-                         struct GNUNET_FSUI_UploadList *ul)
+GNUNET_FSUI_upload_stop (struct GNUNET_FSUI_UploadList *ul)
 {
   void *unused;
   struct GNUNET_FSUI_UploadShared *shared;
+  struct GNUNET_FSUI_Context *ctx;
 
-  GNUNET_GE_ASSERT (ctx->ectx, ul != NULL);
+  if (ul == NULL)
+    return GNUNET_SYSERR;
+  ctx = ul->shared->ctx;
   GNUNET_GE_ASSERT (ctx->ectx, ul->parent == &ctx->activeUploads);
   if ((ul->state == GNUNET_FSUI_ACTIVE) ||
       (ul->state == GNUNET_FSUI_COMPLETED) ||
