@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet
-     (C) 2004, 2005, 2006, 2007 Christian Grothoff (and other contributing authors)
+     (C) 2004, 2005, 2006, 2007, 2008 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -32,47 +32,135 @@
 
 #define DEBUG_FSLIB GNUNET_NO
 
-typedef struct GNUNET_FS_SearchHandle
-{
-  CS_fs_request_search_MESSAGE *req;
-  GNUNET_DatastoreValueIterator callback;
-  void *closure;
-} SEARCH_HANDLE;
-
-typedef struct GNUNET_FS_SearchContext
-{
-  struct GNUNET_GC_Configuration *cfg;
-  struct GNUNET_GE_Context *ectx;
-  struct GNUNET_ClientServerConnection *sock;
-  struct GNUNET_ThreadHandle *thread;
-  struct GNUNET_Mutex *lock;
-  SEARCH_HANDLE **handles;
-  unsigned int handleCount;
-  unsigned int handleSize;
-  int abort;
-} SEARCH_CONTEXT;
+/**
+ * How often should we automatically retry a request
+ * that failed? (Note that searches are retried
+ * indefinitely in any case; this only applies
+ * to upload/delete operations).
+ */
+#define AUTO_RETRY 5
 
 /**
- * How often should we automatically retry if we
- * get a transient error back from gnunetd?
+ * In memory, the search handle is followed
+ * by a copy of the corresponding request of
+ * type "CS_fs_request_search_MESSAGE *".
  */
-#define AUTO_RETRY 4
+struct GNUNET_FS_SearchHandle
+{
+  /**
+   * This is a linked list.
+   */
+  struct GNUNET_FS_SearchHandle *next;
+
+  /**
+   * Function to call with results.
+   */
+  GNUNET_DatastoreValueIterator callback;
+
+  /**
+   * Extra argument to pass to callback.
+   */
+  void *closure;
+};
+
+/**
+ * Context for a set of search operations.
+ */
+struct GNUNET_FS_SearchContext
+{
+  /**
+   * Configuration data.
+   */
+  struct GNUNET_GC_Configuration *cfg;
+
+  /**
+   * Error logging.
+   */
+  struct GNUNET_GE_Context *ectx;
+
+  /**
+   * Connection to gnunetd.
+   */
+  struct GNUNET_ClientServerConnection *sock;
+
+  /**
+   * Thread listening for replies.
+   */
+  struct GNUNET_ThreadHandle *thread;
+
+  /**
+   * Lock for access to this struct.
+   */
+  struct GNUNET_Mutex *lock;
+
+  /**
+   * List of active requests.
+   */
+  struct GNUNET_FS_SearchHandle *handles;
+
+  /**
+   * Flag to signal that we should abort.
+   */
+  int abort;
+
+  /**
+   * Counter for how many times this context has
+   * been suspended.  Results will not be passed
+   * on until the counter is zero.
+   */
+  unsigned int block_results;
+
+#if DEBUG_FSLIB
+  unsigned int total_received;
+
+  unsigned int total_requested;
+#endif
+};
+
+/**
+ * Retransmit all of the requests to gnunetd
+ * (used after a disconnect).
+ */
+static int
+reissue_requests (struct GNUNET_FS_SearchContext *ctx)
+{
+  const CS_fs_request_search_MESSAGE *req;
+  struct GNUNET_FS_SearchHandle *pos;
+
+  pos = ctx->handles;
+  while (pos != NULL)
+    {
+      req = (const CS_fs_request_search_MESSAGE *) &pos[1];
+      if (GNUNET_OK !=
+          GNUNET_client_connection_write (ctx->sock, &req->header))
+        return GNUNET_SYSERR;
+      pos = pos->next;
+    }
+  if (GNUNET_SYSERR == GNUNET_client_connection_ensure_connected (ctx->sock))
+    return GNUNET_SYSERR;
+  return GNUNET_OK;
+}
 
 /**
  * Thread that processes replies from gnunetd and
  * calls the appropriate callback.
  */
 static void *
-processReplies (void *cls)
+reply_process_thread (void *cls)
 {
-  SEARCH_CONTEXT *ctx = cls;
+  struct GNUNET_FS_SearchContext *ctx = cls;
   GNUNET_MessageHeader *hdr;
-  int i;
   int matched;
   const CS_fs_reply_content_MESSAGE *rep;
   GNUNET_HashCode query;
   unsigned int size;
   GNUNET_CronTime delay;
+  const CS_fs_request_search_MESSAGE *req;
+  GNUNET_DatastoreValue *value;
+  struct GNUNET_FS_SearchHandle *pos;
+  struct GNUNET_FS_SearchHandle *spos;
+  struct GNUNET_FS_SearchHandle *prev;
+  int unique;
 
   delay = 100 * GNUNET_CRON_MILLISECONDS;
   while (ctx->abort == GNUNET_NO)
@@ -81,9 +169,7 @@ processReplies (void *cls)
       if (GNUNET_OK == GNUNET_client_connection_read (ctx->sock, &hdr))
         {
 #if DEBUG_FSLIB
-          GNUNET_GE_LOG (ctx->ectx,
-                         GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                         "FSLIB: received message from gnunetd\n");
+          fprintf (stderr, "FSLIB: received message from gnunetd\n");
 #endif
           delay = 100 * GNUNET_CRON_MILLISECONDS;
           /* verify hdr, if reply, process, otherwise
@@ -98,102 +184,127 @@ processReplies (void *cls)
             }
           rep = (const CS_fs_reply_content_MESSAGE *) hdr;
           size = ntohs (hdr->size) - sizeof (CS_fs_reply_content_MESSAGE);
-          if (GNUNET_OK != GNUNET_EC_file_block_check_and_get_query (size, (DBlock *) & rep[1], GNUNET_NO,      /* gnunetd will have checked already */
+          if (GNUNET_OK != GNUNET_EC_file_block_check_and_get_query (size, (GNUNET_EC_DBlock *) & rep[1], GNUNET_NO,    /* gnunetd will have checked already */
                                                                      &query))
             {
               GNUNET_GE_BREAK (ctx->ectx, 0);
               GNUNET_free (hdr);
               continue;
             }
+          unique =
+            GNUNET_EC_file_block_get_type (size,
+                                           (GNUNET_EC_DBlock *) & rep[1]) ==
+            GNUNET_ECRS_BLOCKTYPE_DATA;
+          value = GNUNET_malloc (sizeof (GNUNET_DatastoreValue) + size);
+          value->size = htonl (size + sizeof (GNUNET_DatastoreValue));
+          value->type =
+            htonl (GNUNET_EC_file_block_get_type
+                   (size, (GNUNET_EC_DBlock *) & rep[1]));
+          value->priority = htonl (0);
+          value->anonymity_level = rep->anonymity_level;
+          value->expiration_time = rep->expiration_time;
+          memcpy (&value[1], &rep[1], size);
           matched = 0;
           GNUNET_mutex_lock (ctx->lock);
-          for (i = ctx->handleCount - 1; i >= 0; i--)
+          while (ctx->block_results > 0)
             {
+              GNUNET_mutex_unlock (ctx->lock);
+              GNUNET_thread_sleep (100 * GNUNET_CRON_MILLISECONDS);
+              GNUNET_mutex_lock (ctx->lock);
+            }
+          prev = NULL;
+          pos = ctx->handles;
+          while (pos != NULL)
+            {
+              req = (const CS_fs_request_search_MESSAGE *) &pos[1];
               if (0 ==
-                  memcmp (&query, &ctx->handles[i]->req->query[0],
-                          sizeof (GNUNET_HashCode)))
+                  memcmp (&query, &req->query[0], sizeof (GNUNET_HashCode)))
                 {
-                  GNUNET_DatastoreValue *value;
-
                   matched++;
-                  if (ctx->handles[i]->callback != NULL)
+                  spos = pos;
+                  if (unique)
                     {
-                      value =
-                        GNUNET_malloc (sizeof (GNUNET_DatastoreValue) + size);
-                      value->size =
-                        htonl (size + sizeof (GNUNET_DatastoreValue));
-                      value->type =
-                        htonl (GNUNET_EC_file_block_get_type
-                               (size, (DBlock *) & rep[1]));
-                      value->prio = htonl (0);
-                      value->anonymityLevel = rep->anonymityLevel;
-                      value->expirationTime = rep->expirationTime;
-                      memcpy (&value[1], &rep[1], size);
-                      if (GNUNET_SYSERR == ctx->handles[i]->callback (&query,
-                                                                      value,
-                                                                      ctx->
-                                                                      handles
-                                                                      [i]->
-                                                                      closure,
-                                                                      0))
-                        {
-                          ctx->handles[i]->callback = NULL;
-                        }
-                      GNUNET_free (value);
+                      if (prev == NULL)
+                        ctx->handles = pos->next;
+                      else
+                        prev->next = pos->next;
+                      if (prev == NULL)
+                        pos = ctx->handles;
+                      else
+                        pos = prev->next;
                     }
+                  else
+                    {
+                      prev = pos;
+                      pos = pos->next;
+                    }
+#if DEBUG_FSLIB
+                  fprintf (stderr,
+                           "FSLIB passes response %u to client (%d)\n",
+                           ctx->total_received++, unique);
+#endif
+                  if ((spos->callback != NULL) &&
+                      (GNUNET_SYSERR == spos->callback (&query,
+                                                        value,
+                                                        spos->closure, 0)))
+                    spos->callback = NULL;
+                  if (unique)
+                    GNUNET_free (spos);
+                }
+              else
+                {
+                  prev = pos;
+                  pos = pos->next;
                 }
             }
-          GNUNET_mutex_unlock (ctx->lock);
+          GNUNET_free (value);
 #if DEBUG_FSLIB
           if (matched == 0)
-            GNUNET_GE_LOG (ctx->ectx,
-                           GNUNET_GE_DEBUG | GNUNET_GE_REQUEST |
-                           GNUNET_GE_USER,
-                           "FSLIB: received content but have no pending request\n");
+            fprintf (stderr,
+                     "FSLIB: received content but have no pending request\n");
 #endif
+          GNUNET_mutex_unlock (ctx->lock);
         }
       else
         {
-#if DEBUG_FSLIB
-          GNUNET_GE_LOG (ctx->ectx,
-                         GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                         "FSLIB: error communicating with gnunetd; sleeping for %ums\n",
-                         delay);
-#endif
-          if (ctx->abort == GNUNET_NO)
-            GNUNET_thread_sleep (delay);
-          delay *= 2;
-          if (delay > 5 * GNUNET_CRON_SECONDS)
-            delay = 5 * GNUNET_CRON_SECONDS;
+          while (GNUNET_NO == ctx->abort)
+            {
+              GNUNET_thread_sleep (delay);
+              delay *= 2;
+              if (delay > 60 * GNUNET_CRON_SECONDS)
+                delay = 60 * GNUNET_CRON_SECONDS;
+              if ((GNUNET_OK ==
+                   GNUNET_client_connection_ensure_connected (ctx->sock))
+                  && (GNUNET_OK == reissue_requests (ctx)))
+                break;          /* we're back, continue outer loop! */
+            }
         }
       GNUNET_free_non_null (hdr);
     }
   return NULL;
 }
 
-SEARCH_CONTEXT *
-GNUNET_FS_create_search_context (struct GNUNET_GE_Context * ectx,
-                                 struct GNUNET_GC_Configuration * cfg,
-                                 struct GNUNET_Mutex * lock)
+struct GNUNET_FS_SearchContext *
+GNUNET_FS_create_search_context (struct GNUNET_GE_Context *ectx,
+                                 struct GNUNET_GC_Configuration *cfg)
 {
-  SEARCH_CONTEXT *ret;
+  struct GNUNET_FS_SearchContext *ret;
 
-  GNUNET_GE_ASSERT (ectx, lock != NULL);
-  ret = GNUNET_malloc (sizeof (SEARCH_CONTEXT));
+  ret = GNUNET_malloc (sizeof (struct GNUNET_FS_SearchContext));
+  memset (ret, 0, sizeof (struct GNUNET_FS_SearchContext));
   ret->ectx = ectx;
   ret->cfg = cfg;
-  ret->lock = lock;
+  ret->lock = GNUNET_mutex_create (GNUNET_YES);
   ret->sock = GNUNET_client_connection_create (ectx, cfg);
   if (ret->sock == NULL)
     {
+      GNUNET_mutex_destroy (ret->lock);
       GNUNET_free (ret);
       return NULL;
     }
   ret->handles = NULL;
-  ret->handleCount = 0;
-  ret->handleSize = 0;
   ret->abort = GNUNET_NO;
-  ret->thread = GNUNET_thread_create (&processReplies, ret, 128 * 1024);
+  ret->thread = GNUNET_thread_create (&reply_process_thread, ret, 128 * 1024);
   if (ret->thread == NULL)
     GNUNET_GE_DIE_STRERROR (ectx,
                             GNUNET_GE_FATAL | GNUNET_GE_ADMIN |
@@ -201,21 +312,44 @@ GNUNET_FS_create_search_context (struct GNUNET_GE_Context * ectx,
   return ret;
 }
 
+
+/**
+ * Resume the search context (start sending results again).
+ */
+void
+GNUNET_FS_resume_search_context (struct GNUNET_FS_SearchContext *ctx)
+{
+  ctx->block_results--;
+  GNUNET_thread_stop_sleep (ctx->thread);
+}
+
+void
+GNUNET_FS_suspend_search_context (struct GNUNET_FS_SearchContext *ctx)
+{
+  GNUNET_mutex_lock (ctx->lock);
+  ctx->block_results++;
+  GNUNET_mutex_unlock (ctx->lock);
+}
+
+
 void
 GNUNET_FS_destroy_search_context (struct GNUNET_FS_SearchContext *ctx)
 {
   void *unused;
+  struct GNUNET_FS_SearchHandle *pos;
 
-  GNUNET_mutex_lock (ctx->lock);
-  GNUNET_GE_ASSERT (ctx->ectx, ctx->handleCount == 0);
   ctx->abort = GNUNET_YES;
   GNUNET_client_connection_close_forever (ctx->sock);
-  GNUNET_mutex_unlock (ctx->lock);
   GNUNET_thread_stop_sleep (ctx->thread);
   GNUNET_thread_join (ctx->thread, &unused);
-  ctx->lock = NULL;
   GNUNET_client_connection_destroy (ctx->sock);
-  GNUNET_array_grow (ctx->handles, ctx->handleSize, 0);
+  while (ctx->handles != NULL)
+    {
+      pos = ctx->handles;
+      ctx->handles = pos->next;
+      GNUNET_free (pos);
+    }
+  GNUNET_mutex_destroy (ctx->lock);
   GNUNET_free (ctx);
 }
 
@@ -223,145 +357,109 @@ GNUNET_FS_destroy_search_context (struct GNUNET_FS_SearchContext *ctx)
  * Search for blocks matching the given key and type.
  *
  * @param timeout how long to search
- * @param anonymityLevel what are the anonymity
+ * @param anonymity_level what are the anonymity
  *        requirements for this request? 0 for no
  *        anonymity (DHT/direct transfer ok)
  * @param callback method to call for each result
- * @param prio priority to use for the search
+ * @param priority priority to use for the search
  */
-SEARCH_HANDLE *
-GNUNET_FS_start_search (SEARCH_CONTEXT * ctx,
+int
+GNUNET_FS_start_search (struct GNUNET_FS_SearchContext *ctx,
                         const GNUNET_PeerIdentity * target,
                         unsigned int type,
                         unsigned int keyCount,
                         const GNUNET_HashCode * keys,
                         unsigned int anonymityLevel,
-                        unsigned int prio,
-                        GNUNET_CronTime timeout,
                         GNUNET_DatastoreValueIterator callback, void *closure)
 {
-  SEARCH_HANDLE *ret;
+  struct GNUNET_FS_SearchHandle *ret;
   CS_fs_request_search_MESSAGE *req;
 #if DEBUG_FSLIB
   GNUNET_EncName enc;
 #endif
 
-  ret = GNUNET_malloc (sizeof (SEARCH_HANDLE));
+  ret = GNUNET_malloc (sizeof (struct GNUNET_FS_SearchHandle) +
+                       sizeof (CS_fs_request_search_MESSAGE) +
+                       (keyCount - 1) * sizeof (GNUNET_HashCode));
+  req = (CS_fs_request_search_MESSAGE *) & ret[1];
 #if DEBUG_FSLIB
-  GNUNET_GE_LOG (ctx->ectx,
-                 GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                 "FSLIB: start search (%p)\n", ret);
+  GNUNET_hash_to_enc (keys, &enc);
+  fprintf (stderr, "FSLIB: start search for `%s' (%p)\n", (char *) &enc, ret);
 #endif
-  req =
-    GNUNET_malloc (sizeof (CS_fs_request_search_MESSAGE) +
-                   (keyCount - 1) * sizeof (GNUNET_HashCode));
   req->header.size =
     htons (sizeof (CS_fs_request_search_MESSAGE) +
            (keyCount - 1) * sizeof (GNUNET_HashCode));
   req->header.type = htons (GNUNET_CS_PROTO_GAP_QUERY_START);
-  req->prio = htonl (prio);
-  req->anonymityLevel = htonl (anonymityLevel);
-  req->expiration = GNUNET_htonll (timeout);
+  req->anonymity_level = htonl (anonymityLevel);
   req->type = htonl (type);
   if (target != NULL)
     req->target = *target;
   else
     memset (&req->target, 0, sizeof (GNUNET_PeerIdentity));
   memcpy (&req->query[0], keys, keyCount * sizeof (GNUNET_HashCode));
-  ret->req = req;
   ret->callback = callback;
   ret->closure = closure;
   GNUNET_mutex_lock (ctx->lock);
-  if (ctx->handleCount == ctx->handleSize)
-    {
-      GNUNET_array_grow (ctx->handles, ctx->handleSize,
-                         ctx->handleSize * 2 + 4);
-    }
-  ctx->handles[ctx->handleCount++] = ret;
-  GNUNET_mutex_unlock (ctx->lock);
+  ret->next = ctx->handles;
+  ctx->handles = ret;
 #if DEBUG_FSLIB
-  IF_GELOG (ctx->ectx,
-            GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-            GNUNET_hash_to_enc (&req->query[0], &enc));
-  GNUNET_GE_LOG (ctx->ectx,
-                 GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                 "FSLIB: initiating search for `%s' of type %u\n", &enc,
-                 type);
+  fprintf (stderr,
+           "FSLIB passes request %u to daemon (%d)\n",
+           ctx->total_requested++, type);
 #endif
-  GNUNET_GE_ASSERT (NULL, ctx->sock != NULL);
   if (GNUNET_OK != GNUNET_client_connection_write (ctx->sock, &req->header))
-    {
-      GNUNET_FS_stop_search (ctx, ret);
-      return NULL;
-    }
-#if DEBUG_FSLIB
-  GNUNET_GE_LOG (ctx->ectx,
-                 GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                 "FSLIB: search started (%p)\n", ret);
-#endif
-  return ret;
-}
-
-/**
- * Stop searching.
- */
-void
-GNUNET_FS_stop_search (SEARCH_CONTEXT * ctx, SEARCH_HANDLE * handle)
-{
-  int i;
-
-#if DEBUG_FSLIB
-  GNUNET_GE_LOG (ctx->ectx,
-                 GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                 "FSLIB: stop search (%p)\n", handle);
-#endif
-  handle->req->header.type = htons (GNUNET_CS_PROTO_GAP_QUERY_STOP);
-  GNUNET_GE_ASSERT (NULL, ctx->sock != NULL);
-  if (GNUNET_OK !=
-      GNUNET_client_connection_write (ctx->sock, &handle->req->header))
-    {
-      GNUNET_GE_LOG (ctx->ectx,
-                     GNUNET_GE_WARNING | GNUNET_GE_REQUEST |
-                     GNUNET_GE_DEVELOPER,
-                     "FSLIB: failed to request stop search with gnunetd\n");
-    }
-  GNUNET_mutex_lock (ctx->lock);
-  for (i = ctx->handleCount - 1; i >= 0; i--)
-    if (ctx->handles[i] == handle)
-      {
-        ctx->handles[i] = ctx->handles[--ctx->handleCount];
-        break;
-      }
+    GNUNET_client_connection_close_temporarily (ctx->sock);
   GNUNET_mutex_unlock (ctx->lock);
-  GNUNET_free (handle->req);
-#if DEBUG_FSLIB
-  GNUNET_GE_LOG (ctx->ectx,
-                 GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                 "FSLIB: search stopped (%p)\n", handle);
-#endif
-  GNUNET_free (handle);
+  return GNUNET_OK;
 }
 
+
 /**
- * What is the current average priority of entries
- * in the routing table like?  Returns -1 on error.
+ * Stop searching for blocks matching the given key and type.
+ *
+ * @param callback method to call for each result
+ * @return GNUNET_OK (or GNUNET_SYSERR if this search
+ *   was never started for this context)
  */
 int
-GNUNET_FS_get_current_average_priority (struct GNUNET_ClientServerConnection
-                                        *sock)
+GNUNET_FS_stop_search (struct
+                       GNUNET_FS_SearchContext
+                       *ctx,
+                       GNUNET_DatastoreValueIterator callback, void *closure)
 {
-  GNUNET_MessageHeader req;
-  int ret;
+  struct GNUNET_FS_SearchHandle *pos;
+  struct GNUNET_FS_SearchHandle *prev;
+  CS_fs_request_search_MESSAGE *req;
 
-  req.size = htons (sizeof (GNUNET_MessageHeader));
-  req.type = htons (GNUNET_CS_PROTO_GAP_GET_AVG_PRIORITY);
-  if (GNUNET_OK != GNUNET_client_connection_write (sock, &req))
-    return -1;
-  if (GNUNET_OK != GNUNET_client_connection_read_result (sock, &ret))
-    return -1;
-  return ret;
+  prev = NULL;
+  GNUNET_mutex_lock (ctx->lock);
+  pos = ctx->handles;
+  while ((pos != NULL) &&
+         ((pos->callback != callback) || (pos->closure != closure)))
+    {
+      prev = pos;
+      pos = pos->next;
+    }
+  if (pos != NULL)
+    {
+      if (prev == NULL)
+        ctx->handles = pos->next;
+      else
+        prev->next = pos->next;
+      /* TODO: consider sending "stop" message
+         to gnunetd? */
+      req = (CS_fs_request_search_MESSAGE *) & pos[1];
+      req->header.type = htons (GNUNET_CS_PROTO_GAP_QUERY_STOP);
+      if (GNUNET_OK != GNUNET_client_connection_write (ctx->sock,
+                                                       &req->header))
+        GNUNET_client_connection_close_temporarily (ctx->sock);
+      GNUNET_free (pos);
+    }
+  GNUNET_mutex_unlock (ctx->lock);
+  return GNUNET_SYSERR;
 }
 
+
 /**
  * Insert a block.
  *
@@ -386,9 +484,9 @@ GNUNET_FS_insert (struct GNUNET_ClientServerConnection *sock,
   ri = GNUNET_malloc (sizeof (CS_fs_request_insert_MESSAGE) + size);
   ri->header.size = htons (sizeof (CS_fs_request_insert_MESSAGE) + size);
   ri->header.type = htons (GNUNET_CS_PROTO_GAP_INSERT);
-  ri->prio = block->prio;
-  ri->expiration = block->expirationTime;
-  ri->anonymityLevel = block->anonymityLevel;
+  ri->priority = block->priority;
+  ri->expiration = block->expiration_time;
+  ri->anonymity_level = block->anonymity_level;
   memcpy (&ri[1], &block[1], size);
   retry = AUTO_RETRY;
   do
@@ -400,7 +498,7 @@ GNUNET_FS_insert (struct GNUNET_ClientServerConnection *sock,
         }
       if (GNUNET_OK != GNUNET_client_connection_read_result (sock, &ret))
         {
-          GNUNET_GE_BREAK (NULL, 0);
+          GNUNET_GE_BREAK (NULL, GNUNET_shutdown_test ());
           GNUNET_free (ri);
           return GNUNET_SYSERR;
         }
@@ -435,9 +533,7 @@ GNUNET_FS_prepare_to_index (struct GNUNET_ClientServerConnection *sock,
   memcpy (&ri[1], fn, strlen (fn));
 
 #if DEBUG_FSLIB
-  GNUNET_GE_LOG (ectx,
-                 GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                 "Sending index initialization request to gnunetd\n");
+  fprintf (stderr, "Sending index initialization request to gnunetd\n");
 #endif
   if (GNUNET_OK != GNUNET_client_connection_write (sock, &ri->header))
     {
@@ -446,9 +542,8 @@ GNUNET_FS_prepare_to_index (struct GNUNET_ClientServerConnection *sock,
     }
   GNUNET_free (ri);
 #if DEBUG_FSLIB
-  GNUNET_GE_LOG (ectx,
-                 GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                 "Waiting for confirmation of index initialization request by gnunetd\n");
+  fprintf (stderr,
+           "Waiting for confirmation of index initialization request by gnunetd\n");
 #endif
   if (GNUNET_OK != GNUNET_client_connection_read_result (sock, &ret))
     return GNUNET_SYSERR;
@@ -473,21 +568,28 @@ GNUNET_FS_index (struct GNUNET_ClientServerConnection *sock,
   CS_fs_request_index_MESSAGE *ri;
   unsigned int size;
   int retry;
+#if DEBUG_FSLIB
+  GNUNET_HashCode hc;
+  GNUNET_EncName enc;
+#endif
 
   size = ntohl (block->size) - sizeof (GNUNET_DatastoreValue);
   ri = GNUNET_malloc (sizeof (CS_fs_request_index_MESSAGE) + size);
   ri->header.size = htons (sizeof (CS_fs_request_index_MESSAGE) + size);
   ri->header.type = htons (GNUNET_CS_PROTO_GAP_INDEX);
-  ri->prio = block->prio;
-  ri->expiration = block->expirationTime;
-  ri->anonymityLevel = block->anonymityLevel;
+  ri->priority = block->priority;
+  ri->expiration = block->expiration_time;
+  ri->anonymity_level = block->anonymity_level;
   ri->fileId = *fileHc;
   ri->fileOffset = GNUNET_htonll (offset);
   memcpy (&ri[1], &block[1], size);
 #if DEBUG_FSLIB
-  GNUNET_GE_LOG (ectx,
-                 GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                 "Sending index request to gnunetd\n");
+  GNUNET_EC_file_block_get_query ((const GNUNET_EC_DBlock *) &block[1], size,
+                                  &hc);
+  GNUNET_hash_to_enc (&hc, &enc);
+  fprintf (stderr,
+           "Sending index request for `%s' to gnunetd)\n",
+           (const char *) &enc);
 #endif
   retry = AUTO_RETRY;
   do
@@ -498,9 +600,8 @@ GNUNET_FS_index (struct GNUNET_ClientServerConnection *sock,
           return GNUNET_SYSERR;
         }
 #if DEBUG_FSLIB
-      GNUNET_GE_LOG (ectx,
-                     GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                     "Waiting for confirmation of index request by gnunetd\n");
+      fprintf (stderr,
+               "Waiting for confirmation of index request by gnunetd\n");
 #endif
       if (GNUNET_OK != GNUNET_client_connection_read_result (sock, &ret))
         {
@@ -590,10 +691,10 @@ int
 GNUNET_FS_test_indexed (struct GNUNET_ClientServerConnection *sock,
                         const GNUNET_HashCode * hc)
 {
-  RequestTestindex ri;
+  CS_fs_request_test_index_MESSAGE ri;
   int ret;
 
-  ri.header.size = htons (sizeof (RequestTestindex));
+  ri.header.size = htons (sizeof (CS_fs_request_test_index_MESSAGE));
   ri.header.type = htons (GNUNET_CS_PROTO_GAP_TESTINDEX);
   ri.reserved = htonl (0);
   ri.fileId = *hc;
