@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2001, 2002, 2003, 2004, 2005, 2006 Christian Grothoff (and other contributing authors)
+     (C) 2001, 2002, 2003, 2004, 2005, 2006, 2008 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -33,10 +33,6 @@
 #include "gnunet_namespace_lib.h"
 #include "gnunet_util.h"
 
-/* hmm. Man says time.h, but that doesn't yield the
-   prototype.  Strange... */
-extern char *strptime (const char *s, const char *format, struct tm *tm);
-
 static int errorCode;
 
 static struct GNUNET_GC_Configuration *cfg;
@@ -49,32 +45,30 @@ static struct GNUNET_FSUI_UploadList *ul;
 
 static GNUNET_CronTime start_time;
 
+static GNUNET_CronTime expiration = 2 * GNUNET_CRON_YEARS;
+
 /* ************ config options ******** */
 
 static char *cfgFilename = GNUNET_DEFAULT_CLIENT_CONFIG_FILE;
 
-static struct GNUNET_ECRS_MetaData *meta;
+static struct GNUNET_MetaData *meta;
 
 static struct GNUNET_ECRS_URI *topKeywords;
 
 static struct GNUNET_ECRS_URI *gloKeywords;
 
-static struct GNUNET_ECRS_MetaData *meta;
+static struct GNUNET_MetaData *meta;
 
 static unsigned int anonymity = 1;
 
 static unsigned int priority = 365;
 
-static unsigned int interval = 0;
+static char *uri_string;
 
 static char *next_id;
 
 static char *this_id;
 
-static char *prev_id;
-
-static char *creation_time;
-
 static char *pseudonym;
 
 static int do_insert;
@@ -83,19 +77,12 @@ static int do_no_direct_references;
 
 static int do_copy;
 
-static int is_sporadic;
+static int do_simulate;
 
 static int extract_only;
 
 static int do_disable_creation_time;
 
-static void
-convertId (const char *s, GNUNET_HashCode * id)
-{
-  if ((s != NULL) && (GNUNET_enc_to_hash (s, id) == GNUNET_SYSERR))
-    GNUNET_hash (s, strlen (s), id);
-}
-
 /**
  * We're done with the upload of the file, do the
  * post-processing.
@@ -103,28 +90,24 @@ convertId (const char *s, GNUNET_HashCode * id)
 static void
 postProcess (const struct GNUNET_ECRS_URI *uri)
 {
-  GNUNET_HashCode prevId;
-  GNUNET_HashCode thisId;
-  GNUNET_HashCode nextId;
+  GNUNET_HashCode nsid;
   struct GNUNET_ECRS_URI *nsuri;
   char *us;
 
   if (pseudonym == NULL)
     return;
-  convertId (next_id, &nextId);
-  convertId (this_id, &thisId);
-  convertId (prev_id, &prevId);
+  if (GNUNET_OK != GNUNET_pseudonym_name_to_id (ectx, cfg, pseudonym, &nsid))
+    {
+      printf (_("\tUnknown namespace `%s'\n"), pseudonym);
+      return;
+    }
   nsuri = GNUNET_NS_add_to_namespace (ectx,
                                       cfg,
                                       anonymity,
                                       priority,
                                       GNUNET_get_time () +
-                                      2 * GNUNET_CRON_YEARS, pseudonym,
-                                      (GNUNET_Int32Time) interval,
-                                      prev_id == NULL ? NULL : &prevId,
-                                      this_id == NULL ? NULL : &thisId,
-                                      next_id == NULL ? NULL : &nextId, uri,
-                                      meta);
+                                      expiration, &nsid,
+                                      this_id, next_id, uri, meta);
   if (nsuri != NULL)
     {
       us = GNUNET_ECRS_uri_to_string (nsuri);
@@ -286,10 +269,6 @@ static struct GNUNET_CommandLineOption gnunetinsertOptions[] = {
    0, &GNUNET_getopt_configure_set_one, &extract_only},
   GNUNET_COMMAND_LINE_OPTION_HELP (gettext_noop ("Make files available to GNUnet for sharing.")),       /* -h */
   GNUNET_COMMAND_LINE_OPTION_HOSTNAME,  /* -H */
-  {'i', "interval", "SECONDS",
-   gettext_noop ("set interval for availability of updates to SECONDS"
-                 " (for namespace insertions only)"),
-   1, &GNUNET_getopt_configure_set_uint, &interval},
   {'k', "key", "KEYWORD",
    gettext_noop
    ("add an additional keyword for the top-level file or directory"
@@ -319,22 +298,18 @@ static struct GNUNET_CommandLineOption gnunetinsertOptions[] = {
    gettext_noop
    ("publish the files under the pseudonym NAME (place file into namespace)"),
    1, &GNUNET_getopt_configure_set_string, &pseudonym},
-  {'S', "sporadic", NULL,
-   gettext_noop ("specifies this as an aperiodic but updated publication"
-                 " (for namespace insertions only)"),
-   0, &GNUNET_getopt_configure_set_one, &is_sporadic},
+  {'s', "simulate-only", NULL,
+   gettext_noop ("only simulte the process but do not do any "
+                 "actual publishing (useful to compute URIs)"),
+   0, &GNUNET_getopt_configure_set_one, &do_simulate},
   {'t', "this", "ID",
    gettext_noop ("set the ID of this version of the publication"
                  " (for namespace insertions only)"),
    1, &GNUNET_getopt_configure_set_string, &this_id},
-  {'T', "time", "TIME",
-   gettext_noop
-   ("specify creation time for SBlock (see man-page for format)"),
-   1, &GNUNET_getopt_configure_set_string, &creation_time},
-  {'u', "update", "ID",
-   gettext_noop ("ID of the previous version of the content"
-                 " (for namespace update only)"),
-   1, &GNUNET_getopt_configure_set_string, &prev_id},
+  {'u', "uri", "URI",
+   gettext_noop ("URI to be published (can be used instead of passing a "
+                 "file to add keywords to the file with the respective URI)"),
+   1, &GNUNET_getopt_configure_set_string, &uri_string},
   GNUNET_COMMAND_LINE_OPTION_VERSION (PACKAGE_VERSION), /* -v */
   GNUNET_COMMAND_LINE_OPTION_VERBOSE,
   GNUNET_COMMAND_LINE_OPTION_END,
@@ -354,8 +329,9 @@ main (int argc, char *const *argv)
   int i;
   char *tmp;
   unsigned long long verbose;
+  GNUNET_HashCode pid;
 
-  meta = GNUNET_ECRS_meta_data_create ();
+  meta = GNUNET_meta_data_create ();
   i = GNUNET_init (argc,
                    argv,
                    "gnunet-insert [OPTIONS] FILENAME",
@@ -365,14 +341,29 @@ main (int argc, char *const *argv)
       errorCode = -1;
       goto quit;
     }
-  if (i != argc - 1)
+  if (((uri_string == NULL) || (extract_only)) && (i != argc - 1))
     {
       printf (_
               ("You must specify one and only one filename for insertion.\n"));
       errorCode = -1;
       goto quit;
     }
-  filename = argv[i];
+  if ((uri_string != NULL) && (i != argc))
+    {
+      printf (_("You must NOT specify an URI and a filename.\n"));
+      errorCode = -1;
+      goto quit;
+    }
+  if ((uri_string != NULL) && (extract_only))
+    {
+      printf (_("Cannot extract metadata from a URI!\n"));
+      errorCode = -1;
+      goto quit;
+    }
+  if (uri_string == NULL)
+    filename = argv[i];
+  else
+    filename = NULL;
 
   if (extract_only)
     {
@@ -401,7 +392,8 @@ main (int argc, char *const *argv)
       listKeywords (fname, dirname, l);
       GNUNET_free (dirname);
       EXTRACTOR_removeAll (l);
-      GNUNET_ECRS_meta_data_destroy (meta);
+      GNUNET_meta_data_destroy (meta);
+      meta = NULL;
 
       errorCode = 0;
       goto quit;
@@ -414,33 +406,23 @@ main (int argc, char *const *argv)
   /* check arguments */
   if (pseudonym != NULL)
     {
-      if (GNUNET_OK !=
-          GNUNET_ECRS_namespace_test_exists (ectx, cfg, pseudonym, NULL))
+      if ((GNUNET_OK !=
+           GNUNET_pseudonym_name_to_id (ectx, cfg,
+                                        pseudonym, &pid)) ||
+          (GNUNET_OK != GNUNET_ECRS_namespace_test_exists (ectx, cfg, &pid)))
         {
           printf (_("Could not access namespace `%s' (does not exist?).\n"),
                   pseudonym);
           errorCode = -1;
           goto quit;
         }
-      if (creation_time != NULL)
+      if (NULL == this_id)
         {
-          struct tm t;
-          const char *fmt;
-
-#if ENABLE_NLS
-          fmt = nl_langinfo (D_T_FMT);
-#else
-          fmt = "%Y-%m-%d";
-#endif
-          if ((NULL == strptime (creation_time, fmt, &t)))
-            {
-              GNUNET_GE_LOG_STRERROR (ectx,
-                                      GNUNET_GE_FATAL | GNUNET_GE_USER |
-                                      GNUNET_GE_IMMEDIATE, "strptime");
-              printf (_("Parsing time failed. Use `%s' format.\n"), fmt);
-              errorCode = -1;
-              goto quit;
-            }
+          fprintf (stderr,
+                   _("Option `%s' is required when using option `%s'.\n"),
+                   "-t", "-P");
+          errorCode = -1;
+          goto quit;
         }
     }
   else
@@ -453,14 +435,6 @@ main (int argc, char *const *argv)
           errorCode = -1;
           goto quit;
         }
-      if (NULL != prev_id)
-        {
-          fprintf (stderr,
-                   _("Option `%s' makes no sense without option `%s'.\n"),
-                   "-u", "-P");
-          errorCode = -1;
-          goto quit;
-        }
       if (NULL != this_id)
         {
           fprintf (stderr,
@@ -469,22 +443,34 @@ main (int argc, char *const *argv)
           errorCode = -1;
           goto quit;
         }
-      if (0 != interval)
+    }
+
+  if (uri_string != NULL)
+    {
+      struct GNUNET_ECRS_URI *us
+        = GNUNET_ECRS_string_to_uri (ectx, uri_string);
+      if (us == NULL)
         {
-          fprintf (stderr,
-                   _("Option `%s' makes no sense without option `%s'.\n"),
-                   "-i", "-P");
-          errorCode = -1;
-          goto quit;
-        }
-      if (is_sporadic)
-        {
-          fprintf (stderr,
-                   _("Option `%s' makes no sense without option `%s'.\n"),
-                   "-S", "-P");
           errorCode = -1;
           goto quit;
         }
+      postProcess (us);
+      if (gloKeywords != NULL)
+        GNUNET_ECRS_publish_under_keyword (ectx,
+                                           cfg,
+                                           gloKeywords,
+                                           anonymity,
+                                           priority,
+                                           start_time + expiration, us, meta);
+      if (topKeywords != NULL)
+        GNUNET_ECRS_publish_under_keyword (ectx,
+                                           cfg,
+                                           topKeywords,
+                                           anonymity,
+                                           priority,
+                                           start_time + expiration, us, meta);
+      GNUNET_ECRS_uri_destroy (us);
+      goto quit;
     }
 
   /* fundamental init */
@@ -494,33 +480,34 @@ main (int argc, char *const *argv)
   /* first insert all of the top-level files or directories */
   tmp = GNUNET_expand_file_name (ectx, filename);
   if (!do_disable_creation_time)
-    GNUNET_ECRS_meta_data_add_publication_date (meta);
+    GNUNET_meta_data_add_publication_date (meta);
   start_time = GNUNET_get_time ();
   errorCode = 1;
   ul = GNUNET_FSUI_upload_start (ctx,
                                  tmp,
                                  (GNUNET_FSUI_DirectoryScanCallback) &
                                  GNUNET_disk_directory_scan, ectx, anonymity,
-                                 priority, !do_insert, GNUNET_YES,
-                                 !do_no_direct_references,
-                                 start_time + 2 * GNUNET_CRON_YEARS, meta,
+                                 priority,
+                                 do_simulate ? GNUNET_SYSERR : (!do_insert),
+                                 GNUNET_YES, !do_no_direct_references,
+                                 start_time + expiration, meta,
                                  gloKeywords, topKeywords);
-  if (gloKeywords != NULL)
-    GNUNET_ECRS_uri_destroy (gloKeywords);
-  if (topKeywords != NULL)
-    GNUNET_ECRS_uri_destroy (topKeywords);
   GNUNET_free (tmp);
   if (ul != NULL)
     {
       GNUNET_shutdown_wait_for ();
       if (errorCode == 1)
-        GNUNET_FSUI_upload_abort (ctx, ul);
-      GNUNET_FSUI_upload_stop (ctx, ul);
+        GNUNET_FSUI_upload_abort (ul);
+      GNUNET_FSUI_upload_stop (ul);
     }
-  GNUNET_ECRS_meta_data_destroy (meta);
   GNUNET_FSUI_stop (ctx);
-
 quit:
+  if (meta != NULL)
+    GNUNET_meta_data_destroy (meta);
+  if (gloKeywords != NULL)
+    GNUNET_ECRS_uri_destroy (gloKeywords);
+  if (topKeywords != NULL)
+    GNUNET_ECRS_uri_destroy (topKeywords);
   GNUNET_fini (ectx, cfg);
   return errorCode;
 }
