@@ -36,10 +36,6 @@ static struct GNUNET_GC_Configuration *cfg;
 
 static unsigned int anonymity = 1;
 
-static unsigned int delay = 300;
-
-static unsigned int max_results;
-
 static char *cfgFilename = GNUNET_DEFAULT_CLIENT_CONFIG_FILE;
 
 static char *output_filename;
@@ -61,9 +57,9 @@ itemPrinter (EXTRACTOR_KeywordType type, const char *data, void *closure)
 }
 
 static void
-printMeta (const struct GNUNET_ECRS_MetaData *meta)
+printMeta (const struct GNUNET_MetaData *meta)
 {
-  GNUNET_ECRS_meta_data_get_contents (meta, &itemPrinter, NULL);
+  GNUNET_meta_data_get_contents (meta, &itemPrinter, NULL);
 }
 
 /**
@@ -77,31 +73,23 @@ eventCallback (void *cls, const GNUNET_FSUI_Event * event)
 
   switch (event->type)
     {
-    case GNUNET_FSUI_search_error:
-      errorCode = 3;
-      GNUNET_shutdown_initiate ();
-      break;
     case GNUNET_FSUI_search_aborted:
       errorCode = 4;
       GNUNET_shutdown_initiate ();
       break;
-    case GNUNET_FSUI_search_completed:
-      errorCode = 0;
-      GNUNET_shutdown_initiate ();
-      break;
     case GNUNET_FSUI_search_result:
       /* retain URIs for possible directory dump later */
       GNUNET_array_grow (fis, fiCount, fiCount + 1);
       fis[fiCount - 1].uri =
         GNUNET_ECRS_uri_duplicate (event->data.SearchResult.fi.uri);
       fis[fiCount - 1].meta =
-        GNUNET_ECRS_meta_data_duplicate (event->data.SearchResult.fi.meta);
+        GNUNET_meta_data_duplicate (event->data.SearchResult.fi.meta);
 
       uri = GNUNET_ECRS_uri_to_string (event->data.SearchResult.fi.uri);
       printf ("%s:\n", uri);
       filename =
-        GNUNET_ECRS_meta_data_get_by_type (event->data.SearchResult.fi.meta,
-                                           EXTRACTOR_FILENAME);
+        GNUNET_meta_data_get_by_type (event->data.SearchResult.fi.meta,
+                                      EXTRACTOR_FILENAME);
       if (filename != NULL)
         {
           char *dotdot;
@@ -120,6 +108,7 @@ eventCallback (void *cls, const GNUNET_FSUI_Event * event)
       break;
     case GNUNET_FSUI_search_started:
     case GNUNET_FSUI_search_stopped:
+    case GNUNET_FSUI_search_update:
       break;
     default:
       GNUNET_GE_BREAK (NULL, 0);
@@ -139,15 +128,9 @@ static struct GNUNET_CommandLineOption gnunetsearchOptions[] = {
   GNUNET_COMMAND_LINE_OPTION_HELP (gettext_noop ("Search GNUnet for files.")),  /* -h */
   GNUNET_COMMAND_LINE_OPTION_HOSTNAME,  /* -H */
   GNUNET_COMMAND_LINE_OPTION_LOGGING,   /* -L */
-  {'m', "max", "LIMIT",
-   gettext_noop ("exit after receiving LIMIT results"),
-   1, &GNUNET_getopt_configure_set_uint, &max_results},
   {'o', "output", "FILENAME",
    gettext_noop ("write encountered (decrypted) search results to FILENAME"),
    1, &GNUNET_getopt_configure_set_string, &output_filename},
-  {'t', "timeout", "DELAY",
-   gettext_noop ("wait DELAY seconds for search results before aborting"),
-   1, &GNUNET_getopt_configure_set_uint, &delay},
   GNUNET_COMMAND_LINE_OPTION_VERSION (PACKAGE_VERSION), /* -v */
   GNUNET_COMMAND_LINE_OPTION_VERBOSE,
   GNUNET_COMMAND_LINE_OPTION_END,
@@ -187,8 +170,6 @@ main (int argc, char *const *argv)
       errorCode = -1;
       goto quit;
     }
-  if (max_results == 0)
-    max_results = (unsigned int) -1;    /* infty */
   ctx =
     GNUNET_FSUI_start (ectx, cfg, "gnunet-search", 4, GNUNET_NO,
                        &eventCallback, NULL);
@@ -199,9 +180,7 @@ main (int argc, char *const *argv)
       return GNUNET_SYSERR;
     }
   errorCode = 1;
-  s = GNUNET_FSUI_search_start (ctx,
-                                anonymity, max_results,
-                                delay * GNUNET_CRON_SECONDS, uri);
+  s = GNUNET_FSUI_search_start (ctx, anonymity, uri);
   GNUNET_ECRS_uri_destroy (uri);
   if (s == NULL)
     {
@@ -211,8 +190,8 @@ main (int argc, char *const *argv)
     }
   GNUNET_shutdown_wait_for ();
   if (errorCode == 1)
-    GNUNET_FSUI_search_abort (ctx, s);
-  GNUNET_FSUI_search_stop (ctx, s);
+    GNUNET_FSUI_search_abort (s);
+  GNUNET_FSUI_search_stop (s);
   GNUNET_FSUI_stop (ctx);
 
   if (output_filename != NULL)
@@ -220,9 +199,9 @@ main (int argc, char *const *argv)
       char *outfile;
       unsigned long long n;
       char *data;
-      struct GNUNET_ECRS_MetaData *meta;
+      struct GNUNET_MetaData *meta;
 
-      meta = GNUNET_ECRS_meta_data_create ();
+      meta = GNUNET_meta_data_create ();
       /* ?: anything here to put into meta? */
       if (GNUNET_OK ==
           GNUNET_ECRS_directory_create (ectx, &data, &n, fiCount, fis, meta))
@@ -237,7 +216,7 @@ main (int argc, char *const *argv)
   for (i = 0; i < fiCount; i++)
     {
       GNUNET_ECRS_uri_destroy (fis[i].uri);
-      GNUNET_ECRS_meta_data_destroy (fis[i].meta);
+      GNUNET_meta_data_destroy (fis[i].meta);
     }
   GNUNET_array_grow (fis, fiCount, 0);
 quit:
