@@ -53,7 +53,7 @@ static unsigned long long stored_ops;
 static GNUNET_CronTime start_time;
 
 static int
-putValue (GNUNET_SQstore_ServiceAPI * api, int i)
+putValue (GNUNET_SQstore_ServiceAPI * api, int i, int k)
 {
   GNUNET_DatastoreValue *value;
   size_t size;
@@ -75,12 +75,16 @@ putValue (GNUNET_SQstore_ServiceAPI * api, int i)
   value = GNUNET_malloc (size);
   value->size = htonl (size);
   value->type = htonl (i);
-  value->prio = htonl (GNUNET_random_u32 (GNUNET_RANDOM_QUALITY_WEAK, 100));
-  value->anonymityLevel = htonl (i);
-  value->expirationTime =
+  value->priority =
+    htonl (GNUNET_random_u32 (GNUNET_RANDOM_QUALITY_WEAK, 100));
+  value->anonymity_level = htonl (i);
+  value->expiration_time =
     GNUNET_htonll (GNUNET_get_time () + 60 * GNUNET_CRON_HOURS +
                    GNUNET_random_u32 (GNUNET_RANDOM_QUALITY_WEAK, 1000));
   memset (&value[1], i, size - sizeof (GNUNET_DatastoreValue));
+  if (i > 255)
+    memset (&value[1], i - 255, (size - sizeof (GNUNET_DatastoreValue)) / 2);
+  ((char *) &value[1])[0] = k;
   if (GNUNET_OK != api->put (&key, value))
     {
       GNUNET_free (value);
@@ -119,7 +123,7 @@ test (GNUNET_SQstore_ServiceAPI * api)
       start = GNUNET_get_time ();
       for (j = 0; j < PUT_10; j++)
         {
-          if (GNUNET_OK != putValue (api, j))
+          if (GNUNET_OK != putValue (api, j, i))
             break;
           if (GNUNET_shutdown_test () == GNUNET_YES)
             break;
