@@ -164,9 +164,8 @@ GNUNET_STATS_cs_message_type_to_string (unsigned short type)
     case GNUNET_CS_PROTO_GAP_QUERY_START:
       name = "CS_PROTO_gap_QUERY_START";
       break;
-    case GNUNET_CS_PROTO_GAP_QUERY_STOP:
-      /* case GNUNET_CS_PROTO_GAP_RESULT : */
-      name = "CS_PROTO_gap_QUERY_STOP or CS_PROTO_gap_RESULT";
+    case GNUNET_CS_PROTO_GAP_RESULT:
+      name = "CS_PROTO_gap_RESULT";
       break;
     case GNUNET_CS_PROTO_GAP_INSERT:
       name = "CS_PROTO_gap_INSERT";
@@ -183,9 +182,6 @@ GNUNET_STATS_cs_message_type_to_string (unsigned short type)
     case GNUNET_CS_PROTO_GAP_TESTINDEX:
       name = "CS_PROTO_gap_TESTINDEX";
       break;
-    case GNUNET_CS_PROTO_GAP_GET_AVG_PRIORITY:
-      name = "CS_PROTO_gap_GET_AVG_PRIORITY";
-      break;
     case GNUNET_CS_PROTO_GAP_INIT_INDEX:
       name = "CS_PROTO_gap_INIT_INDEX";
       break;
@@ -227,8 +223,23 @@ GNUNET_STATS_cs_message_type_to_string (unsigned short type)
       name = "GNUNET_CS_PROTO_TRACEKIT_REPLY";
       break;
 
-    case GNUNET_CS_PROTO_CHAT_MSG:
-      name = "GNUNET_CS_PROTO_CHAT_MSG";
+    case GNUNET_CS_PROTO_CHAT_JOIN_REQUEST:
+      name = "GNUNET_CS_PROTO_CHAT_JOIN_REQUEST";
+      break;
+    case GNUNET_CS_PROTO_CHAT_JOIN_NOTIFICATION:
+      name = "GNUNET_CS_PROTO_CHAT_JOIN_NOTIFICATION";
+      break;
+    case GNUNET_CS_PROTO_CHAT_LEAVE_NOTIFICATION:
+      name = "GNUNET_CS_PROTO_CHAT_LEAVE_NOTIFICATION";
+      break;
+    case GNUNET_CS_PROTO_CHAT_MESSAGE_NOTIFICATION:
+      name = "GNUNET_CS_PROTO_CHAT_MESSAGE_NOTIFICATION";
+      break;
+    case GNUNET_CS_PROTO_CHAT_TRANSMIT_REQUEST:
+      name = "GNUNET_CS_PROTO_CHAT_TRANSMIT_REQUEST";
+      break;
+    case GNUNET_CS_PROTO_CHAT_CONFIRMATION_RECEIPT:
+      name = "GNUNET_CS_PROTO_CHAT_CONFIRMATION_RECEIPT";
       break;
 
     case GNUNET_CS_PROTO_DHT_REQUEST_GET:
@@ -238,22 +249,12 @@ GNUNET_STATS_cs_message_type_to_string (unsigned short type)
       name = "GNUNET_CS_PROTO_DHT_REQUEST_PUT";
       break;
 
-    case GNUNET_CS_PROTO_TESTBED_REQUEST:
-      name = "GNUNET_CS_PROTO_TESTBED_REQUEST";
-      break;
-    case GNUNET_CS_PROTO_TESTBED_REPLY:
-      name = "GNUNET_CS_PROTO_TESTBED_REPLY";
-      break;
-
     case GNUNET_CS_PROTO_VPN_MSG:
       name = "GNUNET_CS_PROTO_VPN_MSG";
       break;
     case GNUNET_CS_PROTO_VPN_REPLY:
       name = "GNUNET_CS_PROTO_VPN_REPLY";
       break;
-    case GNUNET_CS_PROTO_VPN_DEBUGOFF:
-      name = "GNUNET_CS_PROTO_VPN_DEBUGOFF";
-      break;
     case GNUNET_CS_PROTO_VPN_TUNNELS:
       name = "GNUNET_CS_PROTO_VPN_TUNNELS";
       break;
@@ -266,9 +267,6 @@ GNUNET_STATS_cs_message_type_to_string (unsigned short type)
     case GNUNET_CS_PROTO_VPN_RESET:
       name = "GNUNET_CS_PROTO_VPN_RESET";
       break;
-    case GNUNET_CS_PROTO_VPN_REALISE:
-      name = "GNUNET_CS_PROTO_VPN_REALISE";
-      break;
     case GNUNET_CS_PROTO_VPN_ADD:
       name = "GNUNET_CS_PROTO_VPN_ADD";
       break;
