@@ -98,11 +98,12 @@ statHandle (const char *name)
         return i;
       }
   GNUNET_array_grow (entries, statCounters, statCounters + 1);
-  entries[statCounters - 1].description = GNUNET_strdup (name);
-  entries[statCounters - 1].descStrLen = strlen (name);
-  entries[statCounters - 1].value = 0;
+  i = statCounters - 1;
+  entries[i].description = GNUNET_strdup (name);
+  entries[i].descStrLen = strlen (name);
+  entries[i].value = 0;
   GNUNET_mutex_unlock (statLock);
-  return statCounters - 1;
+  return i;
 }
 
 /**
@@ -224,12 +225,16 @@ extern unsigned int uiHandleCount;
 static void
 initializeStats ()
 {
-  stat_handle_network_load_up
-    = statHandle (gettext_noop ("% of allowed network load (up)"));
-  stat_handle_network_load_down
-    = statHandle (gettext_noop ("% of allowed network load (down)"));
-  stat_handle_cpu_load = statHandle (gettext_noop ("% of allowed cpu load"));
-  stat_handle_io_load = statHandle (gettext_noop ("% of allowed io load"));
+  stat_handle_network_load_up = statHandle (gettext_noop (      /* xgettext:no-c-format */
+                                                           "% of allowed network load (up)"));
+  stat_handle_network_load_down = statHandle (gettext_noop (    /* xgettext:no-c-format */
+                                                             "% of allowed network load (down)"));
+  stat_handle_cpu_load =
+    statHandle (gettext_noop
+                ( /* xgettext:no-c-format */ "% of allowed cpu load"));
+  stat_handle_io_load =
+    statHandle (gettext_noop
+                ( /* xgettext:no-c-format */ "% of allowed io load"));
   stat_connected = statHandle (gettext_noop ("# of connected peers"));
   stat_bytes_noise_received
     = statHandle (gettext_noop ("# bytes of noise received"));
@@ -265,7 +270,7 @@ immediateUpdates ()
   if (load == -1)
     load = 0;
   statSet (stat_handle_network_load_down, load);
-  statSet (stat_connected, coreAPI->forAllConnectedNodes (NULL, NULL));
+  statSet (stat_connected, coreAPI->p2p_connections_iterate (NULL, NULL));
 #ifdef MINGW
   statSet (stat_handles, plibc_get_handle_count ());
 #endif
@@ -322,8 +327,8 @@ sendStatistics (struct GNUNET_ClientHandle *sock,
         {
           memcpy (&
                   ((char
-                    *) (((CS_stats_reply_MESSAGE_GENERIC *) statMsg))->
-                   values)[mpos], entries[pos].description,
+                    *) (((CS_stats_reply_MESSAGE_GENERIC *)
+                         statMsg))->values)[mpos], entries[pos].description,
                   entries[pos].descStrLen + 1);
           mpos += entries[pos].descStrLen + 1;
         }
@@ -337,7 +342,7 @@ sendStatistics (struct GNUNET_ClientHandle *sock,
          ntohs(statMsg->header.size),
          start, end, statCounters); */
       if (GNUNET_SYSERR ==
-          coreAPI->cs_send_to_client (sock, &statMsg->header, GNUNET_YES))
+          coreAPI->cs_send_message (sock, &statMsg->header, GNUNET_YES))
         break;                  /* abort, socket error! */
       start = end;
     }
@@ -365,8 +370,8 @@ handleMessageSupported (struct GNUNET_ClientHandle *sock,
   cmsg = (CS_stats_get_supported_MESSAGE *) message;
   type = ntohs (cmsg->type);
   htype = ntohs (cmsg->handlerType);
-  supported = coreAPI->p2p_test_handler_registered (type, htype);
-  return coreAPI->sendValueToClient (sock, supported);
+  supported = coreAPI->p2p_message_handler_registered_test (type, htype);
+  return coreAPI->cs_send_value (sock, supported);
 }
 
 /**
@@ -386,8 +391,8 @@ processGetConnectionCountRequest (struct GNUNET_ClientHandle *client,
       GNUNET_GE_BREAK (NULL, 0);
       return GNUNET_SYSERR;
     }
-  return coreAPI->sendValueToClient
-    (client, coreAPI->forAllConnectedNodes (NULL, NULL));
+  return coreAPI->cs_send_value
+    (client, coreAPI->p2p_connections_iterate (NULL, NULL));
 }
 
 /**
@@ -407,7 +412,7 @@ initialize_module_stats (GNUNET_CoreAPIForPlugins * capi)
 {
   GNUNET_GE_ASSERT (capi->ectx, myCoreAPI == NULL);
   myCoreAPI = capi;
-  stats = capi->request_service ("stats");
+  stats = capi->service_request ("stats");
   if (stats == NULL)
     {
       GNUNET_GE_BREAK (capi->ectx, 0);
@@ -423,18 +428,17 @@ initialize_module_stats (GNUNET_CoreAPIForPlugins * capi)
                  GNUNET_CS_PROTO_STATS_GET_STATISTICS,
                  GNUNET_CS_PROTO_STATS_GET_P2P_MESSAGE_SUPPORTED,
                  GNUNET_P2P_PROTO_NOISE);
-  capi->registerClientHandler (GNUNET_CS_PROTO_STATS_GET_STATISTICS,
-                               &sendStatistics);
-  capi->
-    registerClientHandler
+  capi->cs_handler_register (GNUNET_CS_PROTO_STATS_GET_STATISTICS,
+                             &sendStatistics);
+  capi->cs_handler_register
     (GNUNET_CS_PROTO_STATS_GET_P2P_MESSAGE_SUPPORTED,
      &handleMessageSupported);
-  capi->
-    registerClientHandler
+  capi->cs_handler_register
     (GNUNET_CS_PROTO_STATS_GET_CS_MESSAGE_SUPPORTED, &handleMessageSupported);
-  capi->registerClientHandler (GNUNET_CS_PROTO_TRAFFIC_COUNT,
-                               &processGetConnectionCountRequest);
-  capi->registerHandler (GNUNET_P2P_PROTO_NOISE, &processNoise);
+  capi->cs_handler_register (GNUNET_CS_PROTO_TRAFFIC_COUNT,
+                             &processGetConnectionCountRequest);
+  capi->p2p_ciphertext_handler_register (GNUNET_P2P_PROTO_NOISE,
+                                         &processNoise);
   GNUNET_GE_ASSERT (capi->ectx,
                     0 == GNUNET_GC_set_configuration_value_string (capi->cfg,
                                                                    capi->ectx,
@@ -456,19 +460,18 @@ done_module_stats ()
   done_sqstore_stats ();
 #endif
   GNUNET_GE_ASSERT (NULL, myCoreAPI != NULL);
-  coreAPI->unregisterClientHandler (GNUNET_CS_PROTO_STATS_GET_STATISTICS,
-                                    &sendStatistics);
-  coreAPI->
-    unregisterClientHandler
+  coreAPI->cs_handler_unregister (GNUNET_CS_PROTO_STATS_GET_STATISTICS,
+                                  &sendStatistics);
+  coreAPI->cs_handler_unregister
     (GNUNET_CS_PROTO_STATS_GET_P2P_MESSAGE_SUPPORTED,
      &handleMessageSupported);
-  coreAPI->
-    unregisterClientHandler
+  coreAPI->cs_handler_unregister
     (GNUNET_CS_PROTO_STATS_GET_CS_MESSAGE_SUPPORTED, &handleMessageSupported);
-  coreAPI->unregisterClientHandler (GNUNET_CS_PROTO_TRAFFIC_COUNT,
-                                    &processGetConnectionCountRequest);
-  coreAPI->unregisterHandler (GNUNET_P2P_PROTO_NOISE, &processNoise);
-  myCoreAPI->release_service (stats);
+  coreAPI->cs_handler_unregister (GNUNET_CS_PROTO_TRAFFIC_COUNT,
+                                  &processGetConnectionCountRequest);
+  coreAPI->p2p_ciphertext_handler_unregister (GNUNET_P2P_PROTO_NOISE,
+                                              &processNoise);
+  myCoreAPI->service_release (stats);
   stats = NULL;
   myCoreAPI = NULL;
   return GNUNET_OK;
