@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2007 Christian Grothoff (and other contributing authors)
+     (C) 2008 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -19,7 +19,7 @@
 */
 
 /**
- * @file applications/testing/testingtest.c
+ * @file applications/testing/testing.c
  * @brief testcase for testing library
  * @author Christian Grothoff
  */
@@ -110,9 +110,7 @@ GNUNET_TESTING_start_daemon (unsigned short app_port,
     }
   GNUNET_free (dpath);
   updatePort (cfg, "TCP", tra_offset);
-  updatePort (cfg, "TCP6", tra_offset);
   updatePort (cfg, "UDP", tra_offset);
-  updatePort (cfg, "UDP6", tra_offset);
   updatePort (cfg, "HTTP", tra_offset);
   updatePort (cfg, "SMTP", tra_offset);
   GNUNET_GC_set_configuration_value_string (cfg,
@@ -173,7 +171,7 @@ GNUNET_TESTING_start_daemon (unsigned short app_port,
      maybe creating public keys and waiting for
      entropy! */
   if (GNUNET_OK !=
-      GNUNET_wait_for_daemon_running (NULL, cfg, 15 * GNUNET_CRON_MINUTES))
+      GNUNET_wait_for_daemon_running (NULL, cfg, 60 * GNUNET_CRON_SECONDS))
     {
       fprintf (stderr, "Failed to confirm daemon running!\n");
       GNUNET_GC_free (cfg);
@@ -185,7 +183,7 @@ GNUNET_TESTING_start_daemon (unsigned short app_port,
   dpath = NULL;
   round = 0;
   ret = GNUNET_SYSERR;
-  while ((round++ < 10) && (ret == GNUNET_SYSERR))
+  while ((round++ < 20) && (ret == GNUNET_SYSERR))
     {
       sock = GNUNET_client_connection_create (NULL, cfg);
       ret = GNUNET_IDENTITY_get_self (sock, &hello);
@@ -197,7 +195,7 @@ GNUNET_TESTING_start_daemon (unsigned short app_port,
         }
       else
         {
-          GNUNET_thread_sleep (2 * GNUNET_CRON_SECONDS);
+          GNUNET_thread_sleep (1500 * GNUNET_CRON_MILLISECONDS);
         }
       GNUNET_client_connection_destroy (sock);
     }
@@ -283,6 +281,8 @@ GNUNET_TESTING_connect_daemons (unsigned short port1, unsigned short port2)
                                                                  senderIdentity))
                 {
                   ret = GNUNET_OK;
+                  GNUNET_free_non_null (h1);
+                  GNUNET_free_non_null (h2);
                   break;
                 }
               if (GNUNET_YES == GNUNET_IDENTITY_request_connect (sock2,
@@ -290,9 +290,11 @@ GNUNET_TESTING_connect_daemons (unsigned short port1, unsigned short port2)
                                                                  senderIdentity))
                 {
                   ret = GNUNET_OK;
+                  GNUNET_free_non_null (h1);
+                  GNUNET_free_non_null (h2);
                   break;
                 }
-              GNUNET_thread_sleep (100 * GNUNET_CRON_MILLISECONDS);
+              GNUNET_thread_sleep (1500 * GNUNET_CRON_MILLISECONDS);
             }
           GNUNET_free_non_null (h1);
           GNUNET_free_non_null (h2);
@@ -414,5 +416,4 @@ GNUNET_TESTING_stop_daemons (struct GNUNET_TESTING_DaemonContext *peers)
 
 
 
-
 /* end of testing.c */
