@@ -322,12 +322,12 @@ buildReply (unsigned int countTimeUnits)
     if (counters[i] != NULL)
       {
         if (counters[i]->send.slots != 0)
-          buildSummary (&((CS_traffic_info_MESSAGE_GENERIC *) reply)->
-                        counters[count++], &counters[i]->send,
+          buildSummary (&((CS_traffic_info_MESSAGE_GENERIC *) reply)->counters
+                        [count++], &counters[i]->send,
                         GNUNET_TRAFFIC_TYPE_SENT, countTimeUnits, i);
         if (counters[i]->receive.slots != 0)
-          buildSummary (&((CS_traffic_info_MESSAGE_GENERIC *) reply)->
-                        counters[count++], &counters[i]->receive,
+          buildSummary (&((CS_traffic_info_MESSAGE_GENERIC *) reply)->counters
+                        [count++], &counters[i]->receive,
                         GNUNET_TRAFFIC_TYPE_RECEIVED, countTimeUnits, i);
       }
 
@@ -347,7 +347,7 @@ trafficQueryHandler (struct GNUNET_ClientHandle *sock,
     return GNUNET_SYSERR;
   msg = (const CS_traffic_request_MESSAGE *) message;
   reply = buildReply (ntohl (msg->timePeriod));
-  ret = coreAPI->cs_send_to_client (sock, &reply->header, GNUNET_YES);
+  ret = coreAPI->cs_send_message (sock, &reply->header, GNUNET_YES);
   GNUNET_free (reply);
   return ret;
 }
@@ -507,6 +507,8 @@ trafficReceive (const GNUNET_PeerIdentity * sender,
 {
   unsigned short port;
 
+  if (sender == NULL)
+    return GNUNET_OK;
   port = ntohs (header->type);
   updateTrafficReceiveCounter (port, ntohs (header->size));
   GNUNET_mutex_lock (lock);
@@ -581,17 +583,17 @@ provide_module_traffic (GNUNET_CoreAPIForPlugins * capi)
   api.get = &getTrafficStats;
   for (i = 0; i < GNUNET_P2P_PROTO_MAX_USED; i++)
     stat_traffic_transmitted_by_type[i] = 0;
-  coreAPI->connection_register_send_notification_callback (&trafficSend);
+  coreAPI->peer_send_notification_register (&trafficSend);
   for (i = 0; i < GNUNET_P2P_PROTO_MAX_USED; i++)
     {
       stat_traffic_received_by_type[i] = 0;
-      coreAPI->registerHandler (i, &trafficReceive);
-      coreAPI->plaintext_register_handler (i, &plaintextReceive);
+      coreAPI->p2p_ciphertext_handler_register (i, &trafficReceive);
+      coreAPI->p2p_plaintext_handler_register (i, &plaintextReceive);
     }
 
   GNUNET_GE_ASSERT (coreAPI->ectx, counters == NULL);
   lock = GNUNET_mutex_create (GNUNET_NO);
-  stats = capi->request_service ("stats");
+  stats = capi->service_request ("stats");
   return &api;
 }
 
@@ -605,11 +607,11 @@ release_module_traffic ()
 
   for (i = 0; i < GNUNET_P2P_PROTO_MAX_USED; i++)
     {
-      coreAPI->unregisterHandler (i, &trafficReceive);
-      coreAPI->plaintext_unregister_handler (i, &plaintextReceive);
+      coreAPI->p2p_ciphertext_handler_unregister (i, &trafficReceive);
+      coreAPI->p2p_plaintext_handler_unregister (i, &plaintextReceive);
     }
-  coreAPI->connection_unregister_send_notification_callback (&trafficSend);
-  coreAPI->release_service (stats);
+  coreAPI->peer_send_notification_unregister (&trafficSend);
+  coreAPI->service_release (stats);
   stats = NULL;
   for (i = 0; i < max_message_type; i++)
     GNUNET_free_non_null (counters[i]);
@@ -632,15 +634,15 @@ initialize_module_traffic (GNUNET_CoreAPIForPlugins * capi)
 {
   GNUNET_GE_ASSERT (capi->ectx, myCoreAPI == NULL);
   myCoreAPI = capi;
-  myApi = capi->request_service ("traffic");
+  myApi = capi->service_request ("traffic");
   if (myApi == NULL)
     {
       GNUNET_GE_BREAK (capi->ectx, 0);
       myCoreAPI = NULL;
       return GNUNET_SYSERR;
     }
-  capi->registerClientHandler (GNUNET_CS_PROTO_TRAFFIC_QUERY,
-                               &trafficQueryHandler);
+  capi->cs_handler_register (GNUNET_CS_PROTO_TRAFFIC_QUERY,
+                             &trafficQueryHandler);
   GNUNET_GE_ASSERT (capi->ectx,
                     0 == GNUNET_GC_set_configuration_value_string (capi->cfg,
                                                                    capi->ectx,
@@ -660,10 +662,9 @@ done_module_traffic ()
   GNUNET_GE_ASSERT (NULL, myCoreAPI != NULL);
   GNUNET_GE_ASSERT (myCoreAPI->ectx,
                     GNUNET_SYSERR !=
-                    myCoreAPI->
-                    unregisterClientHandler (GNUNET_CS_PROTO_TRAFFIC_QUERY,
-                                             &trafficQueryHandler));
-  myCoreAPI->release_service (myApi);
+                    myCoreAPI->cs_handler_unregister
+                    (GNUNET_CS_PROTO_TRAFFIC_QUERY, &trafficQueryHandler));
+  myCoreAPI->service_release (myApi);
   myApi = NULL;
   myCoreAPI = NULL;
 }
