@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet
-     (C) 2004, 2005, 2006, 2007 Christian Grothoff (and other contributing authors)
+     (C) 2004, 2005, 2006, 2007, 2008 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -29,6 +29,7 @@
 
 #include "gnunet_util_core.h"
 #include "gnunet_core.h"
+#include "gnunet_fs_lib.h"
 #include <extractor.h>
 
 #ifdef __cplusplus
@@ -40,12 +41,12 @@ extern "C"
 #endif
 
 /**
- * Just the version number of the AFS/ESED/ESED2/ECRS implementation.
+ * Version number of the implementation.
  * History:
  *
  * 1.x.x: initial version with triple GNUNET_hash and merkle tree
  * 2.x.x: root node with mime-type, filename and version number
- * 2.1.x: combined CHK/3HASH encoding with 25:1 super-nodes
+ * 2.1.x: combined GNUNET_EC_ContentHashKey/3HASH encoding with 25:1 super-nodes
  * 2.2.x: with directories
  * 3.0.x: with namespaces
  * 3.1.x: with namespace meta-data
@@ -53,9 +54,11 @@ extern "C"
  * 4.0.x: with expiration, variable meta-data, kblocks
  * 4.1.x: with new error and configuration handling
  * 5.0.x: with location URIs
- * 6.x.x: who knows? :-)
+ * 6.0.0: with support for OR in KSKs
+ * 6.1.x: with simplified namespace support
+ * 7.0.0: who knows? :-)
  */
-#define GNUNET_ECRS_VERSION "5.0.1"
+#define GNUNET_ECRS_VERSION "6.0.0"
 
 #define GNUNET_DIRECTORY_MIME  "application/gnunet-directory"
 #define GNUNET_DIRECTORY_MAGIC "\211GND\r\n\032\n"
@@ -68,182 +71,15 @@ extern "C"
 #define GNUNET_ECRS_FILE_INFIX      "chk/"
 #define GNUNET_ECRS_LOCATION_INFIX  "loc/"
 
-
-/**
- * Fixed SBlock updateInterval codes. Positive values
- * are interpreted as durations (in seconds) for periodical
- * updates.
- */
-#define GNUNET_ECRS_SBLOCK_UPDATE_SPORADIC  -1
-#define GNUNET_ECRS_SBLOCK_UPDATE_NONE       0
-
-
-
-/* ***************** metadata API (meta.c) ******************** */
-
-/**
- * Meta data to associate with a file, directory or namespace.
- */
-struct GNUNET_ECRS_MetaData;
-
-/**
- * Iterator over meta data.
- * @return GNUNET_OK to continue to iterate, GNUNET_SYSERR to abort
- */
-typedef int (*GNUNET_ECRS_MetaDataProcessor) (EXTRACTOR_KeywordType type,
-                                              const char *data,
-                                              void *closure);
-
 /**
  * Iterator over keywords
+ *
+ * @param keyword the keyword
+ * @param is_mandatory is the keyword mandatory (in a search)
  * @return GNUNET_OK to continue to iterate, GNUNET_SYSERR to abort
  */
-typedef int (*GNUNET_ECRS_KeywordIterator) (const char *data, void *closure);
-
-/**
- * Create a fresh MetaData token.
- */
-struct GNUNET_ECRS_MetaData *GNUNET_ECRS_meta_data_create (void);
-
-/**
- * Duplicate a MetaData token.
- */
-struct GNUNET_ECRS_MetaData *GNUNET_ECRS_meta_data_duplicate (const struct
-                                                              GNUNET_ECRS_MetaData
-                                                              *meta);
-
-/**
- * Free meta data.
- */
-void GNUNET_ECRS_meta_data_destroy (struct GNUNET_ECRS_MetaData *md);
-
-/**
- * Test if two MDs are equal.
- */
-int GNUNET_ECRS_meta_data_test_equal (const struct GNUNET_ECRS_MetaData *md1,
-                                      const struct GNUNET_ECRS_MetaData *md2);
-
-
-/**
- * Extend metadata.
- * @return GNUNET_OK on success, GNUNET_SYSERR if this entry already exists
- */
-int GNUNET_ECRS_meta_data_insert (struct GNUNET_ECRS_MetaData *md,
-                                  EXTRACTOR_KeywordType type,
-                                  const char *data);
-
-/**
- * Remove an item.
- * @return GNUNET_OK on success, GNUNET_SYSERR if the item does not exist in md
- */
-int GNUNET_ECRS_meta_data_delete (struct GNUNET_ECRS_MetaData *md,
-                                  EXTRACTOR_KeywordType type,
-                                  const char *data);
-
-/**
- * Add the current time as the publication date
- * to the meta-data.
- */
-void GNUNET_ECRS_meta_data_add_publication_date (struct GNUNET_ECRS_MetaData
-                                                 *md);
-
-/**
- * Iterate over MD entries, excluding thumbnails.
- *
- * @return number of entries
- */
-int GNUNET_ECRS_meta_data_get_contents (const struct GNUNET_ECRS_MetaData *md,
-                                        GNUNET_ECRS_MetaDataProcessor
-                                        iterator, void *closure);
-
-/**
- * Get the first MD entry of the given type.
- * @return NULL if we do not have any such entry,
- *  otherwise client is responsible for freeing the value!
- */
-char *GNUNET_ECRS_meta_data_get_by_type (const struct GNUNET_ECRS_MetaData
-                                         *md, EXTRACTOR_KeywordType type);
-
-/**
- * Get the first matching MD entry of the given types.
- * @param ... -1-terminated list of types
- * @return NULL if we do not have any such entry,
- *  otherwise client is responsible for freeing the value!
- */
-char *GNUNET_ECRS_meta_data_get_first_by_types (const struct
-                                                GNUNET_ECRS_MetaData *md,
-                                                ...);
-
-/**
- * Get a thumbnail from the meta-data (if present).
- *
- * @param thumb will be set to the thumbnail data.  Must be
- *        freed by the caller!
- * @return number of bytes in thumbnail, 0 if not available
- */
-size_t GNUNET_ECRS_meta_data_get_thumbnail (const struct GNUNET_ECRS_MetaData
-                                            *md, unsigned char **thumb);
-
-/**
- * Extract meta-data from a file.
- *
- * @return GNUNET_SYSERR on error, otherwise the number
- *   of meta-data items obtained
- */
-int GNUNET_ECRS_meta_data_extract_from_file (struct GNUNET_GE_Context *ectx,
-                                             struct GNUNET_ECRS_MetaData *md,
-                                             const char *filename,
-                                             EXTRACTOR_ExtractorList *
-                                             extractors);
-
-/* = 0 */
-#define GNUNET_ECRS_SERIALIZE_FULL GNUNET_NO
-
-/* = 1 */
-#define GNUNET_ECRS_SERIALIZE_PART GNUNET_YES
-
-/* disallow compression (if speed is important) */
-#define GNUNET_ECRS_SERIALIZE_NO_COMPRESS 2
-
-
-/**
- * Serialize meta-data to target.
- *
- * @param size maximum number of bytes available
- * @param part is it ok to just write SOME of the
- *        meta-data to match the size constraint,
- *        possibly discarding some data? GNUNET_YES/GNUNET_NO.
- * @return number of bytes written on success,
- *         GNUNET_SYSERR on error (typically: not enough
- *         space)
- */
-int GNUNET_ECRS_meta_data_serialize (struct GNUNET_GE_Context *ectx,
-                                     const struct GNUNET_ECRS_MetaData *md,
-                                     char *target, unsigned int size,
-                                     int part);
-
-/**
- * Compute size of the meta-data in
- * serialized form.
- * @part flags (partial ok, may compress?)
- */
-unsigned int GNUNET_ECRS_meta_data_get_serialized_size (const struct
-                                                        GNUNET_ECRS_MetaData
-                                                        *md, int part);
-
-/**
- * Deserialize meta-data.  Initializes md.
- * @param size number of bytes available
- * @return MD on success, NULL on error (i.e.
- *         bad format)
- */
-struct GNUNET_ECRS_MetaData *GNUNET_ECRS_meta_data_deserialize (struct
-                                                                GNUNET_GE_Context
-                                                                *ectx,
-                                                                const char
-                                                                *input,
-                                                                unsigned int
-                                                                size);
+typedef int (*GNUNET_ECRS_KeywordIterator) (const char *keyword,
+                                            int is_mandatory, void *closure);
 
 /**
  * Does the meta-data claim that this is a directory?
@@ -252,8 +88,7 @@ struct GNUNET_ECRS_MetaData *GNUNET_ECRS_meta_data_deserialize (struct
  * @return GNUNET_YES if it is, GNUNET_NO if it is not, GNUNET_SYSERR if
  *  we have no mime-type information (treat as 'GNUNET_NO')
  */
-int GNUNET_ECRS_meta_data_test_for_directory (const struct
-                                              GNUNET_ECRS_MetaData *md);
+int GNUNET_meta_data_test_for_directory (const struct GNUNET_MetaData *md);
 
 /**
  * Suggest a better filename for a file (and do the
@@ -274,6 +109,13 @@ struct GNUNET_ECRS_URI;
  */
 char *GNUNET_ECRS_uri_to_string (const struct GNUNET_ECRS_URI *uri);
 
+/**
+ * Convert keyword URI to a human readable format
+ * (i.e. the search query that was used in the first place)
+ */
+char *GNUNET_ECRS_ksk_uri_to_human_readable_string (const struct
+                                                    GNUNET_ECRS_URI *uri);
+
 /**
  * Convert a UTF-8 String to a URI.
  */
@@ -294,7 +136,8 @@ unsigned int GNUNET_ECRS_uri_get_keyword_count_from_ksk (const struct
                                                          *uri);
 
 /**
- * Iterate over all keywords in this keyword URI?
+ * Iterate over all keywords in this keyword URI.
+ *
  * @return -1 if this is not a keyword URI, otherwise number of
  *   keywords iterated over until iterator aborted
  */
@@ -333,7 +176,7 @@ typedef int (*GNUNET_ECRS_SignFunction) (void *cls,
  *
  * @param baseURI content offered by the sender
  * @param sender identity of the peer with the content
- * @param expirationTime how long will the content be offered?
+ * @param expiration_time how long will the content be offered?
  * @param signer function to call for obtaining
  *        RSA signatures for "sender".
  * @return the location URI
@@ -366,48 +209,51 @@ struct GNUNET_ECRS_URI *GNUNET_ECRS_uri_expand_keywords_with_date (const
                                                                    GNUNET_ECRS_URI
                                                                    *uri);
 
-/**
- * Convert a NULL-terminated array of keywords
- * to an ECRS URI.
- */
-struct GNUNET_ECRS_URI *GNUNET_ECRS_keyword_strings_to_uri (const char
-                                                            *keyword[]);
-
 /**
  * Create an ECRS URI from a single user-supplied string of keywords.
- * The string may contain the reserved word 'AND' to create a boolean
- * search over multiple keywords.
+ * The string is broken up at spaces into individual keywords.
+ * Keywords that start with "+" are mandatory.  Double-quotes can
+ * be used to prevent breaking up strings at spaces (and also
+ * to specify non-mandatory keywords starting with "+").
+ *
+ * Keywords must contain a balanced number of double quotes and
+ * double quotes can not be used in the actual keywords (for
+ * example, the string '""foo bar""' will be turned into two
+ * "OR"ed keywords 'foo' and 'bar', not into '"foo bar"'.
  *
  * @return an ECRS URI for the given keywords, NULL
  *  if keywords is not legal (i.e. empty).
  */
-struct GNUNET_ECRS_URI *GNUNET_ECRS_keyword_string_to_uri (struct GNUNET_GE_Context *ectx, const char *keywords);       /* helper.c */
+struct GNUNET_ECRS_URI *GNUNET_ECRS_keyword_string_to_uri (struct
+                                                           GNUNET_GE_Context
+                                                           *ectx,
+                                                           const char
+                                                           *keywords);
 
 /**
  * Create an ECRS URI from a user-supplied command line of keywords.
- * The command line may contain the reserved word 'AND' to create a
- * boolean search over multiple keywords.
+ * Arguments should start with "+" to indicate mandatory
+ * keywords.
  *
+ * @param argc number of keywords
+ * @param argv keywords (double quotes are not required for
+ *             keywords containing spaces; however, double
+ *             quotes are required for keywords starting with
+ *             "+"); there is no mechanism for having double
+ *             quotes in the actual keywords (if the user
+ *             did specifically specify double quotes, the
+ *             caller should convert each double quote
+ *             into two single quotes).
  * @return an ECRS URI for the given keywords, NULL
  *  if keywords is not legal (i.e. empty).
  */
-struct GNUNET_ECRS_URI *GNUNET_ECRS_keyword_command_line_to_uri (struct GNUNET_GE_Context *ectx, unsigned int argc, const char **argv); /* helper.c */
-
-/**
- * Create an ECRS URI from a user-supplied list of keywords.
- * The keywords are NOT separated by AND but already
- * given individually.
- *
- * @return an ECRS URI for the given keywords, NULL
- *  if keywords is not legal (i.e. empty).
- */
-struct GNUNET_ECRS_URI *GNUNET_ECRS_keyword_list_to_uri (struct
-                                                         GNUNET_GE_Context
-                                                         *ectx,
-                                                         unsigned int
-                                                         num_keywords,
-                                                         const char
-                                                         **keywords);
+struct GNUNET_ECRS_URI *GNUNET_ECRS_keyword_command_line_to_uri (struct
+                                                                 GNUNET_GE_Context
+                                                                 *ectx,
+                                                                 unsigned int
+                                                                 argc,
+                                                                 const char
+                                                                 **argv);
 
 /**
  * Test if two URIs are equal.
@@ -420,15 +266,6 @@ int GNUNET_ECRS_uri_test_equal (const struct GNUNET_ECRS_URI *u1,
  */
 int GNUNET_ECRS_uri_test_sks (const struct GNUNET_ECRS_URI *uri);
 
-/**
- * Get the (globally unique) name for the given
- * namespace.
- *
- * @return the name (GNUNET_hash) of the namespace, caller
- *  must free it.
- */
-char *GNUNET_ECRS_get_namespace_name (const GNUNET_HashCode * nsid);
-
 /**
  * Get the ID of a namespace from the given
  * namespace URI.
@@ -437,10 +274,13 @@ int GNUNET_ECRS_uri_get_namespace_from_sks (const struct GNUNET_ECRS_URI *uri,
                                             GNUNET_HashCode * nsid);
 
 /**
- * Get the content ID of an SKS URI.
+ * Get the content identifier of an SKS URI.
+ *
+ * @return NULL on error
  */
-int GNUNET_ECRS_uri_get_content_hash_from_sks (const struct GNUNET_ECRS_URI
-                                               *uri, GNUNET_HashCode * nsid);
+char *GNUNET_ECRS_uri_get_content_id_from_sks (const struct GNUNET_ECRS_URI
+                                               *uri);
+
 
 /**
  * Is this a keyword URI?
@@ -471,14 +311,13 @@ int GNUNET_ECRS_uri_test_loc (const struct GNUNET_ECRS_URI *uri);
  * in the meta-data and construct one large keyword URI
  * that lists all keywords that can be found in the meta-data).
  */
-struct GNUNET_ECRS_URI *GNUNET_ECRS_meta_data_to_uri (const struct
-                                                      GNUNET_ECRS_MetaData
-                                                      *md);
+struct GNUNET_ECRS_URI *GNUNET_meta_data_to_uri (const struct
+                                                 GNUNET_MetaData *md);
 
 
 typedef struct
 {
-  struct GNUNET_ECRS_MetaData *meta;
+  struct GNUNET_MetaData *meta;
   struct GNUNET_ECRS_URI *uri;
 } GNUNET_ECRS_FileInfo;
 
@@ -493,7 +332,7 @@ GNUNET_ECRS_getopt_configure_set_keywords (GNUNET_CommandLineProcessorContext
                                            const char *value);
 
 /**
- * @param scls must be of type "struct GNUNET_ECRS_MetaData **"
+ * @param scls must be of type "struct GNUNET_MetaData **"
  */
 int
 GNUNET_ECRS_getopt_configure_set_metadata (GNUNET_CommandLineProcessorContext
@@ -520,8 +359,8 @@ typedef void (*GNUNET_ECRS_UploadProgressCallback)
 /**
  * Should the operation be aborted?  Callback used by many functions
  * below to check if the user has aborted the operation early.  Can
- * also be used for time-outs.  Note that sending a signal (SIGALRM,
- * SIGINT) might be required in addition to TestTerminate to achieve
+ * also be used for time-outs.  Note that sending a signal (SIGALRM)
+ * might be required in addition to TestTerminate to achieve
  * an 'instant' time-out in case that the function is currently
  * sleeping or performing some other blocking operation (which would
  * be aborted by any signal, after which the functions will call
@@ -538,13 +377,23 @@ typedef int (*GNUNET_ECRS_TestTerminate) (void *closure);
  * @param priority what is the priority for OUR node to
  *   keep this file available?  Use 0 for maximum anonymity and
  *   minimum reliability...
- * @param doIndex GNUNET_YES for index, GNUNET_NO for insertion
+ * @param doIndex GNUNET_YES for index, GNUNET_NO for insertion,
+ *                GNUNET_SYSERR for simulation
  * @param uri set to the URI of the uploaded file
  * @return GNUNET_SYSERR if the upload failed (i.e. not enough space
  *  or gnunetd not running)
  */
-int GNUNET_ECRS_file_upload (struct GNUNET_GE_Context *ectx, struct GNUNET_GC_Configuration *cfg, const char *filename, int doIndex, unsigned int anonymityLevel, unsigned int priority, GNUNET_CronTime expirationTime,        /* absolute time */
-                             GNUNET_ECRS_UploadProgressCallback upcb, void *upcbClosure, GNUNET_ECRS_TestTerminate tt, void *ttClosure, struct GNUNET_ECRS_URI **uri);  /* upload.c */
+int GNUNET_ECRS_file_upload (struct GNUNET_GE_Context *ectx,
+                             struct GNUNET_GC_Configuration *cfg,
+                             const char *filename,
+                             int doIndex,
+                             unsigned int anonymityLevel,
+                             unsigned int priority,
+                             GNUNET_CronTime expirationTime,
+                             GNUNET_ECRS_UploadProgressCallback upcb,
+                             void *upcbClosure,
+                             GNUNET_ECRS_TestTerminate tt,
+                             void *ttClosure, struct GNUNET_ECRS_URI **uri);
 
 /**
  * Test if a file is indexed.
@@ -586,40 +435,57 @@ int GNUNET_ECRS_get_indexed_files (struct GNUNET_GE_Context *ectx,
  *
  * @return GNUNET_SYSERR if the unindexing failed (i.e. not indexed)
  */
-int GNUNET_ECRS_file_uninde (struct GNUNET_GE_Context *ectx, struct GNUNET_GC_Configuration *cfg, const char *filename, GNUNET_ECRS_UploadProgressCallback upcb, void *upcbClosure, GNUNET_ECRS_TestTerminate tt, void *ttClosure);     /* unindex.c */
+int GNUNET_ECRS_file_unindex (struct GNUNET_GE_Context *ectx,
+                              struct GNUNET_GC_Configuration *cfg,
+                              const char *filename,
+                              GNUNET_ECRS_UploadProgressCallback upcb,
+                              void *upcbClosure,
+                              GNUNET_ECRS_TestTerminate tt, void *ttClosure);
 
 
 /**
  * Create a new namespace (and publish an advertismement).
- * This publishes both an NBlock in the namespace itself
+ * This publishes both an GNUNET_EC_NBlock in the namespace itself
  * as well as KNBlocks under all keywords specified in
  * the advertisementURI.
  *
- * @param name the name for the namespace
- * @param anonymityLevel for the namespace advertismement
+ * @param anonymity_level for the namespace advertismement
  * @param priority for the namespace advertisement
  * @param expiration for the namespace advertisement
  * @param advertisementURI the keyword (!) URI to advertise the
- *        namespace under (KNBlock)
+ *        namespace under (GNUNET_EC_KNBlock)
  * @param meta meta-data for the namespace advertisement
+ *        (will be used to derive a name)
  * @param rootEntry name of the root entry in the namespace (for
  *        the namespace advertisement)
- * @param rootURI set to the URI of the namespace, NULL if
- *        no advertisement was created
  *
- * @return URI on success, NULL on error (namespace already exists)
+ * @return URI on success, NULL on error
  */
-struct GNUNET_ECRS_URI *GNUNET_ECRS_namespace_create (struct GNUNET_GE_Context *ectx, struct GNUNET_GC_Configuration *cfg, const char *name, const struct GNUNET_ECRS_MetaData *meta, unsigned int anonymityLevel, unsigned int priority, GNUNET_CronTime expiration, const struct GNUNET_ECRS_URI *advertisementURI, const GNUNET_HashCode * rootEntry);       /* namespace.c */
+struct GNUNET_ECRS_URI *GNUNET_ECRS_namespace_create (struct GNUNET_GE_Context
+                                                      *ectx,
+                                                      struct
+                                                      GNUNET_GC_Configuration
+                                                      *cfg,
+                                                      const struct
+                                                      GNUNET_MetaData
+                                                      *meta,
+                                                      unsigned int
+                                                      anonymityLevel,
+                                                      unsigned int priority,
+                                                      GNUNET_CronTime
+                                                      expiration,
+                                                      const struct
+                                                      GNUNET_ECRS_URI
+                                                      *advertisementURI,
+                                                      const char *rootEntry);
 
 /**
  * Check if the given namespace exists (locally).
- * @param hc if non-null, also check that this is the
- *   hc of the public key
+ *
  * @return GNUNET_OK if the namespace exists, GNUNET_SYSERR if not
  */
 int GNUNET_ECRS_namespace_test_exists (struct GNUNET_GE_Context *ectx,
                                        struct GNUNET_GC_Configuration *cfg,
-                                       const char *name,
                                        const GNUNET_HashCode * hc);
 
 /**
@@ -629,7 +495,9 @@ int GNUNET_ECRS_namespace_test_exists (struct GNUNET_GE_Context *ectx,
  *
  * @return GNUNET_OK on success, GNUNET_SYSERR on error
  */
-int GNUNET_ECRS_namespace_delete (struct GNUNET_GE_Context *ectx, struct GNUNET_GC_Configuration *cfg, const char *namespaceName);      /* namespace.c */
+int GNUNET_ECRS_namespace_delete (struct GNUNET_GE_Context *ectx,
+                                  struct GNUNET_GC_Configuration *cfg,
+                                  const GNUNET_HashCode * pid);
 
 /**
  * Callback with information about local (!) namespaces.
@@ -648,19 +516,48 @@ typedef int (*GNUNET_ECRS_NamespaceInfoProcessor) (const GNUNET_HashCode * id,
  * @param list where to store the names (is allocated, caller frees)
  * @return GNUNET_SYSERR on error, otherwise the number of pseudonyms in list
  */
-int GNUNET_ECRS_get_namespaces (struct GNUNET_GE_Context *ectx, struct GNUNET_GC_Configuration *cfg, GNUNET_ECRS_NamespaceInfoProcessor cb, void *cls); /* namespace.c */
+int GNUNET_ECRS_get_namespaces (struct GNUNET_GE_Context *ectx,
+                                struct GNUNET_GC_Configuration *cfg,
+                                GNUNET_ECRS_NamespaceInfoProcessor cb,
+                                void *cls);
 
 /**
  * Add an entry into a namespace.
  *
- * @param name in which namespace to publish, use just the
- *        nickname of the namespace
+ * @param pid in which namespace to publish
  * @param dst to which URI should the namespace entry refer?
  * @param md what meta-data should be associated with the
  *        entry?
+ * @param thisId name of this entry in the namespace (keyword/identifier)
+ * @param nextId name of the update for this entry (to be published in
+ *               the future; maybe NULL)
  * @return URI on success, NULL on error
  */
-struct GNUNET_ECRS_URI *GNUNET_ECRS_namespace_add_content (struct GNUNET_GE_Context *ectx, struct GNUNET_GC_Configuration *cfg, const char *name, unsigned int anonymityLevel, unsigned int priority, GNUNET_CronTime expirationTime, GNUNET_Int32Time creationTime, GNUNET_Int32Time updateInterval, const GNUNET_HashCode * thisId, const GNUNET_HashCode * nextId, const struct GNUNET_ECRS_URI *dst, const struct GNUNET_ECRS_MetaData *md);        /* namespace.c */
+struct GNUNET_ECRS_URI *GNUNET_ECRS_namespace_add_content (struct
+                                                           GNUNET_GE_Context
+                                                           *ectx,
+                                                           struct
+                                                           GNUNET_GC_Configuration
+                                                           *cfg,
+                                                           const
+                                                           GNUNET_HashCode *
+                                                           pid,
+                                                           unsigned int
+                                                           anonymityLevel,
+                                                           unsigned int
+                                                           priority,
+                                                           GNUNET_CronTime
+                                                           expirationTime,
+                                                           const
+                                                           char *thisId,
+                                                           const
+                                                           char *nextId,
+                                                           const struct
+                                                           GNUNET_ECRS_URI
+                                                           *dst,
+                                                           const struct
+                                                           GNUNET_MetaData
+                                                           *md);
 
 /**
  * Add an entry into the K-space (keyword space).
@@ -671,7 +568,14 @@ struct GNUNET_ECRS_URI *GNUNET_ECRS_namespace_add_content (struct GNUNET_GE_Cont
  * @param md what meta-data should be associated with the
  *        entry?
  */
-int GNUNET_ECRS_publish_under_keyword (struct GNUNET_GE_Context *ectx, struct GNUNET_GC_Configuration *cfg, const struct GNUNET_ECRS_URI *uri, unsigned int anonymityLevel, unsigned int priority, GNUNET_CronTime expirationTime, const struct GNUNET_ECRS_URI *dst, const struct GNUNET_ECRS_MetaData *md);   /* keyspace.c */
+int GNUNET_ECRS_publish_under_keyword (struct GNUNET_GE_Context *ectx,
+                                       struct GNUNET_GC_Configuration *cfg,
+                                       const struct GNUNET_ECRS_URI *uri,
+                                       unsigned int anonymityLevel,
+                                       unsigned int priority,
+                                       GNUNET_CronTime expirationTime,
+                                       const struct GNUNET_ECRS_URI *dst,
+                                       const struct GNUNET_MetaData *md);
 
 /**
  * The search has found another result.  Callback to notify
@@ -688,21 +592,69 @@ typedef int (*GNUNET_ECRS_SearchResultProcessor)
   (const GNUNET_ECRS_FileInfo * fi,
    const GNUNET_HashCode * key, int isRoot, void *closure);
 
+struct GNUNET_ECRS_SearchContext;
+
 /**
- * Search for content.
+ * Start search for content (asynchronous version).
+ *
+ * @param sc context to use for searching, you can pass NULL (then
+ *        ECRS will manage its own context); if you pass non-NULL,
+ *        search_stop must be called before you can destroy the sc.
+ * @param uri specifies the search parameters;
+ *        this must be a simple URI (with a single
+ *        keyword)
+ */
+struct GNUNET_ECRS_SearchContext *GNUNET_ECRS_search_start (struct
+                                                            GNUNET_GE_Context
+                                                            *ectx,
+                                                            struct
+                                                            GNUNET_GC_Configuration
+                                                            *cfg,
+                                                            struct
+                                                            GNUNET_FS_SearchContext
+                                                            *sc,
+                                                            const struct
+                                                            GNUNET_ECRS_URI
+                                                            *uri,
+                                                            unsigned int
+                                                            anonymityLevel,
+                                                            GNUNET_ECRS_SearchResultProcessor
+                                                            spcb,
+                                                            void
+                                                            *spcbClosure);
+
+/**
+ * Stop search for content.
  *
- * @param timeout how long to wait (relative)
  * @param uri specifies the search parameters
  * @param uri set to the URI of the uploaded file
  */
-int GNUNET_ECRS_search (struct GNUNET_GE_Context *ectx, struct GNUNET_GC_Configuration *cfg, const struct GNUNET_ECRS_URI *uri, unsigned int anonymityLevel, GNUNET_CronTime timeout, GNUNET_ECRS_SearchResultProcessor spcb, void *spcbClosure, GNUNET_ECRS_TestTerminate tt, void *ttClosure);        /* search.c */
+void GNUNET_ECRS_search_stop (struct GNUNET_ECRS_SearchContext *sctx);
+
+/**
+ * Search for content (synchronous version).
+ *
+ * @param uri specifies the search parameters
+ * @param uri set to the URI of the uploaded file
+ */
+int GNUNET_ECRS_search (struct GNUNET_GE_Context *ectx,
+                        struct GNUNET_GC_Configuration *cfg,
+                        const struct GNUNET_ECRS_URI *uri,
+                        unsigned int anonymityLevel,
+                        GNUNET_ECRS_SearchResultProcessor spcb,
+                        void *spcbClosure,
+                        GNUNET_ECRS_TestTerminate tt, void *ttClosure);
 
 /**
  * Notification of ECRS to a client about the progress of an insertion
  * operation.
  *
  * @param totalBytes number of bytes that will need to be downloaded,
- *        excluding inner blocks
+ *        excluding inner blocks; the value given here will
+ *        be one larger than the requested download size to signal
+ *        an error.  In that case, all other values will be 0,
+ *        except form "lastBlock" which will point to an error
+ *        message describing the problem.
  * @param completedBytes number of bytes that have been obtained
  * @param eta absolute estimated time for the completion of the operation
  * @param lastBlockOffset offset of the last block that was downloaded,
@@ -718,16 +670,69 @@ typedef void (*GNUNET_ECRS_DownloadProgressCallback)
    unsigned long long lastBlockOffset,
    const char *lastBlock, unsigned int lastBlockSize, void *closure);
 
+struct GNUNET_ECRS_DownloadContext;
+
 /**
- * GNUNET_ND_DOWNLOAD a file.
+ * Download parts of a file ASYNCHRONOUSLY.  Note that this will store
+ * the blocks at the respective offset in the given file.  Also, the
+ * download is still using the blocking of the underlying ECRS
+ * encoding.  As a result, the download may *write* outside of the
+ * given boundaries (if offset and length do not match the 32k ECRS
+ * block boundaries).  <p>
+ *
+ * This function should be used to focus a download towards a
+ * particular portion of the file (optimization), not to strictly
+ * limit the download to exactly those bytes.
+ *
+ * @param sc context to use for searching, you can pass NULL (then
+ *        ECRS will manage its own context); if you pass non-NULL,
+ *        partial_stop must be called before you can destroy the sc.
+ * @param uri the URI of the file (determines what to download)
+ * @param filename where to store the file, maybe NULL (then no file is
+ *        created on disk)
+ * @param no_temporaries set to GNUNET_YES to disallow generation of temporary files
+ * @param start starting offset
+ * @param length length of the download (starting at offset)
+ */
+struct GNUNET_ECRS_DownloadContext
+  *GNUNET_ECRS_file_download_partial_start (struct GNUNET_GE_Context *ectx,
+                                            struct GNUNET_GC_Configuration
+                                            *cfg,
+                                            struct GNUNET_FS_SearchContext
+                                            *sc,
+                                            const struct GNUNET_ECRS_URI *uri,
+                                            const char *filename,
+                                            unsigned long long offset,
+                                            unsigned long long length,
+                                            unsigned int anonymityLevel,
+                                            int no_temporaries,
+                                            GNUNET_ECRS_DownloadProgressCallback
+                                            dpcb, void *dpcbClosure);
+
+/**
+ * Stop a download (aborts if download is incomplete).
+ */
+int
+GNUNET_ECRS_file_download_partial_stop (struct GNUNET_ECRS_DownloadContext
+                                        *rm);
+
+/**
+ * DOWNLOAD a file.
  *
  * @param uri the URI of the file (determines what to download)
  * @param filename where to store the file
  */
-int GNUNET_ECRS_file_download (struct GNUNET_GE_Context *ectx, struct GNUNET_GC_Configuration *cfg, const struct GNUNET_ECRS_URI *uri, const char *filename, unsigned int anonymityLevel, GNUNET_ECRS_DownloadProgressCallback dpcb, void *dpcbClosure, GNUNET_ECRS_TestTerminate tt, void *ttClosure); /* download.c */
+int GNUNET_ECRS_file_download (struct GNUNET_GE_Context *ectx,
+                               struct GNUNET_GC_Configuration *cfg,
+                               const struct GNUNET_ECRS_URI *uri,
+                               const char *filename,
+                               unsigned int anonymityLevel,
+                               GNUNET_ECRS_DownloadProgressCallback dpcb,
+                               void *dpcbClosure,
+                               GNUNET_ECRS_TestTerminate tt, void *ttClosure);
 
 /**
- * GNUNET_ND_DOWNLOAD parts of a file.  Note that this will store
+ * DOWNLOAD parts of a file.  Note that this will store
  * the blocks at the respective offset in the given file.
  * Also, the download is still using the blocking of the
  * underlying ECRS encoding.  As a result, the download
@@ -745,7 +750,18 @@ int GNUNET_ECRS_file_download (struct GNUNET_GE_Context *ectx, struct GNUNET_GC_
  * @param start starting offset
  * @param length length of the download (starting at offset)
  */
-int GNUNET_ECRS_file_download_partial (struct GNUNET_GE_Context *ectx, struct GNUNET_GC_Configuration *cfg, const struct GNUNET_ECRS_URI *uri, const char *filename, unsigned long long offset, unsigned long long length, unsigned int anonymityLevel, int no_temporaries, GNUNET_ECRS_DownloadProgressCallback dpcb, void *dpcbClosure, GNUNET_ECRS_TestTerminate tt, void *ttClosure);       /* download.c */
+int GNUNET_ECRS_file_download_partial (struct GNUNET_GE_Context *ectx,
+                                       struct GNUNET_GC_Configuration *cfg,
+                                       const struct GNUNET_ECRS_URI *uri,
+                                       const char *filename,
+                                       unsigned long long offset,
+                                       unsigned long long length,
+                                       unsigned int anonymityLevel,
+                                       int no_temporaries,
+                                       GNUNET_ECRS_DownloadProgressCallback
+                                       dpcb, void *dpcbClosure,
+                                       GNUNET_ECRS_TestTerminate tt,
+                                       void *ttClosure);
 
 /**
  * Iterate over all entries in a directory.  Note that directories
@@ -761,7 +777,12 @@ int GNUNET_ECRS_file_download_partial (struct GNUNET_GE_Context *ectx, struct GN
  * @return number of entries on success, GNUNET_SYSERR if the
  *         directory is malformed
  */
-int GNUNET_ECRS_directory_list_contents (struct GNUNET_GE_Context *ectx, const char *data, unsigned long long len, struct GNUNET_ECRS_MetaData **md, GNUNET_ECRS_SearchResultProcessor spcb, void *spcbClosure);        /* directory.c */
+int GNUNET_ECRS_directory_list_contents (struct GNUNET_GE_Context *ectx,
+                                         const char *data,
+                                         unsigned long long len,
+                                         struct GNUNET_MetaData **md,
+                                         GNUNET_ECRS_SearchResultProcessor
+                                         spcb, void *spcbClosure);
 
 /**
  * Create a directory.
@@ -776,7 +797,13 @@ int GNUNET_ECRS_directory_list_contents (struct GNUNET_GE_Context *ectx, const c
  *        is extended with the mime-type for a GNUnet directory.
  * @return GNUNET_OK on success, GNUNET_SYSERR on error
  */
-int GNUNET_ECRS_directory_create (struct GNUNET_GE_Context *ectx, char **data, unsigned long long *len, unsigned int count, const GNUNET_ECRS_FileInfo * fis, struct GNUNET_ECRS_MetaData *meta);       /* directory.c */
+int GNUNET_ECRS_directory_create (struct GNUNET_GE_Context *ectx,
+                                  char **data,
+                                  unsigned long long *len,
+                                  unsigned int count,
+                                  const GNUNET_ECRS_FileInfo * fis,
+                                  struct GNUNET_MetaData *meta);
+
 
 #if 0                           /* keep Emacsens' auto-indent happy */
 {
