@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet
-     (C) 2004, 2005, 2006 Christian Grothoff (and other contributing authors)
+     (C) 2004, 2005, 2006, 2008 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -56,17 +56,18 @@
  * may want to open a window informing the user about the pending
  * shutdown operation.<p>
  *
- * Any "startXXX" operation will result in FSUI state and memory
- * being allocated until it is paired with a "stopXXX" operation.
- * Before calling "stopXXX", one of three things must happen:
+ * Any "start" operation will result in FSUI state and memory
+ * being allocated until it is paired with a "stop" operation.
+ * Before calling "stop", one of three things must happen:
  * Either, the client receives an "error" (something went wrong)
  * or "completed" (action finished) event.  Alternatively, the
- * client may call abortXXX" which will result in an "aborted"
+ * client may call "abort" which will result in an "aborted"
  * event.  In either case, the event itself will NOT result in
  * the memory being released by FSUI -- the client must still
- * call "GNUNET_FSUI_stopXXX" explicitly.  Clients that call
- * "GNUNET_FSUI_stopXXX" before an aborted, error or completed event
- * will be blocked until either of the three events happens.<p>
+ * call "stop" explicitly.  Clients that call "stop" before an
+ * aborted, error or completed event will be blocked until
+ * an error or completion event happens (it is illegal for a
+ * client to call "abort" after calling "stop").<p>
  *
  * Using the Event mechanism, clients can associate an arbitrary
  * pointer with any operation (upload, download, search or
@@ -76,9 +77,6 @@
  * that memory when suspend or stop events are issued.  For all
  * events (other than start/resume), FSUI will track and provide
  * the client pointer as part of the event (cctx field).<p>
- *
- * Note that most of this code is completely new in GNUnet 0.7.0 and
- * thus still highly experimental.  Suggestions are welcome.<p>
  */
 
 #ifndef GNUNET_FSUI_LIB_H
@@ -94,6 +92,11 @@ extern "C"
 #endif
 #endif
 
+/**
+ * How many probes do we do at most per search?
+ */
+#define GNUNET_FSUI_MAX_PROBES 10
+
 /**
  * Entry representing an FSUI download.  FSUI downloads form a tree
  * (for properly representing recursive downloads) with an invisible
@@ -129,11 +132,12 @@ enum GNUNET_FSUI_EventType
   GNUNET_FSUI_search_started,
   GNUNET_FSUI_search_stopped,
   GNUNET_FSUI_search_result,
-  GNUNET_FSUI_search_completed,
   GNUNET_FSUI_search_aborted,
-  GNUNET_FSUI_search_error,
   GNUNET_FSUI_search_suspended,
   GNUNET_FSUI_search_resumed,
+  GNUNET_FSUI_search_paused,
+  GNUNET_FSUI_search_restarted,
+  GNUNET_FSUI_search_update,
   GNUNET_FSUI_download_started,
   GNUNET_FSUI_download_stopped,
   GNUNET_FSUI_download_progress,
@@ -172,6 +176,10 @@ enum GNUNET_FSUI_EventType
  * ACTIVE means that there is currently a thread running
  * the download (and that thread is allowed to continue).<br>
  *
+ * PAUSED means that the user asked to pause the activity
+ * (and that it is to be resumed once the user explicitly
+ * asks for it to be resumed).
+ *
  * COMPLETED means that the download is finished (but the
  * thread has not been joined yet).  The download thread
  * makes the transition from PENDING to COMPLETED when it
@@ -210,13 +218,14 @@ typedef enum
 {
   GNUNET_FSUI_PENDING = 0,
   GNUNET_FSUI_ACTIVE = 1,
-  GNUNET_FSUI_COMPLETED = 2,
-  GNUNET_FSUI_COMPLETED_JOINED = 3,
-  GNUNET_FSUI_ABORTED = 4,
-  GNUNET_FSUI_ABORTED_JOINED = 5,
-  GNUNET_FSUI_ERROR = 6,
-  GNUNET_FSUI_ERROR_JOINED = 7,
-  GNUNET_FSUI_SUSPENDING = 8,
+  GNUNET_FSUI_PAUSED = 2,
+  GNUNET_FSUI_COMPLETED = 3,
+  GNUNET_FSUI_COMPLETED_JOINED = 4,
+  GNUNET_FSUI_ABORTED = 5,
+  GNUNET_FSUI_ABORTED_JOINED = 6,
+  GNUNET_FSUI_ERROR = 7,
+  GNUNET_FSUI_ERROR_JOINED = 8,
+  GNUNET_FSUI_SUSPENDING = 9,
 } GNUNET_FSUI_State;
 
 /**
@@ -325,14 +334,6 @@ typedef struct
 
     } SearchResult;
 
-
-    struct
-    {
-
-      GNUNET_FSUI_SearchContext sc;
-
-    } SearchCompleted;
-
     struct
     {
 
@@ -340,15 +341,6 @@ typedef struct
 
     } SearchAborted;
 
-    struct
-    {
-
-      GNUNET_FSUI_SearchContext sc;
-
-      const char *message;
-
-    } SearchError;
-
     struct
     {
 
@@ -361,14 +353,43 @@ typedef struct
 
       GNUNET_FSUI_SearchContext sc;
 
+      /**
+       * URI used for this search.
+       */
       struct GNUNET_ECRS_URI *searchURI;
 
+      /**
+       * Array of search results.
+       */
       const GNUNET_ECRS_FileInfo *fis;
 
-      unsigned int anonymityLevel;
+      /**
+       * Updated availability rank (negative:
+       * unavailable, positive: available)
+       */
+      int *availability_rank;
 
+      /**
+       * On how many total queries is the given
+       * availability rank based?
+       */
+      unsigned int *availability_certainty;
+
+      /**
+       * Updated applicability rank (the larger,
+       * the better the result fits the search
+       * criteria).
+       */
+      unsigned int *applicability_rank;
+
+      /**
+       * Size of the fis, availability and
+       * applicability arrays.
+       */
       unsigned int fisSize;
 
+      unsigned int anonymityLevel;
+
       GNUNET_FSUI_State state;
 
     } SearchResumed;
@@ -391,6 +412,59 @@ typedef struct
 
     } SearchStopped;
 
+    struct
+    {
+
+      GNUNET_FSUI_SearchContext sc;
+
+    } SearchPaused;
+
+    struct
+    {
+
+      GNUNET_FSUI_SearchContext sc;
+
+    } SearchRestarted;
+
+
+    struct
+    {
+
+      GNUNET_FSUI_SearchContext sc;
+
+      /**
+       * File-Info of the data for which we have
+       * updated ranking information.
+       */
+      GNUNET_ECRS_FileInfo fi;
+
+      /**
+       * The URI of the search for which we provide
+       * the updated ranking information.
+       */
+      const struct GNUNET_ECRS_URI *searchURI;
+
+      /**
+       * Updated availability rank (negative:
+       * unavailable, positive: available)
+       */
+      int availability_rank;
+
+      /**
+       * On how many total queries is the given
+       * availability rank based?
+       */
+      unsigned int availability_certainty;
+
+      /**
+       * Updated applicability rank (the larger,
+       * the better the result fits the search
+       * criteria).
+       */
+      unsigned int applicability_rank;
+
+    } SearchUpdate;
+
 
 
     struct
@@ -810,6 +884,10 @@ struct GNUNET_FSUI_Context;
  * FSUI guarantees is that only one thread at a time will call the
  * callback (so it need not be re-entrant).<p>
  *
+ * The code in the callback is NOT allowed to start, abort, stop,
+ * resume or cancel any pending FSUI operation.  Doing so may result
+ * in deadlock or other kinds of bad behavior.
+ *
  * @return cctx for resume events, otherwise NULL
  */
 typedef void *(*GNUNET_FSUI_EventProcessor) (void *cls,
@@ -846,7 +924,13 @@ typedef void *(*GNUNET_FSUI_EventProcessor) (void *cls,
  * @param closure extra argument to cb
  * @return NULL on error
  */
-struct GNUNET_FSUI_Context *GNUNET_FSUI_start (struct GNUNET_GE_Context *ectx, struct GNUNET_GC_Configuration *cfg, const char *name, unsigned int threadPoolSize, int doResume, GNUNET_FSUI_EventProcessor cb, void *closure); /* fsui.c */
+struct GNUNET_FSUI_Context *GNUNET_FSUI_start (struct GNUNET_GE_Context *ectx,
+                                               struct GNUNET_GC_Configuration
+                                               *cfg, const char *name,
+                                               unsigned int threadPoolSize,
+                                               int doResume,
+                                               GNUNET_FSUI_EventProcessor cb,
+                                               void *closure);
 
 /**
  * Stop all processes under FSUI control (may serialize
@@ -856,42 +940,86 @@ struct GNUNET_FSUI_Context *GNUNET_FSUI_start (struct GNUNET_GE_Context *ectx, s
  */
 void GNUNET_FSUI_stop (struct GNUNET_FSUI_Context *ctx);        /* fsui.c */
 
-
 /**
  * Start a search.
  *
  * @return NULL on error
  */
-struct GNUNET_FSUI_SearchList *GNUNET_FSUI_search_start (struct GNUNET_FSUI_Context *ctx, unsigned int anonymityLevel, unsigned int maxResults, GNUNET_CronTime timeout, const struct GNUNET_ECRS_URI *uri);    /* search.c */
+struct GNUNET_FSUI_SearchList *GNUNET_FSUI_search_start (struct
+                                                         GNUNET_FSUI_Context
+                                                         *ctx,
+                                                         unsigned int
+                                                         anonymityLevel,
+                                                         const struct
+                                                         GNUNET_ECRS_URI
+                                                         *uri);
 
 /**
  * Abort a search.
  *
  * @return GNUNET_SYSERR if such a search is not known
  */
-int GNUNET_FSUI_search_abort (struct GNUNET_FSUI_Context *ctx, struct GNUNET_FSUI_SearchList *sl);      /* search.c */
+int GNUNET_FSUI_search_abort (struct GNUNET_FSUI_SearchList *sl);       /* search.c */
+
+/**
+ * Pause a search.
+ *
+ * @return GNUNET_SYSERR if such a search is not known
+ */
+int GNUNET_FSUI_search_pause (struct GNUNET_FSUI_SearchList *sl);       /* search.c */
+
+/**
+ * Restart a paused search.
+ *
+ * @return GNUNET_SYSERR if such a search is not known
+ */
+int GNUNET_FSUI_search_restart (struct GNUNET_FSUI_SearchList *sl);     /* search.c */
 
 /**
  * Stop a search.
  *
  * @return GNUNET_SYSERR if such a search is not known
  */
-int GNUNET_FSUI_search_stop (struct GNUNET_FSUI_Context *ctx, struct GNUNET_FSUI_SearchList *sl);       /* search.c */
+int GNUNET_FSUI_search_stop (struct GNUNET_FSUI_SearchList *sl);        /* search.c */
 
 /**
  * Start to download a file or directory.
  *
  * @return NULL on error
  */
-struct GNUNET_FSUI_DownloadList *GNUNET_FSUI_download_start (struct GNUNET_FSUI_Context *ctx, unsigned int anonymityLevel, int doRecursive, const struct GNUNET_ECRS_URI *uri, const struct GNUNET_ECRS_MetaData *meta, const char *filename, struct GNUNET_FSUI_SearchList *parentSearch, struct GNUNET_FSUI_DownloadList *parentDownload);    /* download.c */
+struct GNUNET_FSUI_DownloadList *GNUNET_FSUI_download_start (struct
+                                                             GNUNET_FSUI_Context
+                                                             *ctx,
+                                                             unsigned int
+                                                             anonymityLevel,
+                                                             int doRecursive,
+                                                             const struct
+                                                             GNUNET_ECRS_URI
+                                                             *uri,
+                                                             const struct
+                                                             GNUNET_MetaData
+                                                             *meta,
+                                                             const char
+                                                             *filename,
+                                                             struct
+                                                             GNUNET_FSUI_SearchList
+                                                             *parentSearch,
+                                                             struct
+                                                             GNUNET_FSUI_DownloadList
+                                                             *parentDownload);
 
 /**
- * Abort a download.  If the dl is for a recursive
- * download, all sub-downloads will also be aborted.
+ * Abort a download.  If the dl is for a recursive download, all
+ * sub-downloads will also be aborted.  This will also delete all of
+ * the files associated with the download (except if the download has
+ * already completed, in which case GNUNET_NO will be returned).  If
+ * this is a recursive download and some files have been completed,
+ * these files will not be removed (only incomplete downloads will be
+ * removed).
  *
  * @return GNUNET_SYSERR on error
  */
-int GNUNET_FSUI_download_abort (struct GNUNET_FSUI_Context *ctx, struct GNUNET_FSUI_DownloadList *dl);  /* download.c */
+int GNUNET_FSUI_download_abort (struct GNUNET_FSUI_DownloadList *dl);   /* download.c */
 
 /**
  * Stop a download.  If the dl is for a recursive
@@ -899,7 +1027,7 @@ int GNUNET_FSUI_download_abort (struct GNUNET_FSUI_Context *ctx, struct GNUNET_F
  *
  * @return GNUNET_SYSERR on error
  */
-int GNUNET_FSUI_download_stop (struct GNUNET_FSUI_Context *ctx, struct GNUNET_FSUI_DownloadList *dl);   /* download.c */
+int GNUNET_FSUI_download_stop (struct GNUNET_FSUI_DownloadList *dl);    /* download.c */
 
 /**
  * Method that can be used to select files that
@@ -920,7 +1048,7 @@ typedef int (*GNUNET_FSUI_DirectoryScanCallback) (void *data,
  * @param ctx
  * @param filename name of file or directory to upload (directory
  *        implies use of recursion)
- * @param doIndex use indexing, not insertion
+ * @param doIndex GNUNET_YES: use indexing; GNUNET_NO: use insertion; GNUNET_SYSERR: "simulate"
  * @param doExtract use libextractor
  * @param individualKeywords add KBlocks for non-top-level files
  * @param topLevelMetaData metadata for top-level file or directory
@@ -946,7 +1074,7 @@ struct GNUNET_FSUI_UploadList *GNUNET_FSUI_upload_start (struct
                                                          GNUNET_CronTime
                                                          expiration,
                                                          const struct
-                                                         GNUNET_ECRS_MetaData
+                                                         GNUNET_MetaData
                                                          *topLevelMetaData,
                                                          const struct
                                                          GNUNET_ECRS_URI
@@ -962,8 +1090,7 @@ struct GNUNET_FSUI_UploadList *GNUNET_FSUI_upload_start (struct
  *
  * @return GNUNET_SYSERR on error
  */
-int GNUNET_FSUI_upload_abort (struct GNUNET_FSUI_Context *ctx,
-                              struct GNUNET_FSUI_UploadList *ul);
+int GNUNET_FSUI_upload_abort (struct GNUNET_FSUI_UploadList *ul);
 
 /**
  * Stop an upload.  Only to be called for the top-level
@@ -971,8 +1098,7 @@ int GNUNET_FSUI_upload_abort (struct GNUNET_FSUI_Context *ctx,
  *
  * @return GNUNET_SYSERR on error
  */
-int GNUNET_FSUI_upload_stop (struct GNUNET_FSUI_Context *ctx,
-                             struct GNUNET_FSUI_UploadList *ul);
+int GNUNET_FSUI_upload_stop (struct GNUNET_FSUI_UploadList *ul);
 
 
 /**
@@ -993,8 +1119,7 @@ struct GNUNET_FSUI_UnindexList *GNUNET_FSUI_unindex_start (struct
  *
  * @return GNUNET_SYSERR on error
  */
-int GNUNET_FSUI_unindex_abort (struct GNUNET_FSUI_Context *ctx,
-                               struct GNUNET_FSUI_UnindexList *ul);
+int GNUNET_FSUI_unindex_abort (struct GNUNET_FSUI_UnindexList *ul);
 
 
 /**
@@ -1002,8 +1127,7 @@ int GNUNET_FSUI_unindex_abort (struct GNUNET_FSUI_Context *ctx,
  *
  * @return GNUNET_SYSERR on error
  */
-int GNUNET_FSUI_unindex_stop (struct GNUNET_FSUI_Context *ctx,
-                              struct GNUNET_FSUI_UnindexList *ul);
+int GNUNET_FSUI_unindex_stop (struct GNUNET_FSUI_UnindexList *ul);
 
 
 #if 0                           /* keep Emacsens' auto-indent happy */
