@@ -199,6 +199,8 @@ int GNUNET_set_process_priority (struct GNUNET_GE_Context *ectx,
  *           enabled
  * @param doAutoStart GNUNET_YES to enable autostart of the
  *        application, GNUNET_NO to disable it
+ * @param servicename name of the service as displayed by the OS
+ * @param application path to service binary
  * @param username name of the user account to use
  * @param groupname name of the group to use
  * @returns GNUNET_YES on success, GNUNET_NO if unsupported, GNUNET_SYSERR on failure or one of
@@ -212,11 +214,13 @@ int GNUNET_set_process_priority (struct GNUNET_GE_Context *ectx,
  *  Unix
  *    2 startup script could not be opened
  */
-int GNUNET_configure_autostart (struct GNUNET_GE_Context *ectx,
-                                int testCapability,
-                                int doAutoStart,
-                                const char *application,
-                                const char *username, const char *groupname);
+int
+GNUNET_configure_autostart (struct GNUNET_GE_Context *ectx,
+                            int testCapability,
+                            int doAutoStart,
+                            const char *servicename,
+                            const char *application,
+                            const char *username, const char *groupname);
 
 /**
  * @brief Add or remove a service account for GNUnet
@@ -329,27 +333,48 @@ char *GNUNET_get_installation_path (enum GNUNET_INSTALL_PATH_KIND dirkind);
  */
 int GNUNET_pid_file_write (struct GNUNET_GE_Context *ectx,
                            struct GNUNET_GC_Configuration *cfg,
-                           unsigned int pid);
+                           unsigned int pid,
+                           const char *section,
+                           const char *value, const char *def);
 
 /**
  * Delete the PID file (to be called when the daemon
  * shuts down)
  */
 int GNUNET_pid_file_delete (struct GNUNET_GE_Context *ectx,
-                            struct GNUNET_GC_Configuration *cfg);
+                            struct GNUNET_GC_Configuration *cfg,
+                            const char *section,
+                            const char *value, const char *def);
+
+/**
+ * Kill the process that wrote the current PID file
+ * (and try to wait for it to terminate).
+ * 
+ * @return GNUNET_OK kill successful
+ *         GNUNET_NO no PID file exists
+ *         GNUNET_SYSERR kill seems to have failed
+ */
+int GNUNET_pid_file_kill_owner (struct GNUNET_GE_Context *ectx,
+                                struct GNUNET_GC_Configuration *cfg,
+                                const char *section,
+                                const char *value, const char *def);
 
 
 /**
  * Fork and start a new session to go into the background
- * in the way a good deamon should.  Also writes the PID
+ * in the way a good daemon should.  Also writes the PID
  * file.
  *
+ * @param section section in the configuration for the PID filename
+ * @param value value in the configuration for the PID filename
  * @param filedes pointer to an array of 2 file descriptors
  *        to complete the detachment protocol (handshake)
  */
 int GNUNET_terminal_detach (struct GNUNET_GE_Context *ectx,
                             struct GNUNET_GC_Configuration *cfg,
-                            int *filedes);
+                            int *filedes,
+                            const char *section,
+                            const char *value, const char *def);
 
 /**
  * Complete the handshake of detaching from the terminal.
