@@ -20,7 +20,7 @@
 
 /**
  * @file server/tcpserver.c
- * @brief TCP server (gnunetd-client communication using util/tcpio.c).
+ * @brief TCP server (gnunetd-client communication using util/network_client/tcpio.c).
  * @author Christian Grothoff
  *
  * TODO: configuration management (signaling of configuration change)
@@ -86,15 +86,25 @@ typedef struct GNUNET_ClientHandle
 /**
  * Configuration...
  */
-static struct GNUNET_IPv4NetworkSet *trustedNetworks_ = NULL;
+static struct GNUNET_IPv4NetworkSet *trustedNetworksV4;
+static struct GNUNET_IPv6NetworkSet *trustedNetworksV6;
 
 /**
  * Is this IP labeled as trusted for CS connections?
  */
 static int
-isWhitelisted (GNUNET_IPv4Address ip)
+isWhitelisted4 (const struct in_addr *ip)
 {
-  return GNUNET_check_ipv4_listed (trustedNetworks_, ip);
+  return GNUNET_check_ipv4_listed (trustedNetworksV4, ip);
+}
+
+/**
+ * Is this IP labeled as trusted for CS connections?
+ */
+static int
+isWhitelisted6 (const struct in6_addr *ip)
+{
+  return GNUNET_check_ipv6_listed (trustedNetworksV6, ip);
 }
 
 static int
@@ -158,15 +168,45 @@ select_accept_handler (void *ah_cls,
                        const void *addr, unsigned int addr_len)
 {
   struct GNUNET_ClientHandle *session;
-  GNUNET_IPv4Address ip;
-  struct sockaddr_in *a;
+  struct in_addr ip4;
+  struct in6_addr ip6;
+  struct sockaddr_in *a4;
+  struct sockaddr_in6 *a6;
 
-  if (addr_len != sizeof (struct sockaddr_in))
-    return NULL;
-  a = (struct sockaddr_in *) addr;
-  memcpy (&ip, &a->sin_addr, sizeof (GNUNET_IPv4Address));
-  if (!isWhitelisted (ip))
-    return NULL;
+  if (addr_len == sizeof (struct sockaddr_in6))
+    {
+      a6 = (struct sockaddr_in6 *) addr;
+
+      memcpy (&ip6, &a6->sin6_addr, sizeof (struct in6_addr));
+      /* get embedded ipv4 address in case address embedding is used */
+      memcpy (&ip4,
+              &((char *) &ip6)[sizeof (struct in6_addr) -
+                               sizeof (struct in_addr)],
+              sizeof (struct in_addr));
+      if ((!isWhitelisted6 (&ip6))
+          &&
+          (!(((IN6_IS_ADDR_V4COMPAT (&a6->sin6_addr))
+              || (IN6_IS_ADDR_V4MAPPED (&a6->sin6_addr)))
+             && (isWhitelisted4 (&ip4)))))
+        {
+          GNUNET_GE_LOG (ectx,
+                         GNUNET_GE_DEBUG | GNUNET_GE_ADMIN | GNUNET_GE_BULK,
+                         "Rejected connection from untrusted client");
+          return NULL;
+        }
+    }
+  else if (addr_len == sizeof (struct sockaddr_in))
+    {
+      a4 = (struct sockaddr_in *) addr;
+      memcpy (&ip4, &a4->sin_addr, sizeof (struct in_addr));
+      if (!isWhitelisted4 (&ip4))
+        return NULL;
+    }
+  else
+    {
+      GNUNET_GE_BREAK (NULL, 0);
+      return NULL;
+    }
   session = GNUNET_malloc (sizeof (ClientHandle));
   session->sock = sock;
   return session;
@@ -201,15 +241,18 @@ GNUNET_CORE_cs_send_to_client (struct GNUNET_ClientHandle *handle,
                                const GNUNET_MessageHeader * message,
                                int force)
 {
-#if DEBUG_TCPHANDLER
-  GNUNET_GE_LOG (ectx,
-                 GNUNET_GE_DEBUG | GNUNET_GE_DEVELOPER | GNUNET_GE_REQUEST,
-                 "%s: sending reply to client\n", __FUNCTION__);
-#endif
-  return GNUNET_select_write (selector, handle->sock, message, GNUNET_YES,
+  return GNUNET_select_write (selector, handle->sock, message, GNUNET_NO,
                               force);
 }
 
+int
+GNUNET_CORE_cs_test_send_to_client_now (struct GNUNET_ClientHandle *handle,
+                                        unsigned int size, int force)
+{
+  return GNUNET_select_test_write_now (selector, handle->sock,
+                                       size, GNUNET_NO, force);
+}
+
 void
 GNUNET_CORE_cs_terminate_client_connection (struct GNUNET_ClientHandle *sock)
 {
@@ -231,50 +274,36 @@ select_message_handler (void *mh_cls,
 
   ptyp = htons (msg->type);
   GNUNET_mutex_lock (handlerlock);
-  if (ptyp >= max_registeredType)
+  if ((ptyp >= max_registeredType) || (NULL == (callback = handlers[ptyp])))
     {
       GNUNET_GE_LOG (ectx,
                      GNUNET_GE_INFO | GNUNET_GE_USER | GNUNET_GE_BULK,
-                     "%s: Message of type %d not understood: no handler registered\n",
-                     __FUNCTION__, ptyp, max_registeredType);
+                     "Message of type %d not understood: no handler registered\n",
+                     ptyp);
       GNUNET_mutex_unlock (handlerlock);
       return GNUNET_SYSERR;
     }
-  callback = handlers[ptyp];
-  if (callback == NULL)
-    {
-      GNUNET_GE_LOG (ectx,
-                     GNUNET_GE_INFO | GNUNET_GE_USER | GNUNET_GE_BULK,
-                     "%s: Message of type %d not understood: no handler registered\n",
-                     __FUNCTION__, ptyp);
-      GNUNET_mutex_unlock (handlerlock);
-      return GNUNET_SYSERR;
-    }
-  else
-    {
 #if TIME_HANDLERS
-      start = GNUNET_get_time ();
+  start = GNUNET_get_time ();
 #endif
-      if (GNUNET_OK != callback (sender, msg))
-        {
+  if (GNUNET_OK != callback (sender, msg))
+    {
 #if 0
-          GNUNET_GE_LOG (ectx,
-                         GNUNET_GE_INFO | GNUNET_GE_USER | GNUNET_GE_BULK,
-                         "%s: Message of type %d caused error in handler\n",
-                         __FUNCTION__, ptyp);
-#endif
-          GNUNET_mutex_unlock (handlerlock);
-          return GNUNET_SYSERR;
-        }
-#if TIME_HANDLERS
-      if (GNUNET_get_time () - start > GNUNET_CRON_SECONDS)
-        GNUNET_GE_LOG (ectx,
-                       GNUNET_GE_INFO | GNUNET_GE_DEVELOPER |
-                       GNUNET_GE_IMMEDIATE,
-                       "Handling message of type %u took %llu s\n", ptyp,
-                       (GNUNET_get_time () - start) / GNUNET_CRON_SECONDS);
+      GNUNET_GE_LOG (ectx,
+                     GNUNET_GE_INFO | GNUNET_GE_USER | GNUNET_GE_BULK,
+                     "Message of type %d caused error in handler\n", ptyp);
 #endif
+      GNUNET_mutex_unlock (handlerlock);
+      return GNUNET_SYSERR;
     }
+#if TIME_HANDLERS
+  if (GNUNET_get_time () - start > GNUNET_CRON_SECONDS)
+    GNUNET_GE_LOG (ectx,
+                   GNUNET_GE_INFO | GNUNET_GE_DEVELOPER |
+                   GNUNET_GE_IMMEDIATE,
+                   "Handling message of type %u took %llu s\n", ptyp,
+                   (GNUNET_get_time () - start) / GNUNET_CRON_SECONDS);
+#endif
   GNUNET_mutex_unlock (handlerlock);
   return GNUNET_OK;
 }
@@ -300,32 +329,51 @@ startTCPServer ()
 {
   int listenerFD;
   int listenerPort;
-  struct sockaddr_in serverAddr;
+  struct sockaddr_in6 serverAddr6;
+  struct sockaddr_in serverAddr4;
+  struct sockaddr *serverAddr;
+  socklen_t socklen;
   const int on = 1;
 
   listenerPort = getGNUnetPort ();
   if (listenerPort == 0)
     return GNUNET_SYSERR;
-  listenerFD = SOCKET (PF_INET, SOCK_STREAM, 0);
-  if (listenerFD < 0)
+  if ((GNUNET_YES ==
+       GNUNET_GC_get_configuration_value_yesno (cfg, "GNUNETD",
+                                                "DISABLE-IPV6", GNUNET_YES))
+      || (-1 == (listenerFD = SOCKET (PF_INET6, SOCK_STREAM, 0))))
     {
-      GNUNET_GE_LOG_STRERROR (ectx,
-                              GNUNET_GE_FATAL | GNUNET_GE_ADMIN |
-                              GNUNET_GE_USER | GNUNET_GE_IMMEDIATE, "socket");
-      return GNUNET_SYSERR;
+      listenerFD = SOCKET (PF_INET, SOCK_STREAM, 0);
+      if (listenerFD < 0)
+        {
+          GNUNET_GE_LOG_STRERROR (ectx,
+                                  GNUNET_GE_FATAL | GNUNET_GE_ADMIN |
+                                  GNUNET_GE_USER | GNUNET_GE_IMMEDIATE,
+                                  "socket");
+          return GNUNET_SYSERR;
+        }
+      memset (&serverAddr4, 0, sizeof (serverAddr4));
+      serverAddr4.sin_family = AF_INET;
+      serverAddr4.sin_addr.s_addr = htonl (INADDR_ANY);
+      serverAddr4.sin_port = htons (listenerPort);
+      socklen = sizeof (serverAddr4);
+      serverAddr = (struct sockaddr *) &serverAddr4;
+    }
+  else
+    {
+      memset (&serverAddr6, 0, sizeof (serverAddr6));
+      serverAddr6.sin6_family = AF_INET6;
+      serverAddr6.sin6_port = htons (listenerPort);
+      socklen = sizeof (serverAddr6);
+      serverAddr = (struct sockaddr *) &serverAddr6;
     }
   /* fill in the inet address structure */
-  memset (&serverAddr, 0, sizeof (serverAddr));
-  serverAddr.sin_family = AF_INET;
-  serverAddr.sin_addr.s_addr = htonl (INADDR_ANY);
-  serverAddr.sin_port = htons (listenerPort);
   if (SETSOCKOPT (listenerFD, SOL_SOCKET, SO_REUSEADDR, &on, sizeof (on)) < 0)
     GNUNET_GE_LOG_STRERROR (ectx,
                             GNUNET_GE_ERROR | GNUNET_GE_ADMIN |
                             GNUNET_GE_BULK, "setsockopt");
   /* bind the socket */
-  if (BIND (listenerFD,
-            (struct sockaddr *) &serverAddr, sizeof (serverAddr)) < 0)
+  if (BIND (listenerFD, serverAddr, socklen) < 0)
     {
       GNUNET_GE_LOG_STRERROR (ectx,
                               GNUNET_GE_ERROR | GNUNET_GE_ADMIN |
@@ -339,7 +387,7 @@ startTCPServer ()
       CLOSE (listenerFD);
       return GNUNET_SYSERR;
     }
-  selector = GNUNET_select_create ("tcpserver", GNUNET_NO, ectx, NULL, listenerFD, sizeof (struct sockaddr_in), 0,      /* no timeout */
+  selector = GNUNET_select_create ("tcpserver", GNUNET_NO, ectx, NULL, listenerFD, socklen, 0,  /* no timeout */
                                    &select_message_handler,
                                    NULL,
                                    &select_accept_handler,
@@ -369,7 +417,8 @@ GNUNET_CORE_cs_done ()
                                   &shutdownHandler);
   GNUNET_array_grow (handlers, max_registeredType, 0);
   GNUNET_array_grow (exitHandlers, exitHandlerCount, 0);
-  GNUNET_free (trustedNetworks_);
+  GNUNET_free_non_null (trustedNetworksV4);
+  GNUNET_free_non_null (trustedNetworksV6);
   return GNUNET_OK;
 }
 
@@ -404,8 +453,8 @@ GNUNET_CORE_cs_init (struct GNUNET_GE_Context *e,
                                                       "127.0.0.0/8;", &ch))
     return GNUNET_SYSERR;
   GNUNET_GE_ASSERT (ectx, ch != NULL);
-  trustedNetworks_ = GNUNET_parse_ipv4_network_specification (ectx, ch);
-  if (trustedNetworks_ == NULL)
+  trustedNetworksV4 = GNUNET_parse_ipv4_network_specification (ectx, ch);
+  if (trustedNetworksV4 == NULL)
     {
       GNUNET_GE_LOG (ectx,
                      GNUNET_GE_FATAL | GNUNET_GE_USER | GNUNET_GE_ADMIN |
@@ -418,6 +467,32 @@ GNUNET_CORE_cs_init (struct GNUNET_GE_Context *e,
     }
   GNUNET_free (ch);
 
+  if (GNUNET_YES !=
+      GNUNET_GC_get_configuration_value_yesno (cfg, "GNUNETD", "DISABLE-IPV6",
+                                               GNUNET_YES))
+    {
+      ch = NULL;
+      if (-1 == GNUNET_GC_get_configuration_value_string (cfg,
+                                                          "NETWORK",
+                                                          "TRUSTED6",
+                                                          "::1;", &ch))
+        return GNUNET_SYSERR;
+      GNUNET_GE_ASSERT (ectx, ch != NULL);
+      trustedNetworksV6 = GNUNET_parse_ipv6_network_specification (ectx, ch);
+      if (trustedNetworksV6 == NULL)
+        {
+          GNUNET_GE_LOG (ectx,
+                         GNUNET_GE_FATAL | GNUNET_GE_USER | GNUNET_GE_ADMIN |
+                         GNUNET_GE_IMMEDIATE,
+                         _
+                         ("Malformed network specification in the configuration in section `%s' for entry `%s': %s\n"),
+                         "NETWORK", "TRUSTED6", ch);
+          GNUNET_free (ch);
+          return GNUNET_SYSERR;
+        }
+      GNUNET_free (ch);
+    }
+
   GNUNET_CORE_register_handler (GNUNET_CS_PROTO_SHUTDOWN_REQUEST,
                                 &shutdownHandler);
   if ((GNUNET_NO ==
@@ -469,8 +544,9 @@ GNUNET_CORE_register_handler (unsigned short type,
           GNUNET_GE_LOG (ectx,
                          GNUNET_GE_WARNING | GNUNET_GE_DEVELOPER |
                          GNUNET_GE_BULK,
-                         _("%s failed, message type %d already in use.\n"),
-                         __FUNCTION__, type);
+                         _
+                         ("Registering failed, message type %d already in use.\n"),
+                         type);
           return GNUNET_SYSERR;
         }
     }
@@ -561,7 +637,8 @@ GNUNET_CORE_cs_send_error_to_client (struct GNUNET_ClientHandle *sock,
     msgLen = 60000;
   rv = GNUNET_malloc (sizeof (GNUNET_MessageReturnErrorMessage) + msgLen);
   memset (rv, 0, sizeof (GNUNET_MessageReturnErrorMessage) + msgLen);
-  rv->header.size = htons (sizeof (GNUNET_MessageHeader) + msgLen);
+  rv->header.size =
+    htons (sizeof (GNUNET_MessageReturnErrorMessage) + msgLen);
   rv->header.type = htons (GNUNET_CS_PROTO_RETURN_ERROR);
   rv->kind = htonl (kind);
   memcpy (&rv[1], message, strlen (message));
@@ -611,10 +688,11 @@ logClientLogContext (void *ctx,
 }
 
 struct GNUNET_GE_Context *
-GNUNET_CORE_cs_create_client_log_context (GNUNET_GE_KIND mask,
-                                          struct GNUNET_ClientHandle *handle)
+GNUNET_CORE_cs_create_client_log_context (struct GNUNET_ClientHandle *handle)
 {
-  return GNUNET_GE_create_context_callback (mask,
+  return GNUNET_GE_create_context_callback (GNUNET_GE_USER |
+                                            GNUNET_GE_EVENTKIND |
+                                            GNUNET_GE_ROUTEKIND,
                                             &logClientLogContext,
                                             handle,
                                             &freeClientLogContext,
