@@ -24,12 +24,11 @@
  * @author Nils Durner
  * @author Christian Grothoff
  */
+#include "platform.h"
 #include "gnunet_util.h"
 #include "gnunet_directories.h"
 #include "gnunet_setup_lib.h"
 
-#include "platform.h"
-
 typedef int (*ConfigurationPluginMain) (int argc,
                                         char *const *argv,
                                         struct GNUNET_PluginHandle * self,
@@ -71,7 +70,7 @@ set_option_helper (GNUNET_CommandLineProcessorContext * ctx,
     {
       fprintf (stderr,
                _
-               ("Invalid synatx, argument to 'set' must have the format SECTION:OPTION=VALUE.\n"));
+               ("Invalid syntax, argument to 'set' must have the format SECTION:OPTION=VALUE.\n"));
       return GNUNET_SYSERR;
     }
   set_option = GNUNET_strdup (value);
@@ -92,7 +91,7 @@ get_option_helper (GNUNET_CommandLineProcessorContext * ctx,
     {
       fprintf (stderr,
                _
-               ("Invalid synatx, argument to 'get' must have the format SECTION:OPTION.\n"));
+               ("Invalid syntax, argument to 'get' must have the format SECTION:OPTION.\n"));
       return GNUNET_SYSERR;
     }
   get_option = GNUNET_strdup (value);
@@ -116,7 +115,6 @@ static struct GNUNET_CommandLineOption gnunetsetupOptions[] = {
    gettext_noop ("update a value in the configuration file"),
    1, &set_option_helper, NULL},
   GNUNET_COMMAND_LINE_OPTION_VERSION (PACKAGE_VERSION), /* -v */
-  GNUNET_COMMAND_LINE_OPTION_VERBOSE,
   GNUNET_COMMAND_LINE_OPTION_END,
 };
 
@@ -261,7 +259,7 @@ main (int argc, char *const *argv)
           GNUNET_GE_LOG (ectx,
                          GNUNET_GE_WARNING | GNUNET_GE_REQUEST |
                          GNUNET_GE_USER,
-                         _("No interface specified, using default\n"));
+                         _("No interface specified, using default.\n"));
           operation = "config";
 #if HAVE_DIALOG
           operation = "menuconfig";
@@ -413,7 +411,7 @@ main (int argc, char *const *argv)
   GNUNET_free (cfgFilename);
   if (done == GNUNET_NO)
     {
-      fprintf (stderr, _("Unknown operation `%s'\n"), operation);
+      fprintf (stderr, _("Unknown operation '%s'.\n"), operation);
       fprintf (stderr, _("Use --help to get a list of options.\n"));
 #if HAVE_GUILE
       GNUNET_GNS_free_specification (gns);
