@@ -24,11 +24,21 @@
  * @author Christian Grothoff
  */
 
-#include <dialog.h>
-
-#undef _
-#undef GNUNET_OK
 #include "platform.h"
+
+
+#ifdef HAVE_CDIALOG_DIALOG_H
+# undef PACKAGE
+# undef _
+# include <cdialog/dialog.h>
+#else
+# ifdef HAVE_DIALOG_H
+#  include <dialog.h>
+# endif
+#endif
+
+
+#undef GNUNET_OK
 #include "gnunet_util.h"
 #include "gnunet_setup_lib.h"
 
