@@ -33,9 +33,9 @@
  * - support editing of string inputs...
  */
 
+#include "platform.h"
 #include "gnunet_setup_lib.h"
 #include "conf.h"
-#include "platform.h"
 #include <termios.h>
 
 static char
@@ -543,7 +543,7 @@ main_setup_text (int argc,
   char c;
   int ret;
 
-#ifdef OSX
+#if OSX || SOMEBSD
 #  define TCGETS TIOCGETA
 #  define TCSETS TIOCSETA
 #endif
