@@ -24,11 +24,12 @@
  * @author Christian Grothoff
  */
 
+#include "platform.h"
 #include "gnunet_directories.h"
 #include "gnunet_util_boot.h"
 #include "gnunet_util_config.h"
 #include "gnunet_util_error_loggers.h"
-#include "platform.h"
+#include <gcrypt.h>
 
 static GNUNET_GE_KIND
 convertLogLevel (const char *level)
@@ -210,8 +211,11 @@ GNUNET_init (int argc,
 #if ENABLE_NLS
   setlocale (LC_ALL, "");
   path = GNUNET_get_installation_path (GNUNET_IPK_LOCALEDIR);
-  BINDTEXTDOMAIN ("GNUnet", path);
-  GNUNET_free (path);
+  if (path != NULL)
+    {
+      BINDTEXTDOMAIN ("GNUnet", path);
+      GNUNET_free (path);
+    }
   textdomain ("GNUnet");
 #endif
   is_daemon = 0 == strcmp (GNUNET_DEFAULT_DAEMON_CONFIG_FILE, *cfgFileName);
@@ -270,6 +274,14 @@ GNUNET_init (int argc,
                                             "GNUNET_HOME", "~/.gnunet",
                                             &path);
   GNUNET_free (path);
+  if ((GNUNET_YES == GNUNET_GC_have_configuration_value (*cfg,
+                                                         "TESTING",
+                                                         "WEAKRANDOM")) &&
+      (GNUNET_YES == GNUNET_GC_get_configuration_value_yesno (*cfg,
+                                                              "TESTING",
+                                                              "WEAKRANDOM",
+                                                              GNUNET_NO)))
+    GNUNET_disable_entropy_gathering ();
   if (configure_logging (ectx, *cfg) != 0)
     return -1;
   return i;
