@@ -24,17 +24,12 @@
  * @author Christian Grothoff
  */
 
-#include "gnunet_util_network.h"
 #include "platform.h"
+#include "gnunet_util_network.h"
 #include "network.h"
 
 #define DEBUG_IO GNUNET_NO
 
-/**
- * Global lock for gethostbyname.
- */
-static struct GNUNET_Mutex *lock;
-
 #ifndef MINGW
 static struct GNUNET_SignalHandlerContext *sctx;
 
@@ -46,7 +41,6 @@ catcher ()
 
 void __attribute__ ((constructor)) GNUNET_network_io_init ()
 {
-  lock = GNUNET_mutex_create (GNUNET_NO);
 #ifndef MINGW
   sctx = GNUNET_signal_handler_install (SIGPIPE, &catcher);
 #else
@@ -56,8 +50,6 @@ void __attribute__ ((constructor)) GNUNET_network_io_init ()
 
 void __attribute__ ((destructor)) GNUNET_network_io_fini ()
 {
-  GNUNET_mutex_destroy (lock);
-  lock = NULL;
 #ifndef MINGW
   GNUNET_signal_handler_uninstall (SIGPIPE, &catcher, sctx);
   sctx = NULL;
@@ -66,42 +58,6 @@ void __attribute__ ((destructor)) GNUNET_network_io_fini ()
 #endif
 }
 
-/**
- * Get the IP address of the given host.
- * @return GNUNET_OK on success, GNUNET_SYSERR on error
- */
-int
-GNUNET_get_host_by_name (struct GNUNET_GE_Context *ectx, const char *hostname,
-                         GNUNET_IPv4Address * ip)
-{
-  struct hostent *he;
-
-  /* slight hack: re-use config lock */
-  GNUNET_mutex_lock (lock);
-  he = GETHOSTBYNAME (hostname);
-  if (he == NULL)
-    {
-      GNUNET_GE_LOG (ectx,
-                     GNUNET_GE_ERROR | GNUNET_GE_ADMIN | GNUNET_GE_BULK,
-                     _("Could not find IP of host `%s': %s\n"),
-                     hostname, hstrerror (h_errno));
-      GNUNET_mutex_unlock (lock);
-      return GNUNET_SYSERR;
-    }
-  if (he->h_addrtype != AF_INET)
-    {
-      GNUNET_GE_BREAK (ectx, 0);
-      GNUNET_mutex_unlock (lock);
-      return GNUNET_SYSERR;
-    }
-  memcpy (ip,
-          &((struct in_addr *) he->h_addr_list[0])->s_addr,
-          sizeof (struct in_addr));
-  GNUNET_mutex_unlock (lock);
-  return GNUNET_OK;
-}
-
-
 
 struct GNUNET_SocketHandle *
 GNUNET_socket_create (struct GNUNET_GE_Context *ectx,
@@ -155,6 +111,12 @@ GNUNET_socket_destroy (struct GNUNET_SocketHandle *s)
 #ifdef OSX
           (errno != EINVAL) &&  /* OS X returns these instead of ENOTCONN */
           (errno != EHOSTDOWN) && (errno != EHOSTUNREACH) &&
+#endif
+#ifdef FREEBSD
+          (errno != ECONNRESET) &&
+#endif
+#ifdef OPENBSD
+          (errno != EINVAL) &&
 #endif
           (errno != ENOTCONN))
         GNUNET_GE_LOG_STRERROR (s->ectx,
@@ -170,7 +132,7 @@ GNUNET_socket_destroy (struct GNUNET_SocketHandle *s)
 }
 
 /* TODO: log errors! */
-#ifdef OSX
+#if OSX || FREEBSD
 static int
 socket_set_nosigpipe (struct GNUNET_SocketHandle *s, int dontSigPipe)
 {
@@ -245,20 +207,18 @@ GNUNET_socket_recv (struct GNUNET_SocketHandle *s,
   GNUNET_socket_set_blocking (s, 0 != (nc & GNUNET_NC_BLOCKING));
   flags = 0;
 #ifdef CYGWIN
-  if (0 == (nc & GNUNET_NC_IGNORE_INT))
-    flags |= MSG_NOSIGNAL;
-#elif OSX
+  flags |= MSG_NOSIGNAL;
+#elif OSX || FREEBSD
   socket_set_nosigpipe (s, 0 == (nc & GNUNET_NC_IGNORE_INT));
   if (0 == (nc & GNUNET_NC_BLOCKING))
     flags |= MSG_DONTWAIT;
-#elif SOMEBSD || SOLARIS
+#elif SOLARIS
   if (0 == (nc & GNUNET_NC_BLOCKING))
     flags |= MSG_DONTWAIT;
 #elif LINUX
   if (0 == (nc & GNUNET_NC_BLOCKING))
     flags |= MSG_DONTWAIT;
-  if (0 == (nc & GNUNET_NC_IGNORE_INT))
-    flags |= MSG_NOSIGNAL;
+  flags |= MSG_NOSIGNAL;
 #else
   /* good luck */
 #endif
@@ -271,7 +231,11 @@ GNUNET_socket_recv (struct GNUNET_SocketHandle *s,
       GNUNET_GE_ASSERT (NULL, s->checksum == -s->handle);
       if ((ret == (size_t) - 1) &&
           (errno == EINTR) && (0 != (nc & GNUNET_NC_IGNORE_INT)))
-        continue;
+        {
+          if (GNUNET_shutdown_test () == GNUNET_YES)
+            return GNUNET_SYSERR;
+          continue;
+        }
       if ((ret == (size_t) - 1) || (ret > max - pos))
         {
           if (errno == EINTR)
@@ -325,20 +289,18 @@ GNUNET_socket_recv_from (struct GNUNET_SocketHandle *s,
   GNUNET_socket_set_blocking (s, 0 != (nc & GNUNET_NC_BLOCKING));
   flags = 0;
 #ifdef CYGWIN
-  if (0 == (nc & GNUNET_NC_IGNORE_INT))
-    flags |= MSG_NOSIGNAL;
-#elif OSX
+  flags |= MSG_NOSIGNAL;
+#elif OSX || FREEBSD
   socket_set_nosigpipe (s, 0 == (nc & GNUNET_NC_IGNORE_INT));
   if (0 == (nc & GNUNET_NC_BLOCKING))
     flags |= MSG_DONTWAIT;
-#elif SOMEBSD || SOLARIS
+#elif SOLARIS
   if (0 == (nc & GNUNET_NC_BLOCKING))
     flags |= MSG_DONTWAIT;
 #elif LINUX
   if (0 == (nc & GNUNET_NC_BLOCKING))
     flags |= MSG_DONTWAIT;
-  if (0 == (nc & GNUNET_NC_IGNORE_INT))
-    flags |= MSG_NOSIGNAL;
+  flags |= MSG_NOSIGNAL;
 #else
   /* good luck */
 #endif
@@ -400,21 +362,19 @@ GNUNET_socket_send (struct GNUNET_SocketHandle *s,
 
   GNUNET_socket_set_blocking (s, 0 != (nc & GNUNET_NC_BLOCKING));
   flags = 0;
-#if SOMEBSD || SOLARIS
+#if SOLARIS
   if (0 == (nc & GNUNET_NC_BLOCKING))
     flags |= MSG_DONTWAIT;
-#elif OSX
+#elif OSX || FREEBSD
   socket_set_nosigpipe (s, 0 == (nc & GNUNET_NC_IGNORE_INT));
   if (0 == (nc & GNUNET_NC_BLOCKING))
     flags |= MSG_DONTWAIT;
 #elif CYGWIN
-  if (0 == (nc & GNUNET_NC_IGNORE_INT))
-    flags |= MSG_NOSIGNAL;
+  flags |= MSG_NOSIGNAL;
 #elif LINUX
   if (0 == (nc & GNUNET_NC_BLOCKING))
     flags |= MSG_DONTWAIT;
-  if (0 == (nc & GNUNET_NC_IGNORE_INT))
-    flags |= MSG_NOSIGNAL;
+  flags |= MSG_NOSIGNAL;
 #else
   /* pray */
 #endif
@@ -477,21 +437,19 @@ GNUNET_socket_send_to (struct GNUNET_SocketHandle *s,
 
   GNUNET_socket_set_blocking (s, 0 != (nc & GNUNET_NC_BLOCKING));
   flags = 0;
-#if SOMEBSD || SOLARIS
+#if SOLARIS
   if (0 == (nc & GNUNET_NC_BLOCKING))
     flags |= MSG_DONTWAIT;
-#elif OSX
+#elif OSX || FREEBSD
   socket_set_nosigpipe (s, 0 == (nc & GNUNET_NC_IGNORE_INT));
   if (0 == (nc & GNUNET_NC_BLOCKING))
     flags |= MSG_DONTWAIT;
 #elif CYGWIN
-  if (0 == (nc & GNUNET_NC_IGNORE_INT))
-    flags |= MSG_NOSIGNAL;
+  flags |= MSG_NOSIGNAL;
 #elif LINUX
   if (0 == (nc & GNUNET_NC_BLOCKING))
     flags |= MSG_DONTWAIT;
-  if (0 == (nc & GNUNET_NC_IGNORE_INT))
-    flags |= MSG_NOSIGNAL;
+  flags |= MSG_NOSIGNAL;
 #else
   /* pray */
 #endif
