@@ -51,10 +51,13 @@
  */
 static int
 getAddressFromHostname (struct GNUNET_GE_Context *ectx,
-                        GNUNET_IPv4Address * identity)
+                        struct in_addr *identity)
 {
   char hostname[MAX_HOSTNAME];
   int ret;
+  struct sockaddr *my_addr;
+  struct sockaddr_in a4;
+  socklen_t socklen;
 
   if (0 != gethostname (hostname, MAX_HOSTNAME))
     {
@@ -63,7 +66,12 @@ getAddressFromHostname (struct GNUNET_GE_Context *ectx,
                               GNUNET_GE_USER | GNUNET_GE_BULK, "gethostname");
       return GNUNET_SYSERR;
     }
-  ret = GNUNET_get_host_by_name (ectx, hostname, identity);
+  socklen = sizeof (struct sockaddr_in);
+  my_addr = (struct sockaddr *) &a4;
+  ret =
+    GNUNET_get_ip_from_hostname (ectx, hostname, AF_INET, &my_addr, &socklen);
+  if (ret == GNUNET_OK)
+    *identity = a4.sin_addr;
   return ret;
 }
 
@@ -71,7 +79,7 @@ getAddressFromHostname (struct GNUNET_GE_Context *ectx,
 static int
 getAddressFromGetIfAddrs (struct GNUNET_GC_Configuration *cfg,
                           struct GNUNET_GE_Context *ectx,
-                          GNUNET_IPv4Address * identity)
+                          struct in_addr *identity)
 {
   char *interfaces;
   struct ifaddrs *ifa_first;
@@ -128,8 +136,7 @@ getAddressFromGetIfAddrs (struct GNUNET_GC_Configuration *cfg,
 #define MAX_INTERFACES 16
 static int
 getAddressFromIOCTL (struct GNUNET_GC_Configuration *cfg,
-                     struct GNUNET_GE_Context *ectx,
-                     GNUNET_IPv4Address * identity)
+                     struct GNUNET_GE_Context *ectx, struct in_addr *identity)
 {
   char *interfaces;
 #ifndef MINGW
@@ -241,6 +248,7 @@ getAddressFromIOCTL (struct GNUNET_GC_Configuration *cfg,
   GNUNET_free (interfaces);
   return GNUNET_SYSERR;
 #else /* MinGW */
+  char ntop_buf[INET_ADDRSTRLEN];
 
   /* Win 98 or Win NT SP 4 */
   if (GNGetIpAddrTable)
@@ -292,14 +300,17 @@ getAddressFromIOCTL (struct GNUNET_GC_Configuration *cfg,
           return GNUNET_SYSERR;
         }
       else if (iAddrCount > 1)
-        GNUNET_GE_LOG (ectx,
-                       GNUNET_GE_WARNING | GNUNET_GE_BULK | GNUNET_GE_USER,
-                       _("There is more than one IP address specified"
-                         " for interface `%s'.\nGNUnet will "
-                         "use %u.%u.%u.%u.\n"), interfaces,
-                       GNUNET_PRIP (ntohl (dwIP)));
+        {
+          GNUNET_GE_LOG (ectx,
+                         GNUNET_GE_WARNING | GNUNET_GE_BULK | GNUNET_GE_USER,
+                         _("There is more than one IP address specified"
+                           " for interface `%s'.\nGNUnet will "
+                           "use %s.\n"), interfaces,
+                         inet_ntop (AF_INET, &dwIP, ntop_buf,
+                                    INET_ADDRSTRLEN));
+        }
 
-      identity->addr = dwIP;
+      identity->S_un.S_addr = dwIP;
 
       GlobalFree (pTable);
       GlobalFree (pAddrTable);
@@ -345,13 +356,12 @@ getAddressFromIOCTL (struct GNUNET_GC_Configuration *cfg,
           return GNUNET_SYSERR;
         }
       closesocket (s);
-      identity->addr = theHost.sin_addr.S_un.S_addr;
+      *identity = theHost.sin_addr;
     }
 
   GNUNET_GE_LOG (ectx, GNUNET_GE_DEBUG | GNUNET_GE_REQUEST | GNUNET_GE_USER,
-                 _("GNUnet now uses the IP address %u.%u.%u.%u.\n"),
-                 GNUNET_PRIP (ntohl (identity->addr)));
-
+                 _("GNUnet now uses the IP address %s.\n"),
+                 inet_ntop (AF_INET, identity, ntop_buf, INET_ADDRSTRLEN));
   return GNUNET_OK;
 #endif
 }
@@ -364,13 +374,15 @@ getAddressFromIOCTL (struct GNUNET_GC_Configuration *cfg,
  */
 char *
 GNUNET_get_local_ip (struct GNUNET_GC_Configuration *cfg,
-                     struct GNUNET_GE_Context *ectx,
-                     GNUNET_IPv4Address * addr)
+                     struct GNUNET_GE_Context *ectx, struct in_addr *addr)
 {
-  GNUNET_IPv4Address address;
+  struct in_addr address;
+  struct sockaddr *my_addr;
+  struct sockaddr_in a4;
   char *ipString;
   int retval;
-  char buf[65];
+  char buf[INET_ADDRSTRLEN];
+  socklen_t socklen;
 
   retval = GNUNET_SYSERR;
   if (GNUNET_GC_have_configuration_value (cfg, "NETWORK", "IP"))
@@ -380,7 +392,13 @@ GNUNET_get_local_ip (struct GNUNET_GC_Configuration *cfg,
                                                 &ipString);
       if (strlen (ipString) > 0)
         {
-          retval = GNUNET_get_host_by_name (ectx, ipString, &address);
+          socklen = sizeof (struct sockaddr_in);
+          my_addr = (struct sockaddr *) &a4;
+          retval =
+            GNUNET_get_ip_from_hostname (ectx, ipString, AF_INET, &my_addr,
+                                         &socklen);
+          if (retval == GNUNET_OK)
+            address = a4.sin_addr;
         }
       GNUNET_free (ipString);
     }
@@ -398,8 +416,8 @@ GNUNET_get_local_ip (struct GNUNET_GC_Configuration *cfg,
     retval = getAddressFromHostname (ectx, &address);
   if (retval == GNUNET_SYSERR)
     return NULL;
-  GNUNET_snprintf (buf, 64, "%u.%u.%u.%u",
-                   GNUNET_PRIP (ntohl (*(int *) &address)));
+  if (NULL == inet_ntop (AF_INET, &address, buf, INET_ADDRSTRLEN))
+    return NULL;
   if (addr != NULL)
     *addr = address;
   return GNUNET_strdup (buf);
