@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2001, 2002, 2006 Christian Grothoff (and other contributing authors)
+     (C) 2001, 2002, 2006, 2008 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -55,7 +55,7 @@ typedef struct GNUNET_ClientServerConnection
 {
 
   /**
-   * the socket handle, NULL if not life
+   * the socket handle, NULL if not live
    */
   struct GNUNET_SocketHandle *sock;
 
@@ -229,14 +229,37 @@ GNUNET_client_connection_test_connected (struct GNUNET_ClientServerConnection
 }
 
 /**
- * Check a socket, open and connect if it is closed and it is a client-socket.
+ * Check a socket, open and connect if it is closed.  This code
+ * supports IPv4 and IPv6 (and may try both).  It also waits a bounded
+ * amount of time for the connection to succeed and may even retry the
+ * same IP version a few times since gnunetd may just be starting or
+ * out of sockets; hence this code could fail on first attempt, will
+ * then wait a few milliseconds, retry and conceivably succeed.  Since
+ * this is then done for multiple address families, the whole thing is
+ * slightly more complicated then your ordinary connect call.  Not to
+ * mention that the code also supports another thread coming in in the
+ * middle and closing the socket for good -- or even opening it!
  */
 int
 GNUNET_client_connection_ensure_connected (struct
                                            GNUNET_ClientServerConnection
                                            *sock)
 {
-  struct sockaddr_in soaddr;
+  /* list of address families to try for connecting,
+     in order of preference */
+  static int addr_families[] = {
+#ifdef AF_UNSPEC
+    AF_UNSPEC,
+#endif
+#ifdef AF_INET6
+    AF_INET6,
+#endif
+    AF_INET,
+    -1
+  };
+  GNUNET_CronTime select_start;
+  struct sockaddr *soaddr;
+  socklen_t socklen;
   fd_set rset;
   fd_set wset;
   fd_set eset;
@@ -245,7 +268,10 @@ GNUNET_client_connection_ensure_connected (struct
   int osock;
   unsigned short port;
   char *host;
-  GNUNET_IPv4Address ip;
+  int af_index;
+  int soerr;
+  socklen_t soerrlen;
+  int tries;
 
   GNUNET_GE_ASSERT (NULL, sock != NULL);
   if (sock->sock != NULL)
@@ -258,107 +284,173 @@ GNUNET_client_connection_ensure_connected (struct
   host = getGNUnetdHost (sock->ectx, sock->cfg);
   if (host == NULL)
     return GNUNET_SYSERR;
-  if (GNUNET_SYSERR == GNUNET_get_host_by_name (sock->ectx, host, &ip))
+  af_index = 0;
+  /* we immediately advance if there is a DNS lookup error
+   * (which would likely persist) or a socket API error
+   * (which would equally likely persist).  We retry a
+   * few times with a small delay if we may just be having
+   * a connection issue.
+   */
+#define TRIES_PER_AF 2
+#ifdef WINDOWS
+  #define DELAY_PER_RETRY (5000 * GNUNET_CRON_MILLISECONDS)
+#else
+  #define DELAY_PER_RETRY (50 * GNUNET_CRON_MILLISECONDS)
+#endif
+#define ADVANCE() do { af_index++; tries = TRIES_PER_AF; } while(0)
+#define RETRY() do { tries--; if (tries == 0) { ADVANCE(); } else { GNUNET_thread_sleep(DELAY_PER_RETRY); } } while (0)
+  tries = TRIES_PER_AF;
+  /* loop over all possible address families */
+  while (1)
     {
-      GNUNET_free (host);
-      return GNUNET_SYSERR;
-    }
-  GNUNET_mutex_lock (sock->destroylock);
-  if (sock->sock != NULL)
-    {
-      GNUNET_free (host);
-      GNUNET_mutex_unlock (sock->destroylock);
-      return GNUNET_OK;
-    }
-  if (sock->dead == GNUNET_YES)
-    {
-      GNUNET_free (host);
-      GNUNET_mutex_unlock (sock->destroylock);
-      return GNUNET_SYSERR;
-    }
-  osock = SOCKET (PF_INET, SOCK_STREAM, 6);     /* 6: TCP */
-  if (osock == -1)
-    {
-      GNUNET_GE_LOG_STRERROR (sock->ectx,
-                              GNUNET_GE_ERROR | GNUNET_GE_USER |
-                              GNUNET_GE_ADMIN | GNUNET_GE_BULK, "socket");
-      GNUNET_free (host);
-      GNUNET_mutex_unlock (sock->destroylock);
-      return GNUNET_SYSERR;
-    }
-  sock->sock = GNUNET_socket_create (sock->ectx, NULL, osock);
-  GNUNET_socket_set_blocking (sock->sock, GNUNET_NO);
-  memset (&soaddr, 0, sizeof (soaddr));
-  soaddr.sin_family = AF_INET;
-  GNUNET_GE_ASSERT (sock->ectx,
-                    sizeof (struct in_addr) == sizeof (GNUNET_IPv4Address));
-  memcpy (&soaddr.sin_addr, &ip, sizeof (struct in_addr));
-  soaddr.sin_port = htons (port);
-  ret = CONNECT (osock, (struct sockaddr *) &soaddr, sizeof (soaddr));
-  if ((ret != 0) && (errno != EINPROGRESS) && (errno != EWOULDBLOCK))
-    {
-      GNUNET_GE_LOG (sock->ectx,
-                     GNUNET_GE_WARNING | GNUNET_GE_USER | GNUNET_GE_BULK,
-                     _("Cannot connect to %s:%u: %s\n"),
-                     host, port, STRERROR (errno));
-      GNUNET_socket_destroy (sock->sock);
-      sock->sock = NULL;
-      GNUNET_free (host);
-      GNUNET_mutex_unlock (sock->destroylock);
-      return GNUNET_SYSERR;
-    }
-  /* we call select() first with a timeout of WAIT_SECONDS to
-     avoid blocking on a later write indefinitely;
-     Important if a local firewall decides to just drop
-     the TCP handshake... */
-  FD_ZERO (&rset);
-  FD_ZERO (&wset);
-  FD_ZERO (&eset);
-  FD_SET (osock, &wset);
-  FD_SET (osock, &eset);
-#define WAIT_SECONDS 10
-  timeout.tv_sec = WAIT_SECONDS;
-  timeout.tv_usec = 0;
-  errno = 0;
-  ret = SELECT (osock + 1, &rset, &wset, &eset, &timeout);
-  if (ret == -1)
-    {
-      if (errno != EINTR)
+      if (addr_families[af_index] == -1)
+        {
+          GNUNET_GE_LOG (sock->ectx,
+                         GNUNET_GE_WARNING | GNUNET_GE_USER | GNUNET_GE_BULK,
+                         _("Error connecting to %s:%u. Is the daemon running?\n"), host, port);
+          return GNUNET_SYSERR;
+        }
+      soaddr = NULL;
+      socklen = 0;
+      if (GNUNET_SYSERR ==
+          GNUNET_get_ip_from_hostname (sock->ectx, host,
+                                       addr_families[af_index], &soaddr,
+                                       &socklen))
+        {
+          ADVANCE ();
+          continue;
+        }
+      GNUNET_mutex_lock (sock->destroylock);
+      if (sock->sock != NULL)
+        {
+          GNUNET_free (host);
+          GNUNET_mutex_unlock (sock->destroylock);
+          GNUNET_free (soaddr);
+          return GNUNET_OK;
+        }
+      if (sock->dead == GNUNET_YES)
+        {
+          GNUNET_free (host);
+          GNUNET_mutex_unlock (sock->destroylock);
+          GNUNET_free (soaddr);
+          return GNUNET_SYSERR;
+        }
+      if (soaddr->sa_family == AF_INET)
+        {
+          ((struct sockaddr_in *) soaddr)->sin_port = htons (port);
+          osock = SOCKET (PF_INET, SOCK_STREAM, 0);
+        }
+      else
+        {
+#ifdef PF_INET6
+          ((struct sockaddr_in6 *) soaddr)->sin6_port = htons (port);
+          osock = SOCKET (PF_INET6, SOCK_STREAM, 0);
+#else
+          osock = -1;
+          errno = EAFNOSUPPORT;
+#endif
+        }
+      if (osock == -1)
+        {
+          GNUNET_GE_LOG_STRERROR (sock->ectx,
+                                  GNUNET_GE_ERROR | GNUNET_GE_USER |
+                                  GNUNET_GE_ADMIN | GNUNET_GE_BULK, "socket");
+          GNUNET_mutex_unlock (sock->destroylock);
+          GNUNET_free (soaddr);
+          ADVANCE ();
+          continue;
+        }
+      sock->sock = GNUNET_socket_create (sock->ectx, NULL, osock);
+      GNUNET_socket_set_blocking (sock->sock, GNUNET_NO);
+      ret = CONNECT (osock, soaddr, socklen);
+      GNUNET_free (soaddr);
+      if ((ret != 0) && (errno != EINPROGRESS) && (errno != EWOULDBLOCK))
+        {
+          GNUNET_GE_LOG (sock->ectx,
+                         GNUNET_GE_WARNING | GNUNET_GE_USER | GNUNET_GE_BULK,
+                         _("Cannot connect to %s:%u: %s\n"),
+                         host, port, STRERROR (errno));
+          GNUNET_socket_destroy (sock->sock);
+          sock->sock = NULL;
+          GNUNET_mutex_unlock (sock->destroylock);
+          if (errno == ECONNREFUSED)
+            RETRY ();           /* gnunetd may just be restarting */
+          else
+            ADVANCE ();
+          continue;
+        }
+      /* we call select() first with a timeout of WAIT_SECONDS to
+         avoid blocking on a later write indefinitely;
+         Important if a local firewall decides to just drop
+         the TCP handshake... */
+      FD_ZERO (&rset);
+      FD_ZERO (&wset);
+      FD_ZERO (&eset);
+      FD_SET (osock, &wset);
+      FD_SET (osock, &eset);
+      timeout.tv_sec = 0;
+      timeout.tv_usec = DELAY_PER_RETRY * TRIES_PER_AF * 1000;
+      errno = 0;
+      select_start = GNUNET_get_time ();
+      ret = SELECT (osock + 1, &rset, &wset, &eset, &timeout);
+      if (ret == -1)
+        {
+          if (errno != EINTR)
+            GNUNET_GE_LOG_STRERROR (sock->ectx,
+                                    GNUNET_GE_WARNING | GNUNET_GE_USER |
+                                    GNUNET_GE_BULK, "select");
+          GNUNET_socket_destroy (sock->sock);
+          sock->sock = NULL;
+          GNUNET_mutex_unlock (sock->destroylock);
+          if ((GNUNET_get_time () - select_start >
+               TRIES_PER_AF * DELAY_PER_RETRY) || (errno != EINTR))
+            ADVANCE ();         /* spend enough time trying here */
+          else
+            RETRY ();
+          continue;
+        }
+      if ((FD_ISSET (osock, &eset)) || (!FD_ISSET (osock, &wset)))
+        {
+          GNUNET_socket_destroy (sock->sock);
+          sock->sock = NULL;
+          GNUNET_mutex_unlock (sock->destroylock);
+          if (GNUNET_get_time () - select_start >
+              TRIES_PER_AF * DELAY_PER_RETRY)
+            ADVANCE ();         /* spend enough time trying here */
+          else
+            RETRY ();
+          continue;
+        }
+      soerr = 0;
+      soerrlen = sizeof (soerr);
+
+      ret = GETSOCKOPT (osock, SOL_SOCKET, SO_ERROR, &soerr, &soerrlen);
+      if (ret != 0)
         GNUNET_GE_LOG_STRERROR (sock->ectx,
                                 GNUNET_GE_WARNING | GNUNET_GE_USER |
-                                GNUNET_GE_BULK, "select");
-      GNUNET_socket_destroy (sock->sock);
-      sock->sock = NULL;
-      GNUNET_free (host);
-      GNUNET_mutex_unlock (sock->destroylock);
-      return GNUNET_SYSERR;
-    }
-  if (FD_ISSET (osock, &eset))
-    {
-      GNUNET_GE_LOG (sock->ectx,
-                     GNUNET_GE_WARNING | GNUNET_GE_USER | GNUNET_GE_BULK,
-                     _("Error connecting to %s:%u\n"), host, port);
-      GNUNET_socket_destroy (sock->sock);
-      sock->sock = NULL;
-      GNUNET_free (host);
-      GNUNET_mutex_unlock (sock->destroylock);
-      return GNUNET_SYSERR;
-    }
-  if (!FD_ISSET (osock, &wset))
-    {
-      GNUNET_GE_LOG (sock->ectx,
-                     GNUNET_GE_WARNING | GNUNET_GE_USER | GNUNET_GE_BULK,
-                     _("Failed to connect to %s:%u in %ds\n"),
-                     host, port, WAIT_SECONDS);
-      GNUNET_socket_destroy (sock->sock);
-      sock->sock = NULL;
-      GNUNET_free (host);
-      GNUNET_mutex_unlock (sock->destroylock);
-      return GNUNET_SYSERR;
+                                GNUNET_GE_BULK, "getsockopt");
+      if ((soerr != 0) || (ret != 0 && (errno == ENOTSOCK || errno == EBADF)))
+        {
+          GNUNET_socket_destroy (sock->sock);
+          sock->sock = NULL;
+          GNUNET_mutex_unlock (sock->destroylock);
+          if (GNUNET_get_time () - select_start >
+              TRIES_PER_AF * DELAY_PER_RETRY)
+            ADVANCE ();         /* spend enough time trying here */
+          else
+            RETRY ();
+          continue;
+        }
+      /* yayh! connected! */
+      break;
     }
   GNUNET_free (host);
   GNUNET_socket_set_blocking (sock->sock, GNUNET_YES);
   GNUNET_mutex_unlock (sock->destroylock);
+#undef ADVANCE
+#undef RETRY
+#undef TRIES_PER_AF
+#undef DELAY_PER_RETRY
   return GNUNET_OK;
 }
 
@@ -513,7 +605,8 @@ GNUNET_client_connection_read_result (struct GNUNET_ClientServerConnection
     {
       GNUNET_GE_LOG (sock->ectx,
                      GNUNET_GE_WARNING | GNUNET_GE_DEVELOPER | GNUNET_GE_BULK,
-                     _("`%s' failed, reply invalid!\n"), __FUNCTION__);
+                     _
+                     ("Reading result from gnunetd failed, reply invalid!\n"));
       GNUNET_free (rv);
       return GNUNET_SYSERR;
     }
