@@ -24,6 +24,7 @@
  * @brief functions to read or change the OS configuration
  * @author Nils Durner
  * @author Heikki Lindholm
+ * @author Jake Dust
  */
 
 #include "platform.h"
@@ -144,6 +145,12 @@ isOSAutostartCapable ()
       if (ACCESS ("/etc/init.d/", W_OK) == 0)
         return GNUNET_YES;
     }
+  /* Gentoo */
+  else if (ACCESS ("/sbin/rc-update", X_OK) == 0)
+    {
+      if (ACCESS ("/etc/init.d/", W_OK) == 0)
+        return GNUNET_YES;
+    }
   return GNUNET_NO;
 #else
 #ifdef WINDOWS
@@ -171,6 +178,8 @@ isOSAutostartCapable ()
  *           enabled
  * @param doAutoStart GNUNET_YES to enable autostart of the
  *        application, GNUNET_NO to disable it
+ * @param servicename name of the service as displayed by the OS
+ * @param application path to service binary
  * @param username name of the user account to use
  * @param groupname name of the group to use
  * @returns GNUNET_YES on success, GNUNET_NO if unsupported, GNUNET_SYSERR on failure or one of
@@ -188,6 +197,7 @@ int
 GNUNET_configure_autostart (struct GNUNET_GE_Context *ectx,
                             int testCapability,
                             int doAutoStart,
+                            const char *servicename,
                             const char *application,
                             const char *username, const char *groupname)
 {
@@ -209,7 +219,7 @@ GNUNET_configure_autostart (struct GNUNET_GE_Context *ectx,
             username = NULL;
 
           /* Install service */
-          switch (InstallAsService (username))
+          switch (InstallAsService (servicename, application, username))
             {
             case 0:
               break;
@@ -248,7 +258,7 @@ GNUNET_configure_autostart (struct GNUNET_GE_Context *ectx,
                             0, KEY_EXECUTE, &hKey) == ERROR_SUCCESS)
             {
               if (RegSetValueEx (hKey,
-                                 "GNUnet",
+                                 servicename,
                                  0, REG_SZ, szPath,
                                  strlen (szPath)) != ERROR_SUCCESS)
                 return 5;
@@ -263,7 +273,7 @@ GNUNET_configure_autostart (struct GNUNET_GE_Context *ectx,
     {
       if (IsWinNT ())
         {
-          switch (UninstallService ())
+          switch (UninstallService (servicename))
             {
             case 0:
               break;
@@ -287,7 +297,7 @@ GNUNET_configure_autostart (struct GNUNET_GE_Context *ectx,
                             "Software\\Microsoft\\Windows\\CurrentVersion\\Run",
                             0, KEY_SET_VALUE, &hKey) == ERROR_SUCCESS)
             {
-              RegDeleteValue (hKey, "GNUnet");
+              RegDeleteValue (hKey, servicename);
               RegCloseKey (hKey);
             }
           else
@@ -299,21 +309,36 @@ GNUNET_configure_autostart (struct GNUNET_GE_Context *ectx,
 #else
   struct stat buf;
   int ret;
+  int i;
+  char *initscript;
+
+  i = strlen (application) - 1;
+  if (i <= 0)
+    return GNUNET_SYSERR;
+  while ((i > 0) && (application[i] != DIR_SEPARATOR))
+    i--;
+
+  initscript = GNUNET_malloc (strlen (&application[i]) + 13);
+  strcpy (initscript, "/etc/init.d/");
+  strcat (initscript, &application[i]);
 
   /* Unix */
   if ((ACCESS ("/usr/sbin/update-rc.d", X_OK) != 0))
     {
-      GNUNET_GE_LOG_STRERROR_FILE (ectx,
-                                   GNUNET_GE_ERROR | GNUNET_GE_USER |
-                                   GNUNET_GE_ADMIN | GNUNET_GE_IMMEDIATE,
-                                   "access", "/usr/sbin/update-rc.d");
-      return GNUNET_SYSERR;
+      if ((ACCESS ("/sbin/rc-update", X_OK) != 0))
+        {
+          GNUNET_GE_LOG_STRERROR_FILE (ectx,
+                                       GNUNET_GE_ERROR | GNUNET_GE_USER |
+                                       GNUNET_GE_ADMIN | GNUNET_GE_IMMEDIATE,
+                                       "access", "/usr/sbin/update-rc.d");
+          GNUNET_free (initscript);
+          return GNUNET_SYSERR;
+        }
     }
 
   /* Debian */
   if (doAutoStart)
     {
-
       if (ACCESS (application, X_OK) != 0)
         {
           GNUNET_GE_LOG_STRERROR_FILE (ectx,
@@ -321,10 +346,11 @@ GNUNET_configure_autostart (struct GNUNET_GE_Context *ectx,
                                        GNUNET_GE_ADMIN | GNUNET_GE_IMMEDIATE,
                                        "access", application);
         }
-      if (STAT ("/etc/init.d/gnunetd", &buf) == -1)
+
+      if (STAT (initscript, &buf) == -1)
         {
           /* create init file */
-          FILE *f = FOPEN ("/etc/init.d/gnunetd", "w");
+          FILE *f = FOPEN (initscript, "w");
           if (f == NULL)
             {
               GNUNET_GE_LOG_STRERROR_FILE (ectx,
@@ -332,6 +358,7 @@ GNUNET_configure_autostart (struct GNUNET_GE_Context *ectx,
                                            GNUNET_GE_ADMIN |
                                            GNUNET_GE_IMMEDIATE, "fopen",
                                            "/etc/init.d/gnunetd");
+              GNUNET_free (initscript);
               return 2;
             }
 
@@ -341,95 +368,159 @@ GNUNET_configure_autostart (struct GNUNET_GE_Context *ectx,
                    "# Automatically created by %s\n"
                    "#\n"
                    "\n"
-                   "PIDFILE=/var/run/gnunetd/gnunetd.pid\n"
+                   "PIDFILE=/var/run/gnunetd/%s.pid\n"
+                   "APPNAME=%s\n"
                    "\n"
                    "case \"$1\" in\n"
                    "  start)\n"
-                   "  	echo -n \"Starting GNUnet: \"\n"
+                   "  	echo -n \"Starting $APPNAME: \"\n"
                    "  	%s\n && echo ok || echo failed\n"
                    "  	;;\n"
                    "  stop)\n"
-                   "  	echo -n \"Stopping GNUnet: \"\n"
+                   "  	echo -n \"Stopping $APPNAME: \"\n"
                    "  	kill `cat $PIDFILE`\n && echo ok || echo failed\n"
                    "  	;;\n"
                    "  reload)\n"
-                   "  	echo -n \"Reloading GNUnet: \"\n"
+                   "  	echo -n \"Reloading $APPNAME: \"\n"
                    "  	kill -HUP `cat $PIDFILE`\n && echo ok || echo failed\n"
                    "  	;;\n"
                    "  restart|force-reload)\n"
-                   "  	echo \"Restarting GNUnet: gnunetd...\"\n"
+                   "  	echo \"Restarting $APPNAME...\"\n"
                    "  	$0 stop\n"
                    "  	sleep 1\n"
                    "  	$0 start\n"
                    "  	;;\n"
                    "  *)\n"
-                   "  	echo \"Usage: /etc/init.d/gnunetd {start|stop|reload|restart|force-reload}\" >&2\n"
+                   "  	echo \"Usage: %s {start|stop|reload|restart|force-reload}\" >&2\n"
                    "  	exit 1\n"
                    "  	;;\n"
-                   "\n" "esac\n" "exit 0\n", "gnunet-setup", application);
+                   "\n"
+                   "esac\n"
+                   "exit 0\n",
+                   "gnunet-setup",
+                   application, servicename, application, initscript);
           fclose (f);
-          if (0 != CHMOD ("/etc/init.d/gnunetd",
+          if (0 != CHMOD (initscript,
                           S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH))
             {
               GNUNET_GE_LOG_STRERROR_FILE (ectx,
                                            GNUNET_GE_WARNING | GNUNET_GE_USER
                                            | GNUNET_GE_ADMIN |
                                            GNUNET_GE_IMMEDIATE, "chmod",
-                                           "/etc/init.d/gnunetd");
+                                           initscript);
+              GNUNET_free (initscript);
               return GNUNET_SYSERR;
             }
         }
-      if (STAT ("/etc/init.d/gnunetd", &buf) != -1)
+      if (STAT (initscript, &buf) != -1)
         {
           errno = 0;
-          ret = system ("/usr/sbin/update-rc.d gnunetd defaults");
-          if (ret != 0)
+          if (ACCESS ("/usr/sbin/update-rc.d", W_OK) == 0)
             {
-              if (errno != 0)
+              ret = system ("/usr/sbin/update-rc.d gnunetd defaults");
+              if (ret != 0)
                 {
-                  GNUNET_GE_LOG_STRERROR_FILE (ectx,
-                                               GNUNET_GE_WARNING |
-                                               GNUNET_GE_USER |
-                                               GNUNET_GE_ADMIN |
-                                               GNUNET_GE_IMMEDIATE, "system",
-                                               "/usr/sbin/update-rc.d");
+                  if (errno != 0)
+                    {
+                      GNUNET_GE_LOG_STRERROR_FILE (ectx,
+                                                   GNUNET_GE_WARNING |
+                                                   GNUNET_GE_USER |
+                                                   GNUNET_GE_ADMIN |
+                                                   GNUNET_GE_IMMEDIATE,
+                                                   "system",
+                                                   "/usr/sbin/update-rc.d");
+                    }
+                  else
+                    {
+                      GNUNET_GE_LOG (ectx,
+                                     GNUNET_GE_WARNING | GNUNET_GE_USER |
+                                     GNUNET_GE_ADMIN | GNUNET_GE_IMMEDIATE,
+                                     _
+                                     ("Command `%s' failed with error code %u\n"),
+                                     "/usr/sbin/update-rc.d gnunetd defaults",
+                                     WEXITSTATUS (ret));
+                    }
+                  GNUNET_free (initscript);
+                  return GNUNET_SYSERR;
                 }
-              else
+            }
+          else if (ACCESS ("/sbin/rc-update", W_OK) == 0)
+            {
+              ret = system ("/sbin/rc-update add gnunetd default");
+              if (ret != 0)
                 {
-                  GNUNET_GE_LOG (ectx,
-                                 GNUNET_GE_WARNING | GNUNET_GE_USER |
-                                 GNUNET_GE_ADMIN | GNUNET_GE_IMMEDIATE,
-                                 _
-                                 ("Command `%s' failed with error code %u\n"),
-                                 "/usr/sbin/update-rc.d gnunetd defaults",
-                                 WEXITSTATUS (ret));
+                  if (errno != 0)
+                    {
+                      GNUNET_GE_LOG_STRERROR_FILE (ectx,
+                                                   GNUNET_GE_WARNING |
+                                                   GNUNET_GE_USER |
+                                                   GNUNET_GE_ADMIN |
+                                                   GNUNET_GE_IMMEDIATE,
+                                                   "system",
+                                                   "/sbin/rc-update");
+                    }
+                  else
+                    {
+                      GNUNET_GE_LOG (ectx,
+                                     GNUNET_GE_WARNING | GNUNET_GE_USER |
+                                     GNUNET_GE_ADMIN | GNUNET_GE_IMMEDIATE,
+                                     _
+                                     ("Command `%s' failed with error code %u\n"),
+                                     "/sbin/rc-update add gnunetd default",
+                                     WEXITSTATUS (ret));
+                    }
+                  GNUNET_free (initscript);
+                  return GNUNET_SYSERR;
                 }
-              return GNUNET_SYSERR;
             }
         }
+      GNUNET_free (initscript);
       return GNUNET_YES;
     }
   else
     {                           /* REMOVE autostart */
-      if ((UNLINK ("/etc/init.d/gnunetd") == -1) && (errno != ENOENT))
+      if ((UNLINK (initscript) == -1) && (errno != ENOENT))
         {
           GNUNET_GE_LOG_STRERROR_FILE (ectx,
                                        GNUNET_GE_WARNING | GNUNET_GE_USER |
                                        GNUNET_GE_ADMIN | GNUNET_GE_IMMEDIATE,
-                                       "unlink", "/etc/init.d/gnunetd");
+                                       "unlink", initscript);
+          GNUNET_free (initscript);
           return GNUNET_SYSERR;
         }
       errno = 0;
-      if (-1 != system ("/usr/sbin/update-rc.d gnunetd remove"))
+      if (ACCESS ("/usr/sbin/update-rc.d", W_OK) == 0)
         {
-          GNUNET_GE_LOG_STRERROR_FILE (ectx,
-                                       GNUNET_GE_WARNING | GNUNET_GE_USER |
-                                       GNUNET_GE_ADMIN | GNUNET_GE_IMMEDIATE,
-                                       "system", "/usr/sbin/update-rc.d");
-          return GNUNET_SYSERR;
+          ret = system ("/usr/sbin/update-rc.d gnunetd remove");
+          if (ret != 0)
+            {
+              GNUNET_GE_LOG_STRERROR_FILE (ectx,
+                                           GNUNET_GE_WARNING | GNUNET_GE_USER
+                                           | GNUNET_GE_ADMIN |
+                                           GNUNET_GE_IMMEDIATE, "system",
+                                           "/usr/sbin/update-rc.d");
+              GNUNET_free (initscript);
+              return GNUNET_SYSERR;
+            }
         }
+      else if (ACCESS ("/sbin/rc-update", W_OK) == 0)
+        {
+          ret = system ("/sbin/rc-update del gnunetd");
+          if (ret != 0)
+            {
+              GNUNET_GE_LOG_STRERROR_FILE (ectx,
+                                           GNUNET_GE_WARNING | GNUNET_GE_USER
+                                           | GNUNET_GE_ADMIN |
+                                           GNUNET_GE_IMMEDIATE, "system",
+                                           "/sbin/rc-update");
+              GNUNET_free (initscript);
+              return GNUNET_SYSERR;
+            }
+        }
+      GNUNET_free (initscript);
       return GNUNET_YES;
     }
+  GNUNET_free (initscript);
 #endif
   return GNUNET_SYSERR;
 }
