@@ -24,9 +24,9 @@
  * @author Christian Grothoff
  */
 
+#include "platform.h"
 #include "gnunet_util_os.h"
 #include "gnunet_util_string.h"
-#include "platform.h"
 
 int
 GNUNET_configure_user_account (int testCapability,
@@ -87,6 +87,7 @@ GNUNET_configure_user_account (int testCapability,
       /* Debian */
       /* TODO: FreeBSD? http://www.freebsd.org/cgi/man.cgi?query=adduser&sektion=8 */
       char *cmd;
+      int ret;
 
       haveGroup = group_name && strlen (group_name) > 0;
       cmd =
@@ -96,7 +97,17 @@ GNUNET_configure_user_account (int testCapability,
       if (haveGroup)
         {
           sprintf (cmd, "/usr/sbin/addgroup --quiet --system %s", group_name);
-          system (cmd);
+          ret = system (cmd);
+	  if (ret == -1)
+	    GNUNET_GE_LOG_STRERROR(NULL,
+				   GNUNET_GE_ERROR | GNUNET_GE_BULK | GNUNET_GE_ADMIN,
+				   "system");
+	  else if (WEXITSTATUS(ret) != 0)
+	    GNUNET_GE_LOG(NULL,
+			  GNUNET_GE_ERROR | GNUNET_GE_BULK | GNUNET_GE_ADMIN,
+			  _("`%s' returned with error code %u"),
+			  "addgroup",
+			  WEXITSTATUS(ret));	    
         }
 
       sprintf (cmd,
@@ -104,7 +115,17 @@ GNUNET_configure_user_account (int testCapability,
                "--no-create-home %s",
                haveGroup ? "--ingroup" : "",
                haveGroup ? group_name : "", user_name);
-      system (cmd);
+      ret = system (cmd);
+      if (ret == -1)
+	GNUNET_GE_LOG_STRERROR(NULL,
+			       GNUNET_GE_ERROR | GNUNET_GE_BULK | GNUNET_GE_ADMIN,
+			       "system");     
+      else if (WEXITSTATUS(ret) != 0)
+	GNUNET_GE_LOG(NULL,
+		      GNUNET_GE_ERROR | GNUNET_GE_BULK | GNUNET_GE_ADMIN,
+		      _("`%s' returned with error code %u"),
+		      "adduser",
+		      WEXITSTATUS(ret));
       GNUNET_free (cmd);
       return GNUNET_OK;
     }
